// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package hello;

//
// LoadBalancing::LoadAlert servant class.
//
class LoadAlert_impl extends com.ooc.LoadBalancing.LoadAlertPOA
{
    private org.omg.PortableServer.POA poa_;
    private com.ooc.LoadBalancing.Util.LoadAlert alert_;

    public LoadAlert_impl(org.omg.PortableServer.POA poa,
			  com.ooc.LoadBalancing.Util.LoadAlert alert)
    {
	poa_ = poa;
	alert_ = alert;
    }

    public org.omg.PortableServer.POA _default_POA()
    {
	return poa_;
    }

    public void alert()
    {
	alert_.alert();
    }
}

public class Server
{
    static void
    usage(String progName)
    {
	System.err.println("Usage: " + progName + " MEMBER_ID\n");
    }

    static int
    run(org.omg.CORBA.ORB orb, String args[])
	throws org.omg.CORBA.UserException
    {
	//
	// Check arguments
	//
	if(args.length != 1)
	{
	    usage("hello.Server");
	    System.exit(1);
	}
	String memberId = args[0];

	//
	// Resolve Root POA
	//
	org.omg.PortableServer.POA rootPOA =
	    org.omg.PortableServer.POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));

	//
	// Get a reference to the POA manager
	//
	org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

	//
	// Create the member policy.
	//
	com.ooc.LoadBalancing.MemberPolicyValue value =
	    new com.ooc.LoadBalancing.MemberPolicyValue();
	value.group_id = "Hello";
	value.member_id = memberId;
	org.omg.CORBA.Any any = orb.create_any();
	com.ooc.LoadBalancing.MemberPolicyValueHelper.insert(any, value);
	org.omg.CORBA.Policy memberPolicy = orb.create_policy(
	    com.ooc.LoadBalancing.MEMBER_POLICY_ID.value, any);

	//
	// Create a load balanced POA for the hello object.
	//
        org.omg.CORBA.Policy[] pl = new org.omg.CORBA.Policy[3];
        pl[0] = rootPOA.create_lifespan_policy(
            org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
        pl[1] = rootPOA.create_id_assignment_policy(
            org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
	pl[2] = memberPolicy;
	org.omg.PortableServer.POA helloPOA =
            rootPOA.create_POA("helloPOA", manager, pl);

	//
	// Create implementation object
	//
	Hello_impl helloImpl = new Hello_impl(helloPOA, memberId);
	byte[] oid = ("Hello").getBytes();
	helloPOA.activate_object_with_id(oid, helloImpl);
	org.omg.CORBA.Object hello = helloImpl._this(orb);

	//
	// Get the GroupFactory, LoadUpdater and LoadAlert objects.
	//
	com.ooc.LoadBalancing.GroupFactory factory = null;
	com.ooc.LoadBalancing.Util.LoadUpdater updater = null;
	com.ooc.LoadBalancing.Util.LoadAlert alert = null;
	try
	{
	    factory =
		com.ooc.LoadBalancing.GroupFactoryHelper.narrow(
		    orb.resolve_initial_references("Balancer"));
	    updater =
		(com.ooc.LoadBalancing.Util.LoadUpdater)
		    orb.resolve_initial_references("LoadUpdater");
	    alert =
		(com.ooc.LoadBalancing.Util.LoadAlert)
		    orb.resolve_initial_references("LoadAlert");
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	}
	com.ooc.OB.Assert._OB_assert(factory != null);
	com.ooc.OB.Assert._OB_assert(updater != null);
	com.ooc.OB.Assert._OB_assert(alert != null);

	//
	// Create LoadAlert implementation object.
	//
	LoadAlert_impl loadAlertImpl = new LoadAlert_impl(rootPOA, alert);
	com.ooc.LoadBalancing.LoadAlert loadAlert = loadAlertImpl._this(orb);

	//
	// Activate the POA manager.
	//
	manager.activate();

	//
	// Set the LoadAlert for the member.
	//
	try
	{
	    com.ooc.LoadBalancing.Group group = factory.get("Hello");
	    group.set_load_alert(memberId, loadAlert);
	}
	catch(com.ooc.LoadBalancing.GroupNotFound ex)
	{
	    // should be caught during POA creation
	    com.ooc.OB.Assert._OB_assert(false);
	}
	catch(com.ooc.LoadBalancing.MemberNotFound ex)
	{
	    // should be caught during POA creation
	    com.ooc.OB.Assert._OB_assert(false);
	}

	//
	// Register the group with the LoadUpdater.
	//
	updater.register_member(memberId, "Hello");

	//
	// Save reference
	//
	try
	{
	    String ref = orb.object_to_string(hello);
	    String refFile = "Hello.ref";
	    java.io.FileOutputStream file =
		new java.io.FileOutputStream(refFile);
	    java.io.PrintWriter out = new java.io.PrintWriter(file);
	    out.println(ref);
	    out.flush();
	    file.close();
	}
	catch(java.io.IOException ex)
	{
	    System.err.println("hello.Server: can't write to `" +
                               ex.getMessage() + "'");
	    return 1;
	}

	//
	// Start event loop.
	//
	orb.run();

	return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	org.omg.CORBA.ORB orb = null;

	try
	{
	    //
	    // Initialize this member.
	    //
	    com.ooc.LoadBalancing.LB.init(props);
	    com.ooc.LoadBalancing.Util.LBUtil.init(props);

	    //
	    // Initialize the ORB
	    //
	    org.omg.CORBA.StringSeqHolder argsH =
		new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);

	    //
	    // Start the server
	    //
	    status = run(orb, argsH.value);
	}
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}

	if(orb != null)
	{
	    try
	    {
		orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
