// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

//
// TransportManager mediates connect and accept activities
//
public class TransportManager implements TransportObserver
{
    //
    // The protocol id
    //
    private String id_;

    //
    // The profile tag
    //
    private int tag_;

    //
    // The local peer id
    //
    private String localPeer_;

    //
    // A map of remote peer id to ActiveTransport
    //
    private java.util.Hashtable transportMap_ = new java.util.Hashtable(11);
    private java.lang.Object transportMapMonitor_ = new java.lang.Object();

    //
    // The callback acceptor
    //
    private CallbackAcceptor_impl callbackAcceptor_;
    private java.lang.Object callbackAcceptorMonitor_ = new java.lang.Object();

    //
    // Reference counted wrapper for ActiveTransport
    //
    class ActiveTransportWrapper {
	
        private ActiveTransport at_ = null;
	
        private int atRefcount_;
	
        ActiveTransportWrapper(ActiveTransport activeTransport)
	{
	    at_ = activeTransport;
            atRefcount_ = 1;
	}
	
        //
        // Get the current reference count
        //
        int getRefcount()
	{
	    return atRefcount_;
	}
	
        //
        // Increment reference count
        //
        void incrRefcount()
	{
	    atRefcount_++;
	}

        //
        // Decrement reference count
        //
        void decrRefcount()
	{
	    atRefcount_--;
	}

        //
        // Return reference to ActiveTransport
        //
        ActiveTransport getAt()
	{
	    return at_;
	}
    }

    // ------------------------------------------------------------------
    // Private member implementations
    // ------------------------------------------------------------------

    // ------------------------------------------------------------------
    // Constructor
    // ------------------------------------------------------------------

    public
    TransportManager(String id, int tag, String localPeer)
    {
        id_ = id;
        tag_ = tag;
        localPeer_ = localPeer;
        callbackAcceptor_ = null;
    }

    // ------------------------------------------------------------------
    // Public member implementations
    // ------------------------------------------------------------------

    public String
    id()
    {
        return id_;
    }

    public int
    tag()
    {
        return tag_;
    }

    public String
    peer()
    {
        return localPeer_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    //
    // A new connection has been accepted
    //
    com.ooc.OCI.Transport
    addIncomingTransport(com.ooc.OCI.Acceptor acceptor,
                         com.ooc.OCI.Transport delegate)
    {
        ActiveTransport at = new ActiveTransport(localPeer_, this, delegate);

        //
        // For newly created transports, we must first read
        // the remote peer's id. Then we register the
        // transport in our table, so that it is available
        // for callback connections to the same remote peer.
        //
        try
        {
            at.receiveRemotePeerId();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            at.close(false);
            throw ex;
        }

        synchronized(transportMapMonitor_)
        {
            ActiveTransportWrapper atWrapper = new ActiveTransportWrapper(at);
	    
            if( transportMap_.containsKey( at.remotePeer()) )
	    {
                atWrapper.incrRefcount();
            }
	    
            transportMap_.put(at.remotePeer(), atWrapper);
            transportMapMonitor_.notifyAll();
        }
	
        return new Transport_impl(id_, tag_, at, acceptor);
    }

    com.ooc.OCI.Transport
    connect(String remotePeer, com.ooc.OCI.Connector connector,
            com.ooc.OCI.Connector delegate)
    {
        ActiveTransport at = null;
        boolean newTransport = false;

        //
        // Check if there is an existing incoming connection that
        // we can use
        //
        synchronized(transportMapMonitor_)
        {
            ActiveTransportWrapper atWrapper = 
		(ActiveTransportWrapper)transportMap_.get(remotePeer);
            if( atWrapper != null )
                at = atWrapper.getAt();
        }
	
        //
        // If there is no existing incoming connection, and we have
        // a connector delegate, then create a new ActiveTransport
        //
        if(at == null && delegate != null)
        {
            com.ooc.OCI.Transport t = delegate.connect();
            com.ooc.OB.Assert._OB_assert(t != null);
            at = new ActiveTransport(localPeer_, remotePeer, this, t);
            newTransport = true;
        }

        com.ooc.OCI.Transport result = null;

        if(at != null)
        {
            if(newTransport)
            {
                //
                // For newly created connections, we must first send our
                // local peer id so that the remote end can make a
                // callback connection.
                //
                at.sendLocalPeerId();
                result = new Transport_impl(id_, tag_, at, connector);
            }
            else
                result = new CallbackTransport_impl(id_, tag_, at, connector);
        }

        return result;
    }

    com.ooc.OCI.Transport
    connectTimeout(String remotePeer, com.ooc.OCI.Connector connector,
                   com.ooc.OCI.Connector delegate, int timeout)
    {
        ActiveTransportWrapper atWrapper = null;
        ActiveTransport at = null;
        boolean newTransport = false;

        synchronized(transportMapMonitor_)
        {
            //
            // Check if there is an existing incoming connection that
            // we can use
            //
            atWrapper = (ActiveTransportWrapper)transportMap_.get(remotePeer);
            if( atWrapper != null)
                at = atWrapper.getAt();
        }

        if(at == null)
        {
            //
            // If there is no existing incoming connection, and we have
            // a connector delegate, then create a new ActiveTransport
            //
            if(delegate != null)
            {
                com.ooc.OCI.Transport t = delegate.connect_timeout(timeout);
                if(t != null)
                {
                    at = new ActiveTransport(localPeer_, remotePeer, this, t);
                    newTransport = true;
                }
            }
            else
            {
                //
                // Wait for an incoming connection until timeout
                //
                synchronized(transportMapMonitor_)
                {
                    long start = System.currentTimeMillis();
                    long elapsed = 0;
                    while(at == null && elapsed < (long)timeout)
                    {
                        try
                        {
                            transportMapMonitor_.wait(timeout - elapsed);
                        }
                        catch(InterruptedException ex)
                        {
                        }

                        atWrapper = (ActiveTransportWrapper)
			    transportMap_.get(remotePeer);
                        if( atWrapper != null)
                            at = atWrapper.getAt();
                        elapsed = System.currentTimeMillis() - start;
                    }
                }
            }
        }

        com.ooc.OCI.Transport result = null;

        if(at != null)
        {
            if(newTransport)
            {
                //
                // For newly created connections, we must first send our
                // local peer id so that the remote end can make a
                // callback connection.
                //
                at.sendLocalPeerId();
                result = new Transport_impl(id_, tag_, at, connector);
            }
            else
                result = new CallbackTransport_impl(id_, tag_, at, connector);
        }

        return result;
    }

    void
    setCallbackAcceptor(CallbackAcceptor_impl acceptor)
    {
        synchronized(callbackAcceptorMonitor_)
        {
            com.ooc.OB.Assert._OB_assert((callbackAcceptor_ == null && 
                                      acceptor != null) ||
                                     (callbackAcceptor_ != null &&
                                      acceptor == null));
            callbackAcceptor_ = acceptor;
        }
    }

    boolean
    hasCallbackAcceptor()
    {
        synchronized(callbackAcceptorMonitor_)
        {
            return (callbackAcceptor_ != null);
        }
    }


    // ------------------------------------------------------------------
    // TransportObserver member implementations
    // ------------------------------------------------------------------

    //
    // ActiveTransport received an initial message on the callback
    // connection
    //
    public void
    callbackConnection(ActiveTransport t)
    {
        //
        // If the transport is outgoing, then we need to notify the
        // callback dispatcher about a new callback connection
        //
        if(t.type() == ActiveTransport.Outgoing)
        {
            synchronized(callbackAcceptorMonitor_)
            {
                if(callbackAcceptor_ != null)
                {
                    CallbackTransport_impl transport =
                        new CallbackTransport_impl(id_, tag_, t,
                                                   callbackAcceptor_);
                    callbackAcceptor_._OB_connect(transport);
                }
                else
                {
                    //
                    // TODO: What if there is no acceptor? The remote end
                    // should really receive a TRANSIENT exception.
                    //
                }
            }
        }
    }

    //
    // The forward connection of the ActiveTransport has been shutdown
    //
    public void
    shutdown(ActiveTransport t)
    {
        if(t.type() == ActiveTransport.Incoming)
        {
            //
            // Remove the incoming transport from the transport map
            //
            String remotePeer = t.remotePeer();
	    
            if(remotePeer != null)
            {
                synchronized(transportMapMonitor_)
                {
                    ActiveTransportWrapper atWrapper = 
			(ActiveTransportWrapper)transportMap_.get(remotePeer);
		    
                    if(atWrapper != null) 
		    {
			atWrapper.decrRefcount(); 
			
			if(atWrapper.getRefcount() == 0) 
			{
			    transportMap_.remove(remotePeer);
			}
			else
			{
			    transportMap_.put(remotePeer, atWrapper);
			}
		    }
                }
            }
        }
    }
}
