// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosEventChannelAdmin.*;

public class PullClient
{
    static int
    run(ORB orb, String args[])
        throws UserException
    {
        //
        // Parse command line arguments.
        //
        String channelName = null;
        for(int i = 0 ; i < args.length ; i++)
        {
            if(args[i].equals("--help") || args[i].equals("-h"))
            {
                usage();
                return 0;
            }
            else if(!args[i].startsWith("-"))
            {
                channelName = args[i];
            }
            else
            {
                System.err.println("Unknown option `" + args[i] + "'");
                usage();
                return 1;
            }
        }
    
        //
        // Get event channel
        //
        EventChannel e = GetEventChannel(orb, channelName);
	    
        //
        // Get ProxyPullSupplier.
        //
        ConsumerAdmin consumerAdmin = e.for_consumers();
        ProxyPullSupplier supplier = consumerAdmin.obtain_pull_supplier();
	    
        //
        // Since callbacks are not needed a PushSupplier_impl is
        // not created.
        //
        supplier.connect_pull_consumer(null);
    
        //
        // Main loop
        //
        while(true)
        {
    	    Any any = null;
    	
    	    try
    	    {
    	        any = supplier.pull();
    	    }
    	    catch(org.omg.CosEventComm.Disconnected ex)
    	    {
	        break;
	    }
	    catch(SystemException ex)
	    {
	        ex.printStackTrace();
	        break;
	    }
	    
	    try
	    {
	        String s = any.extract_string();
	        System.out.println(s);
	    }
	    catch(BAD_OPERATION ex)
	    {
	    }
        }
    
        try
        {
    	    supplier.disconnect_pull_supplier();
        }
        catch(org.omg.CORBA.TRANSIENT ex)
        {
            // Expected if event service has gone away
        }

	return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        ORB orb = null;

        try
        {
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }

    private static void
    usage()
    {
        System.out.println(
            "Usage: java event.PushClient [options] [channel-name]\n\n" +
	    "Options:\n" +
	    "-h, --help             Show this message.\n");
    }

    static private EventChannel
    GetEventChannel(org.omg.CORBA.ORB orb, String name)
    {
	EventChannel e = null;
	if(name == null)
	{
	    //
	    // Get event channel from the initial reference.
	    //
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("EventService");
		e = EventChannelHelper.narrow(obj);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("Can't resolve `EventService'");
		System.exit(1);
	    }
            catch(org.omg.CORBA.BAD_PARAM ex)
            {
                // narrow() failed
            }
	    
	    if(e == null)
	    {
		System.err.println("`EventService' is not an EventChannel " +
				   " object reference");
		System.exit(1);
	    }
	}
	else
	{
	    //
	    // Get event channel factory. Note that this uses ORBacus
	    // proprietary methods.
	    //
	    com.ooc.OBEventChannelFactory.EventChannelFactory f = null;
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("EventChannelFactory");
		f = com.ooc.OBEventChannelFactory.EventChannelFactoryHelper.
		    narrow(obj);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("Can't resolve `EventChannelFactory'");
		System.exit(1);
	    }
            catch(org.omg.CORBA.BAD_PARAM ex)
            {
                // narrow() failed
            }
	
	    if(f == null)
	    {
		System.err.println("`EventChannelFactory' is not an " +
				   "EventChannelFactory object reference");
		System.exit(1);
	    }

	    //
	    // Get event channel.
	    //
	    try
	    {
		e = f.get_channel_by_id(name);
	    }
	    catch(com.ooc.OBEventChannelFactory.ChannelNotAvailable ex)
	    {
		try
		{
		    e = f.create_channel(name);
		    System.err.println("`Channel " + name + "' created");
		}
		catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex1)
		{
		    System.err.println("Channel exists: try again");
		    System.exit(1);
		}
	    }
	}
	return e;
    }
}
