// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosTypedEventChannelAdmin.*;
import org.omg.CosEventChannelAdmin.*;

public class TypedPushServer
{
    static int
    run(ORB orb, String args[])
        throws UserException
    {
        //
        // Parse command line arguments.
        //
        String channelName = null;
        for(int i = 0 ; i < args.length ; i++)
        {
	    if(args[i].equals("--help") || args[i].equals("-h"))
	    {
	        usage();
                return 0;
	    }
	    else if(!args[i].startsWith("-"))
	    {
	        channelName = args[i];
	    }
	    else
	    {
	        System.err.println("Unknown option `" + args[i] + "'");
	        usage();
	        return 1;
	    }
        }

        //
        // Get event channel.
        //
        TypedEventChannel e = GetTypedEventChannel(orb, channelName);
	    
        //
        // Get TypedProxyPushConsumer.
        //
        TypedSupplierAdmin supplierAdmin = e.for_suppliers();
        TypedProxyPushConsumer consumer =
            supplierAdmin.obtain_typed_push_consumer(
                "IDL:event/TimerUpdate:1.0");

        //
        // Since callbacks are not needed a PushSupplier_impl is
        // not created.
        //
        consumer.connect_push_supplier(null);

        org.omg.CORBA.Object typedConsumer = consumer.get_typed_consumer();

        TimerUpdate timer = null;
        try
        {
            timer = TimerUpdateHelper.narrow(typedConsumer);
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
	    System.out.println("narrow failed!");
	    return 1;
        }

        while(true)
        {
	    java.util.Date date = new java.util.Date();
	    String s = "PushSupplier says: " + date.toString();
	    try
	    {
	        timer.time_update(s);
	    }
	    catch(org.omg.CORBA.SystemException ex)
	    {
	        ex.printStackTrace();
	        break;
	    }

	    try
	    {
	        Thread.sleep(1000);
	    }
	    catch(InterruptedException ex)
	    {
	    }
        }
        try
        {
	    consumer.disconnect_push_consumer();
        }
        catch(org.omg.CORBA.TRANSIENT ex)
        {
            // Expected if event service has gone away
        }

	return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        ORB orb = null;

        try
        {
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }

    private static void
    usage()
    {
        System.out.println(
            "Usage: java event.TypedPushServer [options] [channel-name]\n\n" +
	    "Options:\n" +
	    "-h, --help             Show this message.\n");
    }

    static private TypedEventChannel
    GetTypedEventChannel(org.omg.CORBA.ORB orb, String name)
    {
	TypedEventChannel e = null;
	if(name == null)
	{
	    //
	    // Get event channel from the initial reference.
	    //
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("TypedEventService");
		e = TypedEventChannelHelper.narrow(obj);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("Can't resolve `TypedEventService'");
		System.exit(1);
	    }
            catch(org.omg.CORBA.BAD_PARAM ex)
            {
                // narrow() failed
            }
	    
	    if(e == null)
	    {
		System.err.println("`TypedEventService' is not a " +
				   "TypedEventChannel object reference");
		System.exit(1);
	    }
	}
	else
	{
	    //
	    // Get event channel factory. Note that this uses ORBacus
	    // proprietary methods.
	    //
	    com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory f
		= null;
	    try
	    {
		org.omg.CORBA.Object obj =
		    orb.resolve_initial_references("TypedEventChannelFactory");
		f = com.ooc.OBTypedEventChannelFactory.
		    TypedEventChannelFactoryHelper. narrow(obj);
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("Can't resolve `TypedEventChannelFactory'");
		System.exit(1);
	    }
            catch(org.omg.CORBA.BAD_PARAM ex)
            {
                // narrow() failed
            }
	
	    if(f == null)
	    {
		System.err.println("`TypedEventChannelFactory' is not a " +
				   "TypedEventChannelFactory object " +
				   "reference");
		System.exit(1);
	    }

	    //
	    // Get event channel.
	    //
	    try
	    {
		e = f.get_channel_by_id(name);
	    }
	    catch(com.ooc.OBEventChannelFactory.ChannelNotAvailable ex)
	    {
		try
		{
		    e = f.create_channel(name);
		    System.err.println("`Channel " + name + "' created");
		}
		catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex1)
		{
		    System.err.println("Channel exists: try again");
		    System.exit(1);
		}
	    }
	}
	return e;
    }
}
