// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
package com.ooc.CosEvent;

import java.util.Vector;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;

public class AdminBaseImpl
{
    //
    // The ORB
    //
    private ORB orb_;

    //
    // The POA
    //
    private POA poa_;

    //
    // The message logger
    //
    private EventLogger logger_;
    
    //
    // My pipe segment
    //
    private PipeSegment myPipe_;

    //
    // The admin
    //
    private AdminBase admin_;

    //
    // Set of ProxyConsumers
    //
    private Vector proxies_ = new Vector();

    //
    // ID of the next assigned proxy
    //
    private long nextId_;

    //
    // Has this admin been destroyed?
    //
    private boolean destroyed_;

    //
    // The reaper
    //
    private Reaper reaper_;

    // ----------------------------------------------------------------------
    // AdminBaseImpl private member implementation
    // ----------------------------------------------------------------------

    //
    // Reap proxies
    //
    private void reap()
    {
	//
	// Find the element, remove it from the sequence.
	//
	int i = 0;
	while(i < proxies_.size())
	{
	    ProxyBase proxy = (ProxyBase)proxies_.elementAt(i);
	    if(proxy.destroyed())
	    {
		proxies_.removeElementAt(i);
		continue;
	    }
	    ++i;
	}
    }

    // ----------------------------------------------------------------------
    // AdminBaseImpl constructor
    // ----------------------------------------------------------------------

    public AdminBaseImpl(ORB orb, POA poa, EventLogger logger, Reaper reaper,
			 PipeSegment pipe, AdminBase admin)
    {
	orb_ = orb;
	poa_ = poa;
	logger_ = logger;
	reaper_ = reaper;
	myPipe_ = pipe;
	nextId_ = 0;
	destroyed_ = false;
	admin_ = admin;
    }

    protected void finialize()
	throws Throwable
    {
	com.ooc.OB.Assert._OB_assert(proxies_.isEmpty());
    }

    // ----------------------------------------------------------------------
    // AdminBaseImpl public member implementation
    // ----------------------------------------------------------------------

    //
    // Add a proxy
    //
    public void addProxy(ProxyBase proxy)
    {
	reap();
	proxies_.addElement(proxy);
	if(reaper_ != null)
	    reaper_.add(proxy);
    }

    //
    // Fill constructor info
    //
    public void fillProxyConstructorInfo(ProxyConstructorInfo info)
    {
	info.orb = orb_;
	info.poa = poa_;
	info.logger = logger_;
	info.pipe = myPipe_;
	info.id = nextId_++;
    }

    //
    // Called from the event channel upon destruction.
    //
    public void disconnect()
    {
	com.ooc.OB.Assert._OB_assert(!destroyed_);
	destroyed_ = true;

	//
	// disconnect each proxy.
	//
	for(int i = 0 ; i < proxies_.size() ; ++i)
	    ((ProxyBase)proxies_.elementAt(i)).disconnect();
	proxies_.removeAllElements();

	//
	// Disconnect the servant from the ORB.
	//
	admin_.deactivateObject();
    }

    //
    // Stop accepting new requests.
    //
    public void discardRequests()
    {
	if(!destroyed_)
	{
	    for(int i = 0 ; i < proxies_.size() ; ++i)
		((ProxyBase)proxies_.elementAt(i)).discardRequests();
	}
    }

    //
    // Get the ORB
    //
    public ORB getOrb()
    {
	return orb_;
    }

    //
    // Get the POA
    //
    public POA getPoa()
    {
	return poa_;
    }

    //
    // Get the logger
    //
    public EventLogger getLogger()
    {
	return logger_;
    }

    //
    // Has this admin been destroyed?
    //
    public boolean destroyed()
    {
	return destroyed_;
    }
}
