// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// *********************************************************************

package com.ooc.CosEvent;

import org.omg.CORBA.*;

//
// Implementation for ProxyBase
//
public class ProxyBaseImpl
{
    //
    // My ORB
    //
    private ORB orb_;

    //
    // The POA
    //
    private org.omg.PortableServer.POA poa_;

    //
    // The message logger
    //
    private EventLogger logger_;
    
    //
    // My parent pipe segment
    //
    private PipeSegment parentPipe_;

    //
    // My pipe segment (nil for proxy consumers)
    //
    private PipeSegment myPipe_;

    //
    // My proxy
    //
    private ProxyBase proxy_;

    //
    // My id
    //
    private long id_;

    //
    // My name
    //
    private String name_;

    //
    // The last time stampRequest() was called
    //
    private long lastRequestTime_;

    //
    // The proxy state
    //
    private ProxyState state_;

    // ----------------------------------------------------------------------
    // ProxyBaseImpl constructor/finalize
    // ----------------------------------------------------------------------

    public ProxyBaseImpl(ProxyConstructorInfo info, ProxyBase proxy)
    {
	proxy_ = proxy;
	lastRequestTime_ = System.currentTimeMillis();
	state_ = ProxyState.Disconnected;
	orb_ = info.orb;
	poa_ = info.poa;
	logger_ = info.logger;
	id_ = info.id;
	parentPipe_ = info.pipe;

	name_ = info.base + "-" + id_;

	if(logger_.traceLifecycle() >= 2)
	{
	    String desc = null;
	    try
	    {
		desc = EventUtil.getConnectionAddrDescription(orb_);
	    }
	    catch(NO_RESOURCES ex)
	    {
		desc = "`connection address unavailable'";
	    }
	    logger_.trace("event", "Create " + name_ + " from " + desc);
	}
    }

    protected void finalize()
        throws Throwable
    {
	if(logger_.traceLifecycle() >= 2)
	    logger_.trace("event", "Destroy " + name_);
    }

    // ----------------------------------------------------------------------
    // ProxyBaseImpl public member implementation
    // ----------------------------------------------------------------------

    //
    // Resolve the QoS parameters
    //
    public void resolveQoSParams(QoSParameters params)
    {
	try
	{
	    com.ooc.CORBA.ORB oborb = (com.ooc.CORBA.ORB) orb_;
	    java.util.Properties properties = oborb.properties();
	    params.retryTimeout =
		Long.parseLong(properties.getProperty(EventQoS.RETRY_TIMEOUT));
	    params.retryMultiplier =
		Double.valueOf(properties.getProperty(
		    EventQoS.RETRY_MULTIPLIER)).doubleValue();
	    params.maxRetries =
		Long.parseLong(properties.getProperty(EventQoS.MAX_RETRIES));
	    params.maxEvents =
		Long.parseLong(properties.getProperty(EventQoS.MAX_EVENTS));
	    params.pullInterval =
		Long.parseLong(properties.getProperty(EventQoS.PULL_INTERVAL));
	    params.requestTimeout =
		Long.parseLong(properties.getProperty(
		    EventQoS.REQUEST_TIMEOUT));
	}
	catch(NumberFormatException e)
	{
	    com.ooc.OB.Assert._OB_assert(false);
	}
    }

    //
    // Apply the request timeout to the given object reference and
    // return a new object reference.
    //
    public org.omg.CORBA.Object 
    applyRequestTimeout(org.omg.CORBA.Object obj, QoSParameters params)
    {
        org.omg.CORBA.Policy[] pl = new org.omg.CORBA.Policy[1];
        org.omg.CORBA.Any any = orb_.create_any();
        any.insert_ulong((int)params.requestTimeout);
        try
        {
            pl[0] = orb_.create_policy(com.ooc.OB.TIMEOUT_POLICY_ID.value,
				       any);
	    return obj._set_policy_override(pl, SetOverrideType.ADD_OVERRIDE);
        }
        catch(org.omg.CORBA.PolicyError e)
        {
	    com.ooc.OB.Assert._OB_assert(false);
	    return obj;
	}
	catch(org.omg.CORBA.BAD_OPERATION e)
	{
	    //
	    // Expected if the object is collocated. If this is the
	    // case, no need to set any timeouts.
	    // 
	    return obj;
	}
    }

    //
    // Disconnect the proxy
    //
    public void disconnect()
    {
	if(state_ == ProxyState.Destroyed)
	    return;

	state_ = ProxyState.Destroyed;

	//
	// Destroy the supplier event pipe
	//
	if(myPipe_ != null)
	    myPipe_.destroy();

	proxy_.disconnectPeer();
	proxy_.deactivateObject();
    }

    //
    // Don't accept further requests
    //
    public void discardRequests()
    {
	if(state_ == ProxyState.Destroyed || state_ == ProxyState.Discarding)
	    return;

	state_ = ProxyState.Discarding;

	//
	// Destroy the supplier event pipe
	//
	if(myPipe_ != null)
	    myPipe_.destroy();
    }

    //
    // Has this proxy been destroyed?
    //
    public boolean destroyed()
    {
	return state_ == ProxyState.Destroyed;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return lastRequestTime_;
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	lastRequestTime_ = System.currentTimeMillis();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return name_;
    }

    //
    // Get the ORB
    //
    public ORB getOrb()
    {
	return orb_;
    }

    //
    // Get the POA
    //
    public org.omg.PortableServer.POA getPoa()
    {
	return poa_;
    }

    //
    // Get the logger
    //
    public EventLogger getLogger()
    {
	return logger_;
    }

    //
    // Get the parent pipe segment
    //
    public PipeSegment getParentPipe()
    {
	return parentPipe_;
    }

    //
    // Get my pipe segment (nil for proxy consumers)
    //
    public PipeSegment getMyPipe()
    {
	return myPipe_;
    }

    //
    // Set my pipe segment
    //
    public void setMyPipe(PipeSegment pipe)
    {
	myPipe_ = pipe;
    }

    //
    // Get the id
    //
    public long getId()
    {
	return id_;
    }

    //
    // Get the proxy state
    //
    public ProxyState getState()
    {
	return state_;
    }

    //
    // Set the proxy state
    //
    public void setState(ProxyState state)
    {
	state_ = state;
    }
}
