// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEvent;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import org.omg.PortableServer.POAManagerPackage.*;
import org.omg.CosEventChannelAdmin.*;
import java.util.*;

final public class Server
{
    static void
    usage(String progName)
    {
	System.err.println("Usage:");
	System.err.println(progName + " [options]\n" +
			   "\n" +
			   "Options:\n" +
			   "-h, --help             " +
			   "Show this message.\n" +
			   "-v, --version          " +
			   "Show Event Service version.\n" +
			   "-i, --ior              " +
			   "Print IOR on standard output.\n" +
			   "-t, --typed-service    " +
			   "Run typed event service.\n" +
			   "-u, --untyped-service  " +
			   "Run untyped event service [default].\n");
    }

    static int
    run(ORB orb, String args[], boolean typedService)
    {
        //
	// Set the program name
	//
	String progName = "com.ooc.CosEvent.Server";

	//
	// Resolve Root POA
	//
	org.omg.CORBA.Object poaObj = null;
	try
	{
		poaObj = orb.resolve_initial_references("RootPOA");
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
		throw new RuntimeException();
	}
	POA rootPOA = POAHelper.narrow(poaObj);

	//
	// Get a reference to the POA manager
	//
	POAManager manager = rootPOA.the_POAManager();

	//
	// Get options.
	//
	boolean ior = false;

	int i;
	for(i = 0 ; i < args.length && args[i].startsWith("-") ; i++)
	{
	    if(args[i].equals("--help") || args[i].equals("-h"))
	    {
	        usage(progName);
	        return 0;
	    }
	    else if(args[i].equals("--version") || args[i].equals("-v"))
	    {
                System.err.println("ORBacus Event Service " +
                                   com.ooc.OB.Version.version);
	        return 0;
	    }
	    else if(args[i].equals("--ior") || args[i].equals("-i"))
	    {
	        ior = true;
	    }
	    else if(args[i].equals("--typed-service") || args[i].equals("-t"))
	    {
	        typedService = true;
	    }
	    else if(args[i].equals("--untyped-service") || args[i].equals("-u"))
	    {
	        typedService = false;
	    }
	    else
	    {
	        System.out.println(progName + ": unknown option `" +
				       args[i] + "'\n");
	        usage(progName);
	        return 1;
	    }
	}

	//
	// Create POA for use by factory
	//
	POA poa = null;
	try
	{
	    Policy[] policies = new Policy[2];
	    policies[0] = rootPOA.create_id_assignment_policy(
		org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
	    policies[1] = rootPOA.create_lifespan_policy(
		org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);

	    poa = rootPOA.create_POA("EventServicePOA", manager, policies);
	}
	catch(InvalidPolicy ex)
	{
	    throw new RuntimeException();
	}
	catch(AdapterAlreadyExists ex)
	{
	    throw new RuntimeException();
	}

        //
        // Resolve the Boot Manager
        //
        com.ooc.OB.BootManager bootManager = null;
        try
        {
	    bootManager = com.ooc.OB.BootManagerHelper.narrow(
                orb.resolve_initial_references("BootManager"));
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            throw new RuntimeException();
        }

	String s;
	EventService untyped = null;
	TypedEventService typed = null;

	if(!typedService)
	{
	    //
	    // Initialize the event service
	    //
	    untyped = new EventService(orb, poa);
	    try
	    {
		untyped.initialize();
	    }
	    catch(WrongPolicy ex)
	    {
		throw new RuntimeException();
	    }

	    com.ooc.OBEventChannelFactory.EventChannelFactory
		untypedFactory = untyped.eventChannelFactory();

	    //
	    // Activate the POAManager
	    //
	    try
	    {
		manager.activate();
	    }
	    catch(AdapterInactive ex)
	    {
		throw new RuntimeException();
	    }
	    
	    //
	    // Create the default event channel
	    //
	    org.omg.CosEventChannelAdmin.EventChannel untypedChannel = null;
	    try
	    {
	        untypedChannel =
		    untypedFactory.create_channel("DefaultEventChannel");
	    }
	    catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex)
	    {
	        throw new RuntimeException();
	    }

	    //
	    // Register the default event channel factory with the
	    // Boot Manager
	    //
	    byte[] oid = ("DefaultEventChannelFactory").getBytes();
	    try
	    {
	        bootManager.add_binding(oid, untypedFactory);
	    }
	    catch(com.ooc.OB.BootManagerPackage.AlreadyExists ex)
	    {
	        throw new RuntimeException();
	    }

	    //
	    // Register the default event channel with the Boot Manager
	    //
	    oid = ("DefaultEventChannel").getBytes();
	    try
	    {
	        bootManager.add_binding(oid, untypedChannel);
	    }
	    catch(com.ooc.OB.BootManagerPackage.AlreadyExists ex)
	    {
	        throw new RuntimeException();
	    }
    
	    s = orb.object_to_string(untypedChannel);
        }
	else
	{
	    try
	    {
	        org.omg.CORBA.Object ir =
	    	    orb.resolve_initial_references("InterfaceRepository");
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
	        System.err.println(
		    progName +
	    	    ": no `InterfaceRepository' initial reference.");
	        return 1;
	    }
    
	    //
	    // Initialize the typed event service
	    //
	    typed = new TypedEventService(orb, poa);
	    try
	    {
		typed.initialize();
	    }
	    catch(WrongPolicy ex)
	    {
		throw new RuntimeException();
	    }

	    com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory
		typedFactory = typed.typedEventChannelFactory();

	    //
	    // Activate the POAManager
	    //
	    try
	    {
		manager.activate();
	    }
	    catch(AdapterInactive ex)
	    {
		throw new RuntimeException();
	    }

	    //
	    // Create the default typed event channel
	    //
	    org.omg.CosTypedEventChannelAdmin.TypedEventChannel
		typedChannel = null;
	    try
	    {
	        typedChannel =
		    typedFactory.create_channel("DefaultTypedEventChannel");
	    }
	    catch(com.ooc.OBEventChannelFactory.ChannelAlreadyExists ex)
	    {
	        throw new RuntimeException();
	    }
    
	    //
	    // Register the default event channel factory with the
	    // Boot Manager
	    //
	    byte[] oid = ("DefaultTypedEventChannelFactory").getBytes();
	    try
	    {
	        bootManager.add_binding(oid, typedFactory);
	    }
	    catch(com.ooc.OB.BootManagerPackage.AlreadyExists ex)
	    {
	        throw new RuntimeException();
	    }

	    //
	    // Register the default typed channel with the Boot Manager
	    //
	    oid = ("DefaultTypedEventChannel").getBytes();
	    try
	    {
	        bootManager.add_binding(oid, typedChannel);
	    }
	    catch(com.ooc.OB.BootManagerPackage.AlreadyExists ex)
	    {
	        throw new RuntimeException();
	    }
    
    	    s = orb.object_to_string(typedChannel);
        }

	//
	// Print IOR on standard output.
	//
	if(ior)
	    System.out.println(s);
 
	//
	// Run implementation.
	//
	orb.run();

	return 0;
    }

    public static void
    main(String args[])
    {
        int status = 0;
        ORB orb = null;

        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        try
        {
	    //
	    // Setup properties
	    //
	    args = com.ooc.CORBA.ORB.ParseArgs(args, props, null);
	    
	    boolean typedService = false;

	    //
	    // Check properties first.
	    //
	    {
	        String propRoot = "ooc.event.";
		Enumeration keys = props.keys();
		while(keys.hasMoreElements())
		{
		    String key = (String)keys.nextElement();
		    if(!key.startsWith(propRoot))
		        continue;

	            String value = props.getProperty(key);
		    com.ooc.OB.Assert._OB_assert(value != null);

	            if (key.equals("ooc.event.endpoint"))
	            {
		        //
                        // Set the ooc.orb.oa.endpoint property from this
                        // value
                        //
		        String oaPort =
			    props.getProperty("ooc.orb.oa.endpoint");
		        if (oaPort == null)
		            props.put("ooc.orb.oa.endpoint", value);
	        
	            }
	            else if (key.equals("ooc.event.typed_service"))
		    {
		        //
		        // Check the ooc.event.typed_service
		        // property to determine if we're running the
		        // typed event service or the untyped.
		        //
		        if(value.equals("true"))
		            typedService = true;
		    }
		}
	    }

	    //
	    // We run the server in threaded/thread-pool by default.
	    //
	    props.put("ooc.orb.conc_model", "threaded");

	    //
	    // Set the concurrency model to thread_pool if necessary.
	    //
	    String oaModel = props.getProperty("ooc.orb.oa.conc_model");
	    if(!oaModel.equals("thread_per_client") &&
	       !oaModel.equals("thread_per_request") &&
	       !oaModel.equals("thread_pool"))
		props.put("ooc.orb.oa.conc_model", "thread_pool");

            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value, typedService);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                System.err.println(ex.getMessage());
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
