// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEvent;

import org.omg.CORBA.*;
import org.omg.CosEventChannelAdmin.*;
import java.util.*;

final public class SetDefaultProperties
{
    private static final class DefaultProperties
    {
	String key;
	String value;
	
	DefaultProperties(String k, String v)
	{
	    key = k;
	    value = v;
	}
    }

    private static final DefaultProperties[] defaults_ =
    {
	new DefaultProperties(EventQoS.MAX_RETRIES, "10"),
	new DefaultProperties(EventQoS.MAX_EVENTS, "10"),
	new DefaultProperties(EventQoS.RETRY_TIMEOUT, "1000"),
	new DefaultProperties(EventQoS.RETRY_MULTIPLIER, "2"),
	new DefaultProperties(EventQoS.PULL_INTERVAL, "0"),
	new DefaultProperties(EventQoS.REQUEST_TIMEOUT, "5000")
    };

    private static void
    setDefault(Properties properties, String key)
    {
	for(int i = 0; i < defaults_.length; ++i)
	    if(defaults_[i].key.equals(key))
	    {
		properties.put(defaults_[i].key, defaults_[i].value);
		return;
	    }
    }

    //
    // Set default values for the properties.
    //
    public static void
    setDefaults(ORB orb, EventLogger logger)
    {
	String progName = "EventService";

	com.ooc.CORBA.ORB oborb = (com.ooc.CORBA.ORB)orb;
	Properties properties = oborb.properties();

	//
	// If the value is not set then set to the default.
	//
	int i;
	for(i = 0; i < defaults_.length; ++i)
	    if(properties.getProperty(defaults_[i].key) == null)
		properties.put(defaults_[i].key, defaults_[i].value);
	    
	//
	// Check properties value.
	//
	String propRoot = "ooc.event.";
	Enumeration keys = properties.propertyNames();
	while(keys.hasMoreElements())
	{
	    String key = (String)keys.nextElement();
	    if(!key.startsWith(propRoot))
	        continue;

	    String value = properties.getProperty(key);
	    com.ooc.OB.Assert._OB_assert(value != null);
		
	    if(key.equals(EventQoS.MAX_RETRIES) ||
	       key.equals(EventQoS.PULL_INTERVAL))
	    {
		long v;
		try
		{
		    v = Long.parseLong(value);
		}
		catch(NumberFormatException e)
		{
		    v = -1;
		}
		if(v < 0)
		{
		    logger.warning(progName + ": " + key +
				   " must be >= 0. Using default value.");
		    setDefault(properties, key);
		}
	    }
	    else if(key.equals(EventQoS.MAX_EVENTS))
	    {
		long v;
		try
		{
		    v = Long.parseLong(value);
		}
		catch(NumberFormatException e)
		{
		    v = -1;
		}
		if(v <= 0)
		{
		    logger.warning(progName + ": " + key +
				   " must be > 0. Using default value");
		    setDefault(properties, key);
		}
	    }
	    else if(key.equals(EventQoS.RETRY_TIMEOUT) || 
		    key.equals(EventQoS.REQUEST_TIMEOUT))
	    {
		long v;
		try
		{
		    v = Long.parseLong(value);
		}
		catch(NumberFormatException e)
		{
		    v = -1;
		}
		if(v < 0 || v > 999999)
		{
		    logger.warning(progName + ": " + key +
				   " must be > 0 and < 999999. " +
				   "Using default value");
		    setDefault(properties, key);
		}
	    }
	    else if(key.equals(EventQoS.RETRY_MULTIPLIER))
	    {
		double v;
		try
		{
		    v = Double.valueOf(value).doubleValue();
		}
		catch(NumberFormatException e)
		{
		    v = -1;
		}
		if(v < 0 || v > 999999)
		{
		    logger.warning(progName + ": " + key +
				   " must be > 0 and < 999999. " +
				   "Using default value");
		    setDefault(properties, key);
		}
	    }
	    else if(key.equals("ooc.event.typed_service") ||
		    key.equals("ooc.event.endpoint") ||
		    key.equals("ooc.event.reap_frequency") ||
		    key.equals("ooc.event.inactivity_timeout") ||
		    key.equals("ooc.event.trace.events") ||
		    key.equals("ooc.event.trace.lifecycle"))
	    {
		// Ignore
	    }
	    else
	    {
		logger.warning(progName + ": `" + key +
			       "': unknown property");
	    }
	}
    }
}
