// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEvent;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory_impl;

//
// This class encapsulates the typed event service
// 
class TypedEventService
{
    //
    // My ORB
    //
    private ORB orb_;

    //
    // My POA
    //
    private POA poa_;

    //
    // The event channel factory
    //
    private TypedEventChannelFactory_impl channelFactory_;

    //
    // My proxy reaper
    //
    private Reaper reaper_;

    //
    // Are we initialized?
    //
    private boolean initialized_;

    // ------------------------------------------------------------------
    // TypedEventService constructor/shutdown
    // ------------------------------------------------------------------

    public TypedEventService(ORB orb, POA poa)
    {
	orb_ = orb;
	poa_ = poa;
	initialized_ = false;
    }

    // ------------------------------------------------------------------
    // TypedEventService public member implementation
    // ------------------------------------------------------------------

    //
    // Shutdown the event service
    //
    public void shutdown()
    {
	//
	// Stop the proxy reaper
	//
	if(reaper_ != null)
	{
	    reaper_.destroy();
	    while(reaper_.isAlive())
	    {
		try
		{
		    reaper_.join();
		}
		catch(InterruptedException ex)
		{
		}
	    }
	    reaper_ = null;
	}

	//
	// Shutdown the event service
	//
	if(channelFactory_ != null)
	{
	    channelFactory_.shutdown();
	    channelFactory_ = null;
	}
    }

    //
    // Initialize the event service
    //
    void
    initialize()
	throws WrongPolicy
    {
	if(initialized_)
	    return;

	initialized_ = true;

	//
	// Create the event logger
	//
	com.ooc.CORBA.ORB oborb = (com.ooc.CORBA.ORB) orb_;
	java.util.Properties properties = oborb.properties();
	com.ooc.OB.Logger logger = oborb.logger();
	EventLogger eventLogger = new EventLogger(logger, properties);

	//
	// Create the proxy reaper
	//
	reaper_ = new Reaper(orb_, eventLogger);
	reaper_.start();
    
	//
	// Create and activate the typed event channel factory
	//
	channelFactory_ =
	    new TypedEventChannelFactory_impl(orb_, poa_, eventLogger, reaper_);
	byte[] oid = ("DefaultTypedEventChannelFactory").getBytes();
	try
	{
	    poa_.activate_object_with_id(oid, channelFactory_);
	}
	catch(ObjectAlreadyActive ex)
	{
	    throw new RuntimeException();
	}
	catch(ServantAlreadyActive ex)
	{
	    throw new RuntimeException();
	}
    }

    //
    // Get event typed channel factory
    //
    com.ooc.OBTypedEventChannelFactory.TypedEventChannelFactory
    typedEventChannelFactory()
    {
	return channelFactory_._this(orb_);
    }
}
