// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventChannelAdmin;

import org.omg.CORBA.*;
import com.ooc.CosEvent.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;

final public class EventChannel_impl
    extends org.omg.CosEventChannelAdmin.EventChannelPOA
{
    //
    // The ORB
    //
    private ORB orb_;

    //
    // The POA
    //
    private POA poa_;

    //
    // The message logger
    //
    private EventLogger logger_;

    //
    // My pipe segment.
    //
    private PipeSegment myPipe_;

    //
    // The consumer admin.
    //
    private ConsumerAdmin_impl consumerAdmin_;

    //
    // The supplier admin.
    //
    private SupplierAdmin_impl supplierAdmin_;

    //
    // The channel's Id.
    //
    private String myId_;

    //
    // Has the channel been destroyed?
    //
    private boolean destroy_ = false;

    // -----------------------------------------------------------------
    // EventChannel constructor
    // -----------------------------------------------------------------

    public EventChannel_impl(ORB orb, POA poa, EventLogger logger,
			     Reaper reaper, String myId)
    {
    	orb_ = orb;
	poa_ = poa;
	logger_ = logger;
	myId_ = myId;
	myPipe_ = new PipeSegment();

	org.omg.CORBA.Object poaObj = null;
	try
	{
	     poaObj = orb.resolve_initial_references("RootPOA");
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	}
	POA rootPOA = POAHelper.narrow(poaObj);

	if(logger_.traceLifecycle() >= 1)
	    logger_.trace("event", "creating EventChannel(" + myId_ + ")");

        consumerAdmin_ = new ConsumerAdmin_impl(orb, rootPOA, logger,
						reaper, myPipe_);
	consumerAdmin_._this(orb);
        supplierAdmin_ = new SupplierAdmin_impl(orb, rootPOA, logger,
						reaper, myPipe_);
	supplierAdmin_._this(orb);

	SetDefaultProperties.setDefaults(orb, logger);
    }

    protected void
    finalize()
        throws Throwable
    {
	if(logger_.traceLifecycle() >= 1)
	    logger_.trace("event", "destroying EventChannel(" + myId_ + ")");
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------
    
    synchronized public org.omg.CosEventChannelAdmin.ConsumerAdmin
    for_consumers()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	return consumerAdmin_._this();
    }

    synchronized public org.omg.CosEventChannelAdmin.SupplierAdmin
    for_suppliers()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	return supplierAdmin_._this();
    }

    synchronized public void
    destroy()
    {
	if(destroy_)
	    throw new OBJECT_NOT_EXIST();

	destroy_ = true;

	POA poa = _default_POA();
	try
	{
	    byte[] oid = poa.servant_to_id(this);
	    poa.deactivate_object(oid);
	}
	catch(ServantNotActive ex)
	{
	    throw new RuntimeException();
	}
	catch(WrongPolicy ex)
	{
	    throw new RuntimeException();
	}
	catch(ObjectNotActive ex)
	{
	    throw new RuntimeException();
	}
        
	myPipe_.destroy();

        consumerAdmin_.disconnect();
        supplierAdmin_.disconnect();
    }

    public POA
    _default_POA()
    {
	return poa_;
    }

    // ------------------------------------------------------------------
    // Internal functions
    // ------------------------------------------------------------------
    
    synchronized public void
    discardRequests()
    {
	if(destroy_)
	    return;

	myPipe_.destroy();

	consumerAdmin_.discardRequests();
	supplierAdmin_.discardRequests();
    }

    synchronized public boolean
    destroyed()
    {
	return destroy_;
    }
}
