// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventChannelAdmin;

import com.ooc.CosEvent.*;
import java.util.*;
import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;

//
// The event channel ProxyPullSupplier implementation
//
final class ProxyPullSupplier_impl
    extends org.omg.CosEventChannelAdmin.ProxyPullSupplierPOA
    implements ProxyBase
{
    //
    // The ProxyBaseImpl Object
    //
    private ProxyBaseImpl proxyImpl_;

    //
    // My pull pipe segment
    //
    private PullPipeSegment pullPipe_;

    //
    // My pull consumer
    //
    private org.omg.CosEventComm.PullConsumer consumer_;

    //
    // Maximum number of pending events.
    //
    private long maxEvents_;

    //
    // Can this proxy be reaped?
    //
    private int canReap_;

    // ------------------------------------------------------------------
    // ProxyPullSupplier_impl constructor
    // ------------------------------------------------------------------

    ProxyPullSupplier_impl(ProxyConstructorInfo info)
    {
	info.base = "ProxyPullSupplier";
	proxyImpl_ = new ProxyBaseImpl(info, this);
	canReap_ = 0;

	//
	// Resolve the QoS params
	//
	try
	{
	    com.ooc.CORBA.ORB oborb = (com.ooc.CORBA.ORB) proxyImpl_.getOrb();
	    java.util.Properties properties = oborb.properties();

	    maxEvents_ = Long.parseLong(
		properties.getProperty(EventQoS.MAX_EVENTS));
	}
	catch(NumberFormatException e)
	{
	    throw new RuntimeException();
	}
    }

    protected void
    finalize()
        throws Throwable
    {
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    synchronized public void
    connect_pull_consumer(org.omg.CosEventComm.PullConsumer consumer)
	throws org.omg.CosEventChannelAdmin.AlreadyConnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() == ProxyState.Connected)
	    throw new org.omg.CosEventChannelAdmin.AlreadyConnected();

	//
	// The consumer is permitted to be nil
	//
	if(consumer != null)
	{
	    QoSParameters params = new QoSParameters();
	    proxyImpl_.resolveQoSParams(params);
	    
	    consumer_ = org.omg.CosEventComm.PullConsumerHelper.narrow(
		proxyImpl_.applyRequestTimeout(consumer, params)
            );
	}

	proxyImpl_.stampRequest();
	proxyImpl_.setState(ProxyState.Connected);

	pullPipe_ = new PullPipeSegment(proxyImpl_.getLogger(),
					maxEvents_, proxyImpl_.getId());
	proxyImpl_.setMyPipe(pullPipe_);

	//
	// Connect my pipe segment to parent pipe segment.
	//
	proxyImpl_.getParentPipe().connect(pullPipe_);
    }

    public Any
    pull()
	throws org.omg.CosEventComm.Disconnected
    {
	synchronized(this)
	{
	    canReap_++;
	}

	//
	// Get the first event, and return the data. If myPipe_ is not
	// initialized or the PipeSegment has been destroyed
	// getFirstEventWait() returns 0.
	//
	Event_impl event = null;
	if(pullPipe_ != null)
	    event = pullPipe_.getFirstEventWait();

	synchronized(this)
	{
	    canReap_--;

	    if(proxyImpl_.getState() == ProxyState.Destroyed)
		throw new OBJECT_NOT_EXIST();

	    if(proxyImpl_.getState() == ProxyState.Discarding)
		throw new TRANSIENT();
	    
	    if(proxyImpl_.getState() != ProxyState.Connected)
		throw new org.omg.CosEventComm.Disconnected();
	    
	    proxyImpl_.stampRequest();

	    com.ooc.OB.Assert._OB_assert(event != null);
	    return event.any();
	}
    }

    synchronized public Any
    try_pull(BooleanHolder hasEvent)
	throws org.omg.CosEventComm.Disconnected
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	if(proxyImpl_.getState() != ProxyState.Connected)
	    throw new org.omg.CosEventComm.Disconnected();
	
	proxyImpl_.stampRequest();

	Event_impl event = pullPipe_.getFirstEvent();

	//
	// Determine if we have an event.
	//
	hasEvent.value = (event != null);

	Any any;
	if(hasEvent.value)
	{
	    //
	    // Yes, then get the first event. Return the data.
	    //
	    any = event.any();
	}
	else
	{
	    any = proxyImpl_.getOrb().create_any();
	}
	return any;
    }

    synchronized public void
    disconnect_pull_supplier()
    {
	if(proxyImpl_.getState() == ProxyState.Destroyed)
	    throw new OBJECT_NOT_EXIST();

	if(proxyImpl_.getState() == ProxyState.Discarding)
	    throw new TRANSIENT();

	proxyImpl_.disconnect();
    }

    public POA
    _default_POA()
    {
        return proxyImpl_.getPoa();
    }

    // ------------------------------------------------------------------
    // ProxyBase implementation
    // ------------------------------------------------------------------

    //
    // Disconnect the servant from the ORB
    //
    public void deactivateObject()
    {
	POA poa = _default_POA();
	try
	{
	    byte[] oid = poa.servant_to_id(this);
	    poa.deactivate_object(oid);
	}
	catch(ServantNotActive ex)
	{
	    throw new RuntimeException();
	}
	catch(WrongPolicy ex)
	{
	    throw new RuntimeException();
	}
	catch(ObjectNotActive ex)
	{
	    throw new RuntimeException();
	}
    }
    
    //
    // Called during disconnect
    //
    public void disconnectPeer()
    {
	//
	// If we're connected then tell the consumer.
	//
	if(consumer_ != null)
	{
	    try
	    {
		consumer_.disconnect_pull_consumer();
	    }                                              
	    catch(SystemException e)
	    {
		// A non-existent supplier doesn't matter in this context.
	    }
	    consumer_ = null;
	}
    }

    //
    // Disconnect the proxy
    //
    public synchronized void disconnect()
    {
	proxyImpl_.disconnect();
    }

    //
    // Don't accept further requests
    //
    public synchronized void discardRequests()
    {
	proxyImpl_.discardRequests();
    }

    //
    // Has this proxy been destroyed?
    //
    public synchronized boolean destroyed()
    {
	return proxyImpl_.destroyed();
    }

    //
    // Can this proxy be reaperd?
    //
    public synchronized boolean canReap()
    {
	return canReap_ == 0;
    }

    //
    // What was the time of the last request?
    //
    public long lastRequestTime()
    {
	return proxyImpl_.lastRequestTime();
    }

    //
    // Set the time of the last request
    //
    public void stampRequest()
    {
	proxyImpl_.stampRequest();
    }

    //
    // Get the name of the proxy
    //
    public String getName()
    {
	return proxyImpl_.getName();
    }
}
