// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosEventChannelAdmin;

import java.util.*;
import com.ooc.CosEvent.*;
import org.omg.CORBA.*;

class PullPipeSegment extends PipeSegment
{
    //
    // Queue of pending events.
    //
    private Vector events_ = new Vector();

    //
    // Maximum number of events to queue
    //
    private long maxEvents_;

    //
    // My ID.
    //
    private long id_;

    //
    // The message logger.
    //
    private EventLogger logger_;
    
    public PullPipeSegment(EventLogger logger, long maxEvents, long id)
    {
	logger_ = logger;
	maxEvents_ = maxEvents;
	id_ = id;
    }

    public boolean
    receive(Event_impl event)
    {
	if(destroy_)
	    return false;
	
	synchronized(this)
	{
	    //
	    // If we queue more than maxEvents_ items then we throw oldest
	    // items away first.
	    //
	    events_.insertElementAt(event, 0);
	
	    if(events_.size() > maxEvents_)
	    {
		int size = events_.size();
		Event_impl e = (Event_impl)events_.elementAt(size - 1);

		if(logger_.traceEvents() >= 2)
		    logger_.trace("event", "ProxyPullSupplier(" + id_ +
			      ") Discarding event (" + e.eventId() + ")");

		events_.removeElementAt(size - 1);
	    }

	    //
	    // Wake any waiting pull() events.
	    //
	    notify();

	    return true;
	}
    }

    synchronized public void
    destroy()
    {
	destroy_ = true;
	notify();

	//
	// Discard each event in the queue
	//
	events_.removeAllElements();
    }

    synchronized public Event_impl
    getFirstEvent()
    {
	if(destroy_ || events_.size() == 0)
	    return null;
	
	Event_impl event = (Event_impl)events_.elementAt(events_.size() - 1);
	events_.removeElementAt(events_.size() - 1);
	return event;
    }

    synchronized public Event_impl
    getFirstEventWait()
    {
	//
	// Block until there is an event, and we're not destroyed
	//
	while(events_.size() == 0 && !destroy_)
	{
	    try
	    {
		wait();
	    }
	    catch(InterruptedException ex)
	    {
	    }
	}

	if(destroy_)
	    return null;

	com.ooc.OB.Assert._OB_assert(events_.size() > 0);
	return getFirstEvent();
    }
}
