// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.CosEventComm.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import java.io.*;
import java.util.*;

class EventPullConsumer_impl extends PullConsumerPOA
{
    private boolean disconnected_;
    private static int tracelevel_;
    private POA poa_; // My POA
    private StringBuffer buf_;

    public 
    EventPullConsumer_impl(int tl, POA poa, StringBuffer buf)
    {
	tracelevel_ = tl;
	disconnected_ = false;
	poa_ = poa;
        buf_ = buf;
    }

    synchronized public boolean
    disconnected()
    {
	return disconnected_;
    }

    synchronized public void
    disconnect_pull_consumer()
    {
        buf_.append("Disconnect\n");
	disconnected_ = true;
        TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 2, 
			   "Disconnect Pull Consumer");
    }

    public POA
    _default_POA()
    {
	return poa_;
    }
}

//
// Base class for proxy wrapper for pulling and displaying events.
//
abstract class EventPullerBase
{
    StringBuffer buf_;

    public void 
    receiveAny(Any event)
    {
        try
        {
            String s = event.extract_string();
            buf_.append(s);
            buf_.append(" ");
        }
        catch(MARSHAL ex) 
        {
            buf_.append("unknown\n");
        }       
    }
    
    public
    EventPullerBase(StringBuffer buf) 
    {
	buf_ = buf;
    }

    public abstract boolean pull() throws Disconnected;
}

class UntypedEventPuller extends EventPullerBase
{
    private ProxyPullSupplier proxy_;

    public
    UntypedEventPuller(ProxyPullSupplier proxy, StringBuffer buf)
    {
	super(buf);	
	proxy_ = proxy;
    }

    public boolean
    pull()
    {
	BooleanHolder hasEvent = new BooleanHolder();
	try
	{
	    Any event = proxy_.try_pull(hasEvent);
	    if(hasEvent.value)
		receiveAny(event);
	}
	catch(org.omg.CosEventComm.Disconnected ex)
	{
	    ex.printStackTrace();
	    System.exit(1);
	}
	return hasEvent.value;
    }
}

class EventPuller extends EventPullerBase
{
    EventPullerBase pullerImpl_;

    public
    EventPuller(StringBuffer buf)
    {
	super(buf);
	pullerImpl_ = null;
    }

    public void 
    connect(ProxyPullSupplier proxy)
    {
	pullerImpl_ = new UntypedEventPuller(proxy, buf_);
    }

    public boolean
    pull()
	throws Disconnected
    {
	if(pullerImpl_ == null)
	    return false;

	return pullerImpl_.pull();
    }
}

public class PullEventClient
{ 
    private static int evt_num = 0;
    private static int tracelevel_ = 0;
    
    //
    // Checking conditions about the time taken and the number of events
    // received.  'taken' refers to the number of pulls taken thus far.
    // 'allowed' refers to the number of pulls allowed before exiting.
    // 'expect' is the numberof events expected to be received.  Finally,
    // 'received' is the number of events that have been received thus far.
    //
    private static boolean 
    CheckConditions(int taken, int allowed, int expect, int received)
    {
	//
	// See if we have waited too long for an event
	//
	if(taken == allowed)
	{
	    if(expect == -1)
	    {
		return true;
	    }
	    else if(received < expect)
	    {
		TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 2, 
				   "Not enough events");
		return true;
	    }
	    else 
	    {
		TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 2, 
			           "Received correct number of events");
		return false;
	    }
	}
	return false;
    }

    private static void 
    Usage()
    {
	System.out.println("pulleventclient <proxy-ior> [--expect n] " +
			   "[--disconnect]\n" +
			   "   --expect:  Specify the number of events to " +
			   "expect\n" +
			   "   --disconnect:  Call connect on the proxy and " + 
			   "disconnect\n");
    }

    static int
    run(ORB orb, String args[])
    {
        int allowed = 10;

	if(args.length > 4)
	{
	    Usage();
	    return 1;
	}

	//
	// How many events should we expect?
	//
	int expect = -1;

	//
	// Connect and exit.
	//
	boolean disconnect = false;

	//
	// Check for disconnect or expect arguments.
	//

	int i;
	for(i = 0 ; i < args.length ; i++)
	{
	    if(args[i].equalsIgnoreCase("--disconnect"))
	    {
		disconnect=true;
	    }
	    if(args[i].equalsIgnoreCase("--expect"))
	    {
		expect = Integer.parseInt(args[i+1]);
	    }
	}

	TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 2, "Starting");

        StringBuffer outputBuffer = new StringBuffer();
	EventPuller puller = new EventPuller(outputBuffer);

	//
	// Resolve Root POA.
	//
	POA rootPOA = null;
	try
	{
	    rootPOA = POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    throw new RuntimeException();
	}

	//
	// Get a reference to the POA manager.
	//
	POAManager manager = rootPOA.the_POAManager();

	//
	// Create a persistent POA.
	//
	Policy[] pl = new Policy[2];
	pl[0] = rootPOA.create_lifespan_policy(
	    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
	pl[1] = rootPOA.create_id_assignment_policy(
	    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

	POA consumerPOA = null;
	try
	{
	    consumerPOA = rootPOA.create_POA("consumer", manager, pl);
	}
	catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
	{
	    throw new RuntimeException();
	}
	catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
	{
	    throw new RuntimeException();
	}

	//
	// Instantiate a servant to receive disconnect callbacks.
	//
	EventPullConsumer_impl consumerImpl = 
	    new EventPullConsumer_impl(tracelevel_, consumerPOA, outputBuffer);

	//
	// Connect the supplier to the POA with the same name each
	// time.
	//
	byte[] oid = ("DefaultConsumer").getBytes();
	try
	{
	    consumerPOA.activate_object_with_id(oid, consumerImpl);
	}
	catch(org.omg.PortableServer.POAPackage.ServantAlreadyActive ex)
	{
	    throw new RuntimeException();
	}
	catch(org.omg.PortableServer.POAPackage.ObjectAlreadyActive ex)
	{
	    throw new RuntimeException();
	}
	catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
	{
	    throw new RuntimeException();
	}
       
	org.omg.CosEventComm.PullConsumer consumer = 
	    consumerImpl._this(orb);

	//
	// Activate the POAManager.
	//
	try
	{
	    manager.activate();
	}
	catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	{
	    throw new RuntimeException();
	}

	//
	// Initialize the proxy.
	//
	org.omg.CORBA.Object obj = orb.string_to_object(args[0]);

        try
	{
	    ProxyPullSupplier proxy = ProxyPullSupplierHelper.narrow(obj);
     
	    try
	    {
		proxy.connect_pull_consumer(consumer);
	    }
	    catch(org.omg.CosEventChannelAdmin.AlreadyConnected ex)
	    {
	    }

	    if(disconnect) 
	    {
		TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 1, 
				"Connected Pull Consumer");
		return 0;
	    }
		
	    TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 1, 
			       "Ready to receive events");

	    puller.connect(proxy);
	}
	catch(BAD_PARAM ex)
	{
	    System.err.println("Couldn't narrow Proxy Pull Supplier");
	    System.exit(1);
	}

	//
	// Maintain some statistics about the event transfer.
	//
	int received = 0;
	int taken = 0; // In seconds.
	

	while(!consumerImpl.disconnected())
	{
	    try
	    {
		// 
		// We've exceeded the parameters for the run.  Exit.
		//
		if(CheckConditions(taken, allowed, expect, received))
		{
		    TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 3, 
				   "Runtime limits reached.  Exiting");
		    return 1;
		}

		boolean hasEvent = puller.pull();
		    
		//
		// A brief pause to allow reception of the event.
		//
		if(!hasEvent)
		{
		    taken++;
		    try
		    {
			Thread.sleep(1000); 
		    }
		    catch(java.lang.InterruptedException ex)
		    {
		    }
		    continue;
		}

		TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 3, 
				   "Received Event"); 
		    
		//
		// We have an event, so now we can reset the attempts
		// taken and increment the number of received events.
		//
		taken = 0;
		received++;

		if((expect != -1) && (received > expect))
		{
		    TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 
				       3, "Too many events returned, " +
				       "Exiting");
		    return 1;
		}
		    
	    }
	    catch(Disconnected ex)
	    {
		TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 2, 
				   "ERROR: Caught unexpected " +
				   "disconnected exception");
	      
		return 1;
	    }
	    catch(org.omg.CORBA.OBJECT_NOT_EXIST ex)
	    {
		TSU.PrintStatusMsg(tracelevel_, "PullEventClient", 2, 
			       "CAUGHT OBJECT_NOT_EXIST");
		try
		{
		    Thread.sleep(4000); 
		}
		catch(java.lang.InterruptedException expt)
		{
		}
                System.out.println(outputBuffer);
		return 1;
	    }
	}
        System.out.println(outputBuffer);
	return 0;
    }

    public static void
    main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

    	int status = 0;
	ORB orb = null;	

        try
        {
            //
            // Create ORB.
            //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);

	    java.util.Properties prop = 
		((com.ooc.OBCORBA.ORB_impl)orb).properties();
	    java.util.Enumeration keys = prop.keys();

	    //
	    // Get the trace level.
	    //
	    while(keys.hasMoreElements())
	    {
		String key = (String)keys.nextElement();
		String value = prop.getProperty(key);

		if(key.startsWith("ooc.testsuite.event.trace"))
		{
		    tracelevel_ = Integer.parseInt(prop.getProperty(key));
		    break;
		}
	    }

	    status = run(orb, argsH.value);
	}	
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}
	
	if(orb != null)
	{
	    try
	    {
	        orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
