// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package event;

import org.omg.CORBA.*;
import org.omg.CosEventChannelAdmin.*;
import org.omg.CosEventComm.*;
import org.omg.PortableServer.*;
import org.omg.PortableServer.POAPackage.*;
import java.io.*;
import java.util.*;

public class PushEventClient
{
    private static int tracelevel_ = 0;

    private static void
    Usage()
    {
	System.out.println("pusheventclient <proxy-ior> [--disconnect]\n" +
			   "   --disconnect:  Call connect on the proxy " +
			   "and disconnect");
    }

    static int
    run(ORB orb, String args[])
    {
        StringBuffer outputBuffer = new StringBuffer();

        try
	{
	    //
	    // Resolve Root POA.
	    //
	    POA rootPOA = null;
	    try
	    {
		rootPOA = POAHelper.narrow(
		    orb.resolve_initial_references("RootPOA"));
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		throw new RuntimeException();
	    }

	    //
	    // Get a reference to the POA manager.
	    //
	    POAManager manager = rootPOA.the_POAManager();

	    if(args.length > 3)
	    {
		Usage();
		return 1;
	    }

	    //
	    // Just connect and exit.
	    //
	    boolean disconnect = false;

	    //
	    // Find whether to disconnect or not
	    //
	    for(int i = 0 ; i < args.length ; i++)
	    {
		if(args[i].equals("--disconnect"))
		{
		    disconnect=true;
		}
	    }

	    //
	    // Create a persistent POA.
	    //
	    Policy[] pl = new Policy[2];
	    pl[0] = rootPOA.create_lifespan_policy(
		org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
	    pl[1] = rootPOA.create_id_assignment_policy(
		org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

	    POA consumerPOA = null;
	    try
	    {
		consumerPOA = rootPOA.create_POA("consumer", manager, pl);
	    }
	    catch(org.omg.PortableServer.POAPackage.AdapterAlreadyExists ex)
	    {
		throw new RuntimeException();
	    }
	    catch(org.omg.PortableServer.POAPackage.InvalidPolicy ex)
	    {
		throw new RuntimeException();
	    }

	    //
	    // Create the servant
	    //
	    PushConsumer_impl pc = new PushConsumer_impl(orb, consumerPOA,
							 outputBuffer);

	    //
	    // Connect the supplier to the POA with the same name each
	    // time.
	    //
	    byte[] oid = ("DefaultConsumer").getBytes();
	    try
	    {
		consumerPOA.activate_object_with_id(oid, pc);
	    }
	    catch(org.omg.PortableServer.POAPackage.ServantAlreadyActive ex)
	    {
		throw new RuntimeException();
	    }
	    catch(org.omg.PortableServer.POAPackage.ObjectAlreadyActive ex)
	    {
		throw new RuntimeException();
	    }
	    catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
	    {
		throw new RuntimeException();
	    }
	    org.omg.CosEventComm.PushConsumer consumer = pc._this(orb);

	    //
	    // Activate the POAManager.
	    //
	    try
	    {
		manager.activate();
	    }
	    catch(org.omg.PortableServer.POAManagerPackage.AdapterInactive ex)
	    {
		throw new RuntimeException();
	    }

	    //
	    // Initialize the proxy.
	    //
	    org.omg.CORBA.Object obj = orb.string_to_object(args[0]);
	    ProxyPushSupplier proxy = ProxyPushSupplierHelper.narrow(obj);

	    //
	    // Connect the proxy. If we're already connected we'll get an
	    // exception. This should only happen if we're reusing the
	    // same proxy-object -- this this case the connected
	    // SequencePushConsumer must be the same servant.
	    //
	    try
	    {
		proxy.connect_push_consumer(consumer);
	    }
	    catch(org.omg.CosEventChannelAdmin.AlreadyConnected ex)
	    {
	    }
	    catch(org.omg.CosEventChannelAdmin.TypeError ex)
	    {
		ex.printStackTrace();
		return 1;
	    }
	    
	    if(disconnect)
	    {
		TSU.PrintStatusMsg(tracelevel_, "PushEventClient", 2, 
			       "Connected Any Push Consumer");
		return 0;
	    }
	    
	    //
	    // Run implementation.
	    //
	    orb.run();
	}
	catch(BAD_PARAM ex)
	{
	    System.err.println("Call to narrow failed!");
	    return 1;
	}

	//
	// Expected if ORB/Client has already been shutdown
	//
	catch(TRANSIENT ex)
	{
	}
	catch(BAD_INV_ORDER ex)
	{
	}
	catch(INITIALIZE ex)
	{
	}
	catch(NO_RESPONSE ex)
	{
        }
        System.out.println(outputBuffer);
	return 0;
    }

    public static void
    main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

    	int status = 0;
	ORB orb = null;	

        try
        {
            //
            // Create ORB.
            //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);

	    java.util.Properties prop = 
		((com.ooc.OBCORBA.ORB_impl)orb).properties();
	    java.util.Enumeration keys = prop.keys();

	    //
	    // Get the trace level.
	    //
	    while(keys.hasMoreElements())
	    {
		String key = (String)keys.nextElement();
		String value = prop.getProperty(key);

		if(key.startsWith("ooc.testsuite.event.trace"))
		{
		    tracelevel_ = Integer.parseInt(prop.getProperty(key));
		    break;
		}
	    }

	    status = run(orb, argsH.value);
	}	
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}
	
	if(orb != null)
	{
	    try
	    {
	        orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}

class PushConsumer_impl extends PushConsumerPOA
{
    private ORB orb_; // The ORB
    private POA poa_; // My POA
    private StringBuffer buf_;

    PushConsumer_impl(ORB orb, POA poa, StringBuffer buf)
    {
	orb_ = orb;
	poa_ = poa;
        buf_ = buf;
    }
    
    public void
    disconnect_push_consumer()
    {
        buf_.append("Disconnect\n");

        byte[] oid = null;
        try
        {
            oid = poa_.servant_to_id(this);
        }
        catch(ServantNotActive ex)
        {
            throw new RuntimeException();
        }
        catch(WrongPolicy ex)
        {
            throw new RuntimeException();
        }

        try
        {
            poa_.deactivate_object(oid);
        }
        catch(ObjectNotActive ex)
        {
            throw new RuntimeException();
        }
        catch(WrongPolicy ex)
        {
            throw new RuntimeException();
        }

        orb_.shutdown(false);
    }
    
    public void
    push(Any any)
    {
        try
        {
            String s = any.extract_string();
            buf_.append(s);
            buf_.append(" ");
        }
        catch(MARSHAL ex) 
        {
            buf_.append("unknown\n");
        }       
    }

    public POA
    _default_POA()
    {
	return poa_;
    }
}
