// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.IMRConsole;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import com.ooc.IMR.*;
import org.omg.CORBA.*;

final class ErrorDialog extends JDialog
{
    private static JFrame parent_;

    static void
    initialize(JFrame parent)
    {
	parent_ = parent;
    }

    static void
    showWarning(String msg)
    {
	JOptionPane.showMessageDialog(
	    parent_, msg, "Warning!", JOptionPane.WARNING_MESSAGE);
    }

    static void
    showError(String msg)
    {
	JOptionPane.showMessageDialog(
	    parent_, msg, "Error!", JOptionPane.ERROR_MESSAGE);
    }

    static void
    showUnfoundObjectReference(UnfoundObjectReference ex)
    {
	String title = "Unfound Object Exception";

	String msg = "Cannot find: " + ex.getMessage() + "\n\n" +
	             "The IMR Console attempted to access\n" + 
	             "an object which no longer exists within the\n" + 
		     "IMR.\n\n" + 
		     "You may want to refresh the console.";

	JOptionPane.showMessageDialog(parent_, msg, title,
				      JOptionPane.WARNING_MESSAGE);
    }
    
    static void
    showNoSuchOAD(NoSuchOAD ex)
    {
	String title = "No Such OAD Exception!";

	String msg = "No such OAD: " + ex.getMessage() + "\n\n" +
	             "The IMR Console attempted to access\n" + 
	             "an OAD that doesn't exist within\n" + 
		     "the IMR.\n\n" + 
		     "You may want to refresh the console.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    static void
    showOADAlreadyExists(OADAlreadyExists ex)
    {
	String title = "OAD Already Exists Exception!";

	String msg = "OAD already exists: " + ex.getMessage() + "\n\n" +
	             "The IMR Console attempted to create\n" + 
	             "an OAD that has already been created within \n" + 
		     "the IMR.\n\n" + 
		     "You may want to refresh the console.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    static void
    showOADRunning(OADRunning ex)
    {
	String title = "OAD Running Exception!";

	String msg = "OAD is running: " + ex.getMessage() + "\n\n" +
	             "The IMR Console attempted to destroy\n" + 
	             "an OAD that is currently running.\n\n" + 
		     "You may want to shutdown the OAD.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    static void
    showOADNotRunning(OADNotRunning ex)
    {
	String title = "OAD Not Running Exception!";
	String msg = "OAD is not running: " + ex.getMessage();
	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    static void
    showServerRunning(ServerRunning ex)
    {
	String title = "Server Running Exception!";
	String msg = "Server is running: " + ex.getMessage();
	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    static void
    showServerNotRunning(ServerNotRunning ex)
    {
	String title = "Server Not Running Exception!";
	String msg = "Server is not running: " + ex.getMessage();
	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }
    
    static void
    showServerAlreadyRegistered(ServerAlreadyRegistered ex)
    {
	String title = "Server Already Registered Exception!";

	String msg = "Server already registered: " + ex.getMessage() + "\n\n" +
	             "The IMR Console attempted to register\n" + 
	             "a server that has already been registered within \n" + 
		     "the IMR.\n\n" + 
		     "You may want to refresh the console.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }

    static void
    showPOAAlreadyRegistered(POAAlreadyRegistered ex)
    {
	String title = "POA Already Registered Exception!";

	String msg = "POA already registered: " + ex.getMessage() + "\n\n" +
	             "The IMR Console attempted to register\n" + 
	             "a POA that has already been registered with \n" + 
		     "the same server within the IMR.\n\n" + 
		     "You may want to refresh the console.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }

    static void
    showNoSuchServer(NoSuchServer ex)
    {
	String title = "No Such Server Exception!";

	String msg = "Cannot find server: " + ex.getMessage() + "\n\n" +
	             "The IMR Console attempted to access\n" + 
	             "a server which no longer exists within the\n" + 
		     "IMR.\n\n" + 
		     "You may want to refresh the console.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }

    static void
    showNoSuchPOA(_NoSuchPOA ex)
    {
	String title = "No Such POA Exception!";

	String msg = "Cannot find POA: " + ex.getMessage() + "\n\n" +
	             "The IMR Console attempted to access\n" + 
	             "a POA which no longer exists within the\n" + 
		     "IMR.\n\n" + 
		     "You may want to refresh the console.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }

    static void
    showBadValue(NumberField field, String name)
    {
	String title = "Error! Bad Value";

	String msg = "The value for `" + name + "' is either\n" +
	             "not of type " + field.getType() + 
		     ", or is outside its\n" + 
		     "valid range:\n\n" + field.getRange() + "\n\n" +
		     "Please re-enter a suitable value.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }

    static void
    showBadValue(JTextField field, String name)
    {
	String title = "Error! Bad Value";

	String msg = "The value for `" + name + "' is invalid\n" +
		     "Please re-enter a suitable value.";

	JOptionPane.showMessageDialog(parent_, msg, title,
                                      JOptionPane.ERROR_MESSAGE);
    }

    static void
    showSystemException(org.omg.CORBA.SystemException ex)
    {
	String title = "CORBA System Exception";
	String msg =
	    "A CORBA System Exception has occured.\n\n" +
	    ex.toString() + "\n"
	    + "Completion Status: " + ex.completed.value() + " ";
	switch(ex.completed.value())
	{
	    case CompletionStatus._COMPLETED_YES:
		msg += "(YES)";
		break;
	    case CompletionStatus._COMPLETED_NO:
		msg += "(NO)";
		break;
	    case CompletionStatus._COMPLETED_MAYBE:
		msg += "(MAYBE)";
		break;
	}
	msg += "\n\nPlease try again.";

	JOptionPane.showMessageDialog(
	    parent_, msg, title, JOptionPane.ERROR_MESSAGE);
    }

    static void
    showNoPermissionException()
    {
        String title = "No permission";
	String msg = "Implementation Repository is not running" +
		     " in administrative mode.";
	JOptionPane.showMessageDialog(
	    parent_, msg, title, JOptionPane.ERROR_MESSAGE);
    }

    static boolean
    showConfirm(String title, String msg)
    {
	java.lang.Object[] options = {"Ok", "Cancel"};
	int res = JOptionPane.showOptionDialog(parent_, msg, title,
	                                       JOptionPane.YES_NO_OPTION,
	                                       JOptionPane.QUESTION_MESSAGE,
	                                       null, options, options[1]);

	if(res == 0)
	{
	    return true;
	}

	return false;
    }

    static void
    showAbout(ImageIcon icon)
    {
	String title = "About IMR Console";

	String msg = Version.name + " " + Version.version + "\n\n" +
	             "For more information refer to:\n" +
		     "http://www.ooc.com\n" +
		     "http://www.ooc.de\n";

	JOptionPane.showMessageDialog(parent_, msg, title,
	                              JOptionPane.INFORMATION_MESSAGE, icon);
    }
}
