// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.IMRConsole;

import java.util.*;
import org.omg.CORBA.*;
import com.ooc.IMR.*;
import com.ooc.Util.CORBA.*;
import com.ooc.Util.*;
import com.ooc.Util.GUI.*;

final public class Main
{
    //
    // NOTE: If you want to use a locale-specific resource bundle,
    //       you do NOT need to change the value for bundle_
    //
    static final String bundle_ = "com.ooc.IMRConsole.AppResources";
 
    public static void
    main(String args[])
    {
        Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        //
        // Initialize the AppStandards class, which loads our locale-
        // specific resource bundle
        //
        AppStandards.init(bundle_);

        try
        {
            //
            // Create ORB
            //
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
	    ORB orb = ORBManager.init(argsH, props);

            //
            // Process the command-line arguments
            //
            GUIOptionFilter opt = new GUIOptionFilter();
            BooleanOption helpOption = new BooleanOption('h', "help");
            BooleanOption versionOption = new BooleanOption('v', "version");
            opt.addOption(helpOption);
            opt.addOption(versionOption);

            try
            {
                opt.parse(argsH.value);
            }
            catch(OptionException ex)
            {
                System.err.println("Error: " + ex.getMessage());
                System.err.println();
                usage();
            }

            if(argsH.value.length != 0)
                usage();

            if(versionOption.isDefined())
            {
                System.out.println("ORBacus IMR Console " +
                                   com.ooc.OB.Version.version);
                System.exit(0);
            }
            else if(helpOption.isDefined())
                usage();

            //
            // Resolve IMR
            //
	    org.omg.CORBA.Object imrObj = null;

	    try
	    {
  		imrObj = orb.resolve_initial_references("IMR");
	    }
	    catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	    {
		System.err.println("Invalid Name.");
		System.exit(1);
	    }

	    //
	    // Check if IMR is accessible
	    //
	    boolean exists = false;
	    if(imrObj != null)
	    {
		try
		{
		    exists = !imrObj._non_existent();
		}
		catch(COMM_FAILURE ex)
		{
		}
	    }

	    if(!exists)
	    {
		System.err.println("Couldn't access IMR");
		System.exit(1);
	    }

	    //
	    // Check if IMR reference is an IMR::ServerDomain
	    //
	    ServerDomain da = null;
            try
            {
                da = ServerDomainHelper.narrow(imrObj);
            }
            catch(BAD_PARAM ex)
	    {
		System.err.println(
		    "`IMR' initial reference is not an IMR::ServerDomain");
		System.exit(1);
	    }

	    //
	    // Create console
	    //
	    new Console(orb, da);
        }
        catch(SystemException ex)
        {
            ex.printStackTrace();
            System.exit(1);
        }
    }

    private static void
    usage()
    {
	System.out.println(AppStandards.getString("UsageKey"));
        System.exit(1);
    }
}
