// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.IMRConsole;

import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

final class NumberField extends JTextField
{
    final static int SHORT = 0;
    final static int FLOAT = 1;
    final static int LONG = 2;

    private int type_;
    private String save_;
    private String hiVal_;
    private String lowVal_;
    private boolean tempLoss_;
    private String range_;

    public boolean
    isValid()
    {
	String value = getText();

	if(validateType(value))
	    return validateRange(value);

	return false;
    }

    NumberField(String value, int type)
    {
        super(value);

	type_ = type;
	save_ = value;
	hiVal_ = null;
	lowVal_ = null;
	tempLoss_ = false;
	range_ = "(unknown)";
    }

    NumberField(String value, int type, String hiVal, String lowVal)
    {
        super(value);

	type_ = type;
	save_ = value;
	hiVal_ = hiVal;
	lowVal_ = lowVal;
	tempLoss_ = false;
	range_ = "(" + lowVal_ + ", " + hiVal_ + ")";
    }

    String
    getType()
    {
	switch(type_)
	{
	case SHORT:
	    return "Short";
	case FLOAT:
	    return "Float";
	case LONG:
	    return "Long";
	}

	return "(unknown)";
    }

    String
    getRange()
    {
	if(type_ == SHORT)
	{
	    short hi = Short.MAX_VALUE;
	    short low = Short.MIN_VALUE;

	    if(hiVal_ != null)
	    {
		hi = Short.parseShort(hiVal_);
		low = Short.parseShort(lowVal_);
	    }
    
	    range_ = "(" + low + ", " + hi + ")";
	}
	else if(type_ == FLOAT)
	{
	    float hi = Float.MAX_VALUE;
	    float low = -Float.MAX_VALUE;

	    if(hiVal_ != null)
	    {
		hi = (new Float(hiVal_)).floatValue();
		low = (new Float(lowVal_)).floatValue();
	    }

	    range_ = "(" + low + ", " + hi + ")";
	}
	else if(type_ == LONG)
	{
	    long hi = Long.MAX_VALUE;
	    long low = Long.MIN_VALUE;

	    if(hiVal_ != null)
	    {
		hi = Long.parseLong(hiVal_);
		low = Long.parseLong(lowVal_);
	    }

	    range_ = "(" + low + ", " + hi + ")";
	}

	return range_;
    }

    private boolean
    validateType(String value)
    {
        try
        {
	    if(type_ == SHORT)
	        Short.parseShort(value);
	    else if(type_ == FLOAT)
	        Float.valueOf(value);
	    else if(type_ == LONG)
	        Long.parseLong(value);
        }
        catch(NumberFormatException ex)
        {
            return false;
        }

	return true;
    }

    private boolean
    validateRange(String value)
    {
        try
        {
	    if(type_ == SHORT)
	    {
	        short val = Short.parseShort(value);
		short hi = Short.MAX_VALUE;
		short low = Short.MIN_VALUE;

		if(hiVal_ != null)
		{
		    hi = Short.parseShort(hiVal_);
		    low = Short.parseShort(lowVal_);
		}
	
		if(val <= hi && val >= low)
		{
		    return true;
		}
	    }
	    else if(type_ == FLOAT)
	    {
	        float val = (new Float(value)).floatValue();
		float hi = Float.MAX_VALUE;
		float low = -Float.MAX_VALUE;

		if(hiVal_ != null)
		{
		    hi = (new Float(hiVal_)).floatValue();
		    low = (new Float(lowVal_)).floatValue();
		}

		if(val <= hi && val >= low)
		{
		    return true;
		}
	    }
	    else if(type_ == LONG)
	    {
	        long val = Long.parseLong(value);
		long hi = Long.MAX_VALUE;
		long low = Long.MIN_VALUE;

		if(hiVal_ != null)
		{
		    hi = Long.parseLong(hiVal_);
		    low = Long.parseLong(lowVal_);
		}

		if(val <= hi && val >= low)
		{
		    return true;
		}
	    }
        }
        catch(NumberFormatException ex)
        {
        }

	return false;
    }
}
