// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA  
//              
// All Rights Reserved   
//   
// **********************************************************************

package com.ooc.IMRConsole;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import com.ooc.IMR.*;

final class ObjectTree extends JTree
    implements TreeExpansionListener
{
    private ServiceObject root_;
    private ServiceObject daObj_;
    private ServiceManager serviceMgr_;

    public void
    treeExpanded(TreeExpansionEvent event)
    {
	TreePath path = event.getPath();
	try
	{
	    refresh(path);
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	    ErrorDialog.showSystemException(ex); 
	}
    }

    public void
    treeCollapsed(TreeExpansionEvent event)
    {
    }

    ObjectTree(JFrame parent,
	       ServiceObject daObj, 
	       ServiceManager serviceMgr)
    {
	daObj_ = daObj;
	serviceMgr_ = serviceMgr;

	root_ = new ServiceObject();
	root_.add(daObj_);
	((DefaultTreeModel)getModel()).setRoot(root_);

	DefaultTreeSelectionModel treeSelectionModel = 
	    new DefaultTreeSelectionModel();
	treeSelectionModel.setSelectionMode(
	    TreeSelectionModel.SINGLE_TREE_SELECTION);

	setRootVisible(false);
        setShowsRootHandles(true);
        setSelectionModel(treeSelectionModel);
	putClientProperty("JTree.lineStyle", "Angled");
        setEditable(false);
  	addTreeExpansionListener(this);
	setCellRenderer(new ObjectTreeRenderer());

	refresh();
    }

    void
    clear()
    {
	root_.removeAllChildren();
        ((DefaultTreeModel)getModel()).reload(); 
    }

    ServiceObject
    add(String objectName, int objectType, ServiceObject parent)
    {
	ServiceObject newObj = 
	    serviceMgr_.createObject(objectName, objectType, parent);
        connectUp(parent, newObj, true);

	return newObj;
    }

    void
    remove(ServiceObject object)
    {
	((DefaultTreeModel)getModel()).removeNodeFromParent(object);
    }

    void
    move(ServiceObject object, ServiceObject newParent)
    {
	((DefaultTreeModel)getModel()).removeNodeFromParent(object);
        connectUp(newParent, object, true);
    }

    ServiceObject
    getSelectedObject()
    {
        TreePath path = getSelectionPath();

        if(path == null)
            return null;
	else
	    return (ServiceObject)path.getLastPathComponent();
    }

    void
    refresh()
    {
  	refresh(null);
    }

    void
    refreshOAD(ServiceObject oadObj) throws UnfoundObjectReference
    {
	Vector updateList = update(oadObj,
				   serviceMgr_.getServerNames(oadObj),
				   ServiceObject.SERVER);
	DefaultTreeModel model = (DefaultTreeModel)getModel();
  	if(isExpanded(new TreePath(model.getPathToRoot(oadObj))))
	    for(int i = 0; i < updateList.size(); i++)
		refreshServer((ServiceObject)updateList.elementAt(i));
    }

    void
    refreshServer(ServiceObject saObj) throws UnfoundObjectReference
    {
	Vector updateList = update(saObj,
				   serviceMgr_.getPOANames(saObj),
				   ServiceObject.POA);
	DefaultTreeModel model = (DefaultTreeModel)getModel();
  	if(isExpanded(new TreePath(model.getPathToRoot(saObj))))
	    for(int i = 0; i < updateList.size(); i++)
		refreshPOA((ServiceObject)updateList.elementAt(i));
    }

    void
    refreshPOA(ServiceObject poaObj) throws UnfoundObjectReference
    {
	poaObj.toObject();
	((DefaultTreeModel)getModel()).nodeChanged(poaObj);
    }

    private void
    connectUp(ServiceObject parent, ServiceObject child, boolean display)
    {
	int childCount = parent.getChildCount();
	int index = -1;

	if(child.isOrphanedServerOAD())
	    index = childCount;
	else
	{
	    ServiceObject sibling = null;
	    do
	    {
		index++;
		if(index >= childCount)
		    break;
		sibling = (ServiceObject)parent.getChildAt(index);
	    }
	    while(child.getName().compareTo(sibling.getName()) > 0);
	}
	
        ((DefaultTreeModel)getModel()).insertNodeInto(child, parent, index);
        if(display) 
            scrollPathToVisible(new TreePath(child.getPath()));
    }

    private Vector
    update(ServiceObject parent, String names[], int childType)
    {
        ServiceObject child = null;
	Vector updateList = new Vector();
        int childCount = names.length;
	int childIndex = 0;
        String currentName = "";
	DefaultTreeModel model = (DefaultTreeModel)getModel();

	model.nodeChanged(parent);
	
        for(int i = 0; i < childCount; i++)
        {
            currentName = names[i];

	    if(childIndex >= parent.getChildCount())
	    {
		child =
		    serviceMgr_.createObject(currentName, childType, parent);
                connectUp(parent, child, false);
		updateList.addElement(child);
	    }
	    else
	    {
		boolean currentIsOrphanedServerOAD =
		    childType == ServiceObject.OAD &&
		    currentName.equals(ServiceObject.ORPHANED_SERVER_OAD_NAME);
                child = (ServiceObject)parent.getChildAt(childIndex);
	        if((currentIsOrphanedServerOAD &&
		    !child.isOrphanedServerOAD()) ||
		   currentName.compareTo(child.getName()) > 0)
                {
                    model.removeNodeFromParent(child);
		    childIndex--;
                    i--;
                }
                else if((!currentIsOrphanedServerOAD &&
			 child.isOrphanedServerOAD()) ||
			currentName.compareTo(child.getName()) < 0)
                {
		    child = serviceMgr_.createObject(currentName,
			                             childType,
						     parent);
                    connectUp(parent, child, false);
	            updateList.addElement(child);
                }
	        else
	            updateList.addElement(child);
	    }

	    childIndex++;
        }

	childCount = parent.getChildCount();
	for(; childIndex < childCount; childIndex++)
	    model.removeNodeFromParent(
		(ServiceObject)parent.getChildAt(childIndex));
	
	return updateList;
    }

    private Enumeration
    getAllObjects()
    {
	return daObj_.depthFirstEnumeration();
    }

    private void
    refresh(TreePath path)
    {
	try
	{
	    if(path == null)
		refreshDomain();
	    else
	    {
		ServiceObject object =
		    (ServiceObject)path.getLastPathComponent();
		    
		if(object.isType(ServiceObject.DOMAIN))
		    refreshDomain();
		else if(object.isType(ServiceObject.OAD))
		    refreshOAD(object);
		else if(object.isType(ServiceObject.SERVER))
		    refreshServer(object);
	    }
	}
	catch(UnfoundObjectReference ex)
	{
            ErrorDialog.showUnfoundObjectReference(ex);
	}
    }

    private void
    refreshDomain() throws UnfoundObjectReference
    {
	daObj_.toObject();

	Vector updateList = update(daObj_,
			    serviceMgr_.getOADNames(),
			    ServiceObject.OAD);
	DefaultTreeModel model = (DefaultTreeModel)getModel();
	if(isExpanded(new TreePath(model.getPathToRoot(daObj_))))
	    for(int i = 0; i < updateList.size(); i++)
		refreshOAD((ServiceObject)updateList.elementAt(i));
    }
}
