// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************
 
package naming;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import java.io.*;

public class Server
{
    private static void
    usage()
    {
	System.out.println(
	    "Usage: java naming.Server [options]\n\n" +
	    "Options:\n" +
	    "-r repetitions     Number of repetitions.\n" +
	    "-h, --help         Display this help message.");

        System.exit(1);
    }

    static int
    run(ORB orb, String args[])
    {
        //
        // Parse command line arguments
        //
        int count = 100;
        int argc = 0;
        while(argc < args.length)
        {
	    if(args[argc].equals("-r"))
	    {
	        if(argc + 1 < args.length)
	        {
		    argc++;
                    try
                    {
                        count = Integer.parseInt(args[argc]);
                    }
                    catch(NumberFormatException e)
                    {
                    }
	        }
	        else
		    usage();
	    }
	    else if(args[argc].equals("-h") || args[argc].equals("--help"))
	        usage();
		
	    argc++;
        }
	    
        //
        // Resolve Root POA
        //
        org.omg.CORBA.Object poaObj = null;
        try
        {
            poaObj = orb.resolve_initial_references("RootPOA");
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            throw new RuntimeException();
        }

        POA rootPOA = POAHelper.narrow(poaObj);

        //
	// Get a reference to the POA manager
        //
        POAManager manager = rootPOA.the_POAManager();

        //
        // Create some implementations
        //
        Named_impl implA = new Named_impl();
        Named a = implA._this(orb);
	    
        //
        // Get naming service
        //
        org.omg.CORBA.Object obj = null;
    
        try
        {
            obj = orb.resolve_initial_references("NameService");
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
            System.out.println("Can't resolve `NameService'");
            return 1;
        }

        if(obj == null)
        {
	    System.out.println("`NameService' is a nil object reference");
	    return 1;
        }

        NamingContext nc = null;
        try
        {
            nc = NamingContextHelper.narrow(obj);
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
	    System.out.println("`NameService' is not " +
			       "a NamingContext object reference");
	    return 1;
        }

        NameComponent[] nc1Name = new NameComponent[1];
        nc1Name[0] = new NameComponent();
        NameComponent[] nc2Name = new NameComponent[2];
        nc2Name[0] = new NameComponent();
        nc2Name[1] = new NameComponent();
        NameComponent[] nc3Name = new NameComponent[3];
        nc3Name[0] = new NameComponent();
        nc3Name[1] = new NameComponent();
        nc3Name[2] = new NameComponent();
        long tv;
        try
        {
            System.out.println("Number of Iterations: " + count);           
            System.out.println();

	    //
	    // Bind names with the Naming Service
	    //
	    nc1Name[0].id = "Object";
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc1Name[0].kind = Integer.toString(i);
                nc.bind(nc1Name, a);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Bind in /: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");
            
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc1Name[0].kind = Integer.toString(i);
                nc.resolve(nc1Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Resolve in /: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");

            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc1Name[0].kind = Integer.toString(i);
                nc.unbind(nc1Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Unbind from /: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");

            System.out.println();
 
            nc1Name[0].id = "Context";
            nc1Name[0].kind = "";
            nc.bind_new_context(nc1Name);

	    nc2Name[0].id = "Context";
	    nc2Name[0].kind = "";
	    nc2Name[1].id = "Object";
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc2Name[1].kind = Integer.toString(i);
                nc.bind(nc2Name, a);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Bind in /Context: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");
            
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc2Name[1].kind = Integer.toString(i);
                nc.resolve(nc2Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Resolve in /Context: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");

            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc2Name[1].kind = Integer.toString(i);
                nc.unbind(nc2Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Unbind from /Context: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");

            System.out.println();

	    nc2Name[0].id = "Context";
	    nc2Name[0].kind = "";
	    nc2Name[1].id = "Nested";
	    nc2Name[1].kind = "";
            nc.bind_new_context(nc2Name);

	    nc3Name[0].id = "Context";
	    nc3Name[0].kind = "";
	    nc3Name[1].id = "Nested";
	    nc3Name[1].kind = "";
	    nc3Name[2].id = "Object";
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc3Name[2].kind = Integer.toString(i);
                nc.bind(nc3Name, a);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Bind in /Context/Nested: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");
            
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc3Name[2].kind = Integer.toString(i);
                nc.resolve(nc3Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Resolve in /Context/Nested: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");

            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc3Name[2].kind = Integer.toString(i);
                nc.unbind(nc3Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Unbind from /Context/Nested: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");

            nc.unbind(nc2Name);
            nc.unbind(nc1Name);

            System.out.println();

	    //
	    // Create and bind some Naming Contexts
	    //
            nc1Name[0].id = "Context";
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc1Name[0].kind = Integer.toString(i);
                nc.bind_new_context(nc1Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Bind new context in /: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");

            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc1Name[0].kind = Integer.toString(i);
                nc.unbind(nc1Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Unbind context from /: " + tv + " ms (" +
                               tv / (1.0 * count) + " ms/binding)");

            System.out.println();

            nc1Name[0].id = "Context";
            nc1Name[0].kind = "";
            nc.bind_new_context(nc1Name);

	    nc2Name[0].id = "Context";
	    nc2Name[0].kind = "";
	    nc2Name[1].id = "NestedContext";
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc2Name[1].kind = Integer.toString(i);
                nc.bind_new_context(nc2Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Bind new context in /Context: " + tv +
                               " ms (" + tv / (1.0 * count) + " ms/binding)");

            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc2Name[1].kind = Integer.toString(i);
                nc.unbind(nc2Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Unbind context from /Context: " + tv +
                               " ms (" + tv / (1.0 * count) + " ms/binding)");

            nc.unbind(nc1Name);
            System.out.println();

            nc1Name[0].id = "Context";
            for (int i = 0; i < count; i++)
            {
                nc1Name[0].kind = Integer.toString(i);
                nc.bind_new_context(nc1Name);
            }

            nc2Name[0].id = "Context";
            nc2Name[1].id = "NestedContext";            
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc2Name[0].kind = Integer.toString(i);
                nc2Name[1].kind = Integer.toString(i);
                nc.bind_new_context(nc2Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Bind new context in /Context#: " + tv +
                               " ms (" + tv / (1.0 * count) + " ms/binding)");
            
            tv = System.currentTimeMillis();
            for (int i = 0; i < count; i++)
            {
                nc2Name[0].kind = Integer.toString(i);
                nc2Name[1].kind = Integer.toString(i);
                nc.unbind(nc2Name);
            }
            tv = System.currentTimeMillis() - tv;
            System.out.println("Unbind context from /Context#: " + tv + 
                               " ms (" + tv / (1.0 * count) + " ms/binding)");
            
            nc1Name[0].id = "Context";
            for (int i = 0; i < count; i++)
            {
                nc1Name[0].kind = Integer.toString(i);
                nc.unbind(nc1Name);
            }
            System.out.println();
        }
        catch(NotFound ex)
        {
	    System.err.print("Got a `NotFound' exception (");
	    switch(ex.why.value())
	    {
	    case NotFoundReason._missing_node:
	        System.err.print("missing node");
	        break;

	    case NotFoundReason._not_context:
	        System.err.print("not context");
	        break;
    
	    case NotFoundReason._not_object:
	        System.err.print("not object");
	        break;
	    }
	    System.err.println(")");
	    ex.printStackTrace();
	    return 1;
        }
        catch(CannotProceed ex)
        {
	    System.err.println("Got a `CannotProceed' exception");
	    ex.printStackTrace();
	    return 1;
        }
        catch(InvalidName ex)
        {
	    System.err.println("Got an `InvalidName' exception");
	    ex.printStackTrace();
	    return 1;
        }
        catch(AlreadyBound ex)
        {
	    System.err.println("Got an `AlreadyBound' exception");
	    ex.printStackTrace();
	    return 1;
        }

	return 0;
    }

    public static void
    main(String args[])
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	ORB orb = null;
	
	try
	{
	    //
	    // Create ORB
	    //
	    orb = ORB.init(args, props);
	    status = run(orb, args);
        }
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}

	if(orb != null)
	{
	    try
	    {
	        orb.destroy();
	    }
	    catch(Exception ex)
	    {
	        System.err.println(ex.getMessage());
		ex.printStackTrace();
		status = 1;
	    }
	}

        System.exit(status);
    }
}
