// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

//
// This implementation supports persistence as well as the
// extended naming service with timestamps and dynamic updates.
//

package com.ooc.CosNaming;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.CosNaming.OBNamingContextPackage.*;

final public class OBNamingContext_impl extends OBNamingContextPOA
{
    static private int count_; // Running static count for keys.

    private POA userPOA_;
    private POA systemPOA_;
    private POA rootPOA_;
    private ORB orb_;
    private byte[] key_;

    private Policy[] timeoutPolicy_; // Timeout for callback

    private java.util.Hashtable bindings_; // Set of bindings in this context.
    private java.util.Hashtable ncs_; // Global set of naming contexts.
    private NamingDatabase store_; // The database.
    private int maxBindings_; // The maximum number of bindings for list()
    private boolean noUpdates_; // No automatic updates?
    private boolean destroyed_; // Has the context been destroyed?

    //
    // Clients to which this service reports events.
    //
    private java.util.Vector listeners_ = new java.util.Vector();

    private NamingContext myRef_; // Object-reference to myself

    private com.ooc.OB.Logger logger_; // Tracing information
    private int traceLevel_;

    private boolean isRoot_; // Is this the Root Context?

    //
    // InternalBinding
    //
    private static class InternalBinding
    {
	private BindingUnion bndUnion_;
	private NameComponent comp_;
	private long timestamp_;

        InternalBinding(NameComponent nc,
                       BindingUnion bndUnion,
                       long timestamp)
        {
            bndUnion_ = bndUnion;
            comp_ = nc;
            timestamp_ = timestamp;
        }
                       
        synchronized Binding
        toBinding()
        {
            Binding b = new Binding();

            b.binding_name = new NameComponent[1];
            b.binding_name[0] = new NameComponent(comp_.id, comp_.kind);
            b.binding_type = bndUnion_.discriminator();

            return b;
        }

        synchronized ExtendedBinding
        toExtBinding()
        {
            ExtendedBinding b = new ExtendedBinding();

            b.binding_name = new NameComponent[1];
            b.binding_name[0] = new NameComponent(comp_.id, comp_.kind);
            b.bound_obj = bndUnion_;
            b.timestamp = (int)timestamp_;

            return b;
        }

        synchronized void
        rename(NameComponent nc)
        {
            comp_.id = nc.id;
            comp_.kind = nc.kind;
	    timestamp_ = System.currentTimeMillis() / 1000;
        }

        synchronized BindingUnion
        bindingUnion()
        {
            return bndUnion_;
        }

        synchronized void
        rebind(NameComponent[] n, BindingUnion bndUnion, long timestamp)
            throws NotFound
        {
            //
            // Contexts must be rebound to contexts, objects
            // to objects.
            //
            if(bndUnion.discriminator() != bndUnion_.discriminator())
            {
                //
                // INS (98-10-11) 4-12:
                //
                // If rebind or rebind_context raise a
                // NotFound exception because an already
                // existing binding is of the wrong type, the
                // rest_of_name member of the exception has a
                // sequence length of 1.
                //
                if(bndUnion_.discriminator() == BindingType.ncontext)
                    throw new NotFound(NotFoundReason.not_object, n);
                else
                    throw new NotFound(NotFoundReason.not_context, n);
            }
            
            //
            // Update the binding information.
            //
            comp_ = n[0];
            bndUnion_ = bndUnion;
            timestamp_ = timestamp;
        }

        synchronized boolean
        isContext()
        {
            return bndUnion_.discriminator() == BindingType.ncontext;
        }

        synchronized org.omg.CORBA.Object
        object()
        {
            if(bndUnion_.discriminator() == BindingType.ncontext)
                return bndUnion_.nc();
	    else
		return bndUnion_.obj();
        }

        synchronized org.omg.CosNaming.NamingContext
        context(NameComponent[] n)
            throws NotFound
        {
	    //
	    // Binding must belong to a context.
	    //
	    if(bndUnion_.discriminator() != BindingType.ncontext)
		throw new NotFound(NotFoundReason.not_context, n);
            return bndUnion_.nc();
        }
    }

    //
    // This is used to create a unique name for a NameComponent for
    // use in the hashtable. We use the same algorithm as the
    // to_string method except we are only interested in the first
    // component.
    //
    private String
    bindingHashString(NameComponent[] name)
    {
	String result = "";
	result += addEscapes(name[0].id);
	if(name[0].kind.length() > 0)
	{
	    result += '.';
	    result += addEscapes(name[0].kind);
	}

	return result;
    }

    private void
    cleanup()
    {
	//
	// Remove ourself from the NamingContextSet. This will stop the
	// naming database from dumping this context.
	//
	com.ooc.OB.ObjectIdHasher key = new com.ooc.OB.ObjectIdHasher(key_);

	if(ncs_ != null)
	    ncs_.remove(key);

	if(store_ != null)
	    store_.destroy(key_);
	
	//
	// Deactivate the servant.
	//
        try
        {
	    if(isRoot_)
                userPOA_.deactivate_object(key_);
	    else
                systemPOA_.deactivate_object(key_);
        }
        catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
        {
            throw new RuntimeException();
        }
        catch(org.omg.PortableServer.POAPackage.ObjectNotActive ex)
        {
            throw new RuntimeException();
        }
    }
    
    private void
    doBind(NameComponent[] n, BindingUnion bndUnion, boolean rebind,
	   long timestamp)
	throws NotFound, AlreadyBound, CannotProceed, InvalidName
    {
	//
	// Check if name is valid.
	//
	if(n.length == 0)
	    throw new InvalidName();
	if(n[0].id.length() == 0 && n[0].kind.length() == 0)
	    throw new InvalidName();

        if(n.length == 1)
        {
	    //
	    // We have to synchronize on the bindings and the store_ here
	    // since we don't want the set of bindings changed in the
	    // middle of the operation. Note that we have to acquire the
	    // lock on the store_ before locking the binding set to avoid
	    // a deadlock.
	    //

	    //
	    // This deals with the fact that store_ may be null.
	    //
	    java.lang.Object syncStore;
	    if(store_ != null)
		syncStore = store_;
	    else
		syncStore = bindings_;

	    synchronized(syncStore)
	    {
            synchronized(bindings_)
            {
                InternalBinding binding =
                    (InternalBinding)bindings_.get(bindingHashString(n));
        
                if(binding != null)
                {
                    //
                    // Name is already bound.
                    //
                    if(!rebind)
                        throw new AlreadyBound();
                    
                    //
                    // Update the binding information.
                    //
                    binding.rebind(n, bndUnion, timestamp);
                }
                else
                {
                    //
                    // Bind the name and the object.
                    //
		    // We need to populate the binding information before
		    // we insert it in the binding list. Otherwise, it
		    // would be possible to resolve an incomplete binding.
		    //
                    bindings_.put(bindingHashString(n), 
                                  new InternalBinding(n[0], bndUnion,
						      timestamp));
                }
            
		if(store_ != null)
		    store_.bind(key_, n, bndUnion, timestamp, rebind);
            }
	    }

            return;
        }

        InternalBinding binding =
            (InternalBinding)bindings_.get(bindingHashString(n));
	if(binding == null)
	    //
	    // Missing node if binding was not found.
	    //
            throw new NotFound(NotFoundReason.missing_node, n);

        //
        // The context in which to bind.
        //
        NamingContext context = binding.context(n);
    
        //
        // Create the rest of the name.
        //
        NameComponent[] rest = new NameComponent[n.length - 1];
        System.arraycopy(n, 1, rest, 0, rest.length);
        
        //
        // Let the next context handle the binding.
        //
        if(bndUnion.discriminator() == BindingType.ncontext)
        {
            if(rebind)
                context.rebind_context(rest, bndUnion.nc());
            else
                context.bind_context(rest, bndUnion.nc());
        }
        else
        {
            if(rebind)
                context.rebind(rest, bndUnion.obj());
            else
                context.bind(rest, bndUnion.obj());
	}
    }

    //
    // Notify all listeners that a binding has been added.
    //
    private void
    bindingAdded(BindingUnion bndUnion, NameComponent[] n)
    {
	//
	// Check whether change occured in this context.
	//
	if(n.length != 1)
	    return;

	if(!noUpdates_) synchronized(listeners_)
	{
	    boolean isContext =
		(bndUnion.discriminator() == BindingType.ncontext);

	    if(traceLevel_ >= 3)
		logger_.trace("naming", isContext ?
			      "context binding added" :
			      "object binding added");
		
	    java.util.Enumeration e = listeners_.elements();
	    while(e.hasMoreElements())
	    {
		com.ooc.CosNaming.BindingListener l =
		    (com.ooc.CosNaming.BindingListener)e.nextElement();

		try
		{
		    if(isContext)
			l.context_added(myRef_, bndUnion.nc(), n[0]);
		    else
			l.object_added(myRef_, bndUnion.obj(), n[0]);
		}
		catch(SystemException ex)
		{
		    if(traceLevel_ >= 3)
			logger_.trace("naming",
				      "SystemException: removing listener");
		    listeners_.removeElement(l);
		}
	    }
	}
    }

    //
    // Notify all listeners that a binding has been removed.
    //
    private void
    bindingRemoved(NameComponent[] n, boolean isContext)
    {
	//
	// Check whether change occured in this context
	//
	if(n.length != 1)
	    return;

	if(!noUpdates_) synchronized(listeners_)
	{
	    if(traceLevel_ >= 3)
		logger_.trace("naming", isContext ?
			      "context binding removed" :
			      "object binding removed");
    
	    java.util.Enumeration e = listeners_.elements();
	    while(e.hasMoreElements())
	    {
		com.ooc.CosNaming.BindingListener l =
		    (com.ooc.CosNaming.BindingListener)e.nextElement();

		try
		{
		    if(isContext)
			l.context_removed(myRef_, n[0]);
		    else
			l.object_removed(myRef_, n[0]);
		}
		catch(SystemException ex)
		{
		    if(traceLevel_ >= 2)
			logger_.trace("naming",
				      "SystemException: removing listener");
		    listeners_.removeElement(l);
		}
	    }
	}
    }

    //
    // Notify all listeners that a binding has been changed.
    //
    private void
    bindingChanged(BindingUnion bndUnion, NameComponent[] oldName,
		   NameComponent[] newName)
    {
	//
	// Check whether change occured in this context
	//
	if(oldName.length != 1)
	    return;

	if(!noUpdates_) synchronized(listeners_)
	{
	    boolean isContext =
		(bndUnion.discriminator() == BindingType.ncontext);
	    if(traceLevel_ >= 3)
		logger_.trace("naming", isContext ?
			      "context binding changed" :
			      "object binding changed");

	    java.util.Enumeration e = listeners_.elements();
	    while(e.hasMoreElements())
	    {
		com.ooc.CosNaming.BindingListener l =
		    (com.ooc.CosNaming.BindingListener)e.nextElement();

		try
		{
		    if(isContext)
			l.context_changed(myRef_, bndUnion.nc(),
					  oldName[0], newName[0]);
		    else
			l.object_changed(myRef_, bndUnion.obj(),
					 oldName[0], newName[0]);
		}
		catch(SystemException ex)
		{
		    if(traceLevel_ >= 2)
			logger_.trace("naming",
				      "SystemException: removing listener");
		    listeners_.removeElement(l);
		}
	    }
	}
    }

    //
    // Add stringified-name escapes. The characters '.', '/' and '\\'
    // must be escaped with a '\\'.
    //
    private String
    addEscapes(String str)
    {
	String result = "";

	int len = str.length();
	for(int i = 0 ; i < len ; i++)
	{
	    char c = str.charAt(i);
	    switch(c)
	    {
	    case '.':
		result += "\\.";
		break;

	    case '/':
		result += "\\/";
		break;

	    case '\\':
		result += "\\\\";
		break;

	    default:
		result += c;
		break;
	    }
	}

	return result;
    }

    //
    // Escape characters according to URL encoding scheme
    // except for the characters listed in 4.6.3. We cannot use the
    // java.net.URLEncoder class to do this since it appears to
    // escape the '/' character.
    //
    static private String
    addURLEscapes(String s)
    {
	String result = "";
	for(int i = 0 ; i < s.length() ; i++)
	{
	    char ch = s.charAt(i);
	    //
	    // As per RFC 2396.
	    //
	    if(Character.isLetter(ch) || Character.isDigit(ch))
		result += ch;
	    else
	    {
		switch(ch)
		{
		case ';': case '/': case '?': case ':':
		case '@': case '&': case '=': case  '+':
		case '$': case ',': case '-': case '_':
		case '.': case '!': case '~': case '*':
		case '\'':case '(': case ')':
		    result += ch;
		    break;

		default:
		    result += '%';
		    result += Character.toUpperCase(
			Character.forDigit((ch >> 4) & 0xF, 16));
		    result += Character.toUpperCase(
			Character.forDigit(ch & 0xF, 16));
		}
	    }
	}

	return result;
    }

    OBNamingContext_impl
    createNewContext()
    {
	return new OBNamingContext_impl(userPOA_, systemPOA_, orb_, store_,
					ncs_, null, timeoutPolicy_, noUpdates_,
					maxBindings_, traceLevel_);
    }

    //
    // Constructor and destructor
    //
    OBNamingContext_impl(POA userPOA, POA systemPOA, ORB orb,
    			 NamingDatabase store, java.util.Hashtable ncs,
			 byte[] key, Policy[] timeoutPolicy, boolean noUpdates,
			 int maxBindings, int traceLevel)
    {
	userPOA_ = userPOA;
	systemPOA_ = systemPOA;
	orb_ = orb;
	key_ = key;
	bindings_ = new java.util.Hashtable(1023);
	ncs_ = ncs;
	store_ = store;
	timeoutPolicy_ = timeoutPolicy;
	noUpdates_ = noUpdates;
	destroyed_ = false;
	maxBindings_ = maxBindings;
	traceLevel_ = traceLevel;
	isRoot_ = false;

	com.ooc.OBCORBA.ORB_impl oborb = (com.ooc.OBCORBA.ORB_impl)orb;
	logger_ = oborb.logger();

	try
	{
	    rootPOA_ = POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	    throw new RuntimeException();
	}
	
	if(key != null)
	{
	    byte[] rootKey = ("NameService").getBytes();
	    if(key_.length == rootKey.length)
	    {
	        int idx;
	        for(idx = 0; idx < key_.length; ++idx)
		    if(key_[idx] != rootKey[idx])
		        break;
	        isRoot_ = (idx == key_.length);
	    }
	}

	try
	{
	    if(key == null)
	        key_ = systemPOA_.activate_object(this);
	    else
	    {
	        key_ = key;
	        if(isRoot_)
	            userPOA_.activate_object_with_id(key_, this);
	        else
	            systemPOA_.activate_object_with_id(key_, this);
	    }
	}
	catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
	{
	    throw new RuntimeException();
	}
	catch(org.omg.PortableServer.POAPackage.ObjectAlreadyActive ex)
	{
	    throw new RuntimeException();
	}
	catch(org.omg.PortableServer.POAPackage.ServantAlreadyActive ex)
	{
	    throw new RuntimeException();
	}

	myRef_ = _this();
	com.ooc.OB.ObjectIdHasher oid = new com.ooc.OB.ObjectIdHasher(key_);
	ncs_.put(oid, myRef_);

	if(store_ != null)
	    store_.create(key_, (int)(System.currentTimeMillis() / 1000));
    }

    protected void
    finalize()
        throws Throwable
    {
	if(ncs_ != null)
	{
	    com.ooc.OB.ObjectIdHasher oid = new com.ooc.OB.ObjectIdHasher(key_);
	    ncs_.remove(oid);
	}
        super.finalize();
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public org.omg.PortableServer.POA
    _default_POA()
    {
        if(isRoot_)
            return userPOA_;
	else
            return systemPOA_;
    }

    //
    // Implementation of CosNaming::NamingContext.
    //

    public void
    bind(NameComponent[] n, org.omg.CORBA.Object obj)
        throws NotFound, CannotProceed, InvalidName, AlreadyBound
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	BindingUnion bndUnion = new BindingUnion();
	bndUnion.obj(obj);

	doBind(n, bndUnion, false, System.currentTimeMillis() / 1000);

	bindingAdded(bndUnion, n);
    }

    public void
    rebind(NameComponent[] n, org.omg.CORBA.Object obj)
        throws NotFound, CannotProceed, InvalidName
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	BindingUnion bndUnion = new BindingUnion();
	bndUnion.obj(obj);

	try
	{

	    doBind(n, bndUnion, true, System.currentTimeMillis() / 1000);
	}
	catch(AlreadyBound ex)
	{
	    //
	    // AlreadyBound must not be thrown here, so convert it.  This
	    // shouldn't happen -- therefore an application error.
	    //
	    com.ooc.OB.Assert._OB_assert(false);
	}

	bindingChanged(bndUnion, n, n);
    }

    public void
    bind_context(NameComponent[] n, org.omg.CosNaming.NamingContext nc)
        throws NotFound, CannotProceed, InvalidName, AlreadyBound
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	if(nc == null)
	    throw new BAD_PARAM();

	BindingUnion bndUnion = new BindingUnion();
	bndUnion.nc(nc);

	doBind(n, bndUnion, false, System.currentTimeMillis() / 1000);
 
	bindingAdded(bndUnion, n);
    }

    public void
    rebind_context(NameComponent[] n, org.omg.CosNaming.NamingContext nc)
        throws NotFound, CannotProceed, InvalidName
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	if(nc == null)
	    throw new BAD_PARAM();

	BindingUnion bndUnion = new BindingUnion();
	bndUnion.nc(nc);

	try
	{
	    doBind(n, bndUnion, true, System.currentTimeMillis() / 1000);
	}
	catch(AlreadyBound ex)
	{
	    //
	    // AlreadyBound must not be thrown here, so convert it.
	    // This shouldn't happen -- therefore an application
	    // error.
	    //
	    com.ooc.OB.Assert._OB_assert(false);
	}

	bindingChanged(bndUnion, n, n);
    }

    public org.omg.CORBA.Object
    resolve(NameComponent[] n)
        throws NotFound, CannotProceed, InvalidName
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	//
	// Check if name is valid.
	//
	if(n.length == 0)
	    throw new InvalidName();
	if(n[0].id.length() == 0 && n[0].kind.length() == 0)
	    throw new InvalidName();

        InternalBinding binding =
            (InternalBinding)bindings_.get(bindingHashString(n));
         if(binding == null)       
	     //
	     // Missing node if binding was not found.
	     //
            throw new NotFound(NotFoundReason.missing_node, n);

	if(n.length == 1)
	    //
	    // Object or context found, return it.
	    //
            return binding.object();

        //
        // Context in which to resolve.
        //
        NamingContext context = binding.context(n);

	//
	// Create the rest of the name.
	//
	NameComponent[] rest = new NameComponent[n.length - 1];
	System.arraycopy(n, 1, rest, 0, rest.length);

	//
	// Have the context handle the resolving of the name.
	//
	return context.resolve(rest);
    }

    public void
    unbind(NameComponent[] n)
        throws NotFound, CannotProceed, InvalidName
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	//
	// Check if name is valid.
	//
	if(n.length == 0)
	    throw new InvalidName();
	if(n[0].id.length() == 0 && n[0].kind.length() == 0)
	    throw new InvalidName();

        InternalBinding binding;

        if(n.length == 1)
        {
	    boolean isContext = false;

	    //
	    // We have to synchronize on the bindings and the store_ here
	    // since we don't want the set of bindings changed in the
	    // middle of the operation. Note that we have to acquire the
	    // lock on the store_ before locking the binding set to avoid
	    // a deadlock.
	    //

	    //
	    // This deals with the fact that store_ may be null.
	    //
	    java.lang.Object syncStore;
	    if(store_ != null)
		syncStore = store_;
	    else
		syncStore = bindings_;

	    synchronized(syncStore)
	    {
            synchronized(bindings_)
            {
                binding = (InternalBinding)bindings_.get(bindingHashString(n));
		if(binding == null)             
		    //
		    // Missing node if binding was not found.
		    //
                    throw new NotFound(NotFoundReason.missing_node, n);
                
                //
                // Do unbind.
                //
                bindings_.remove(bindingHashString(n));
            
		if(store_ != null)
		    store_.unbind(key_, n);

		isContext = binding.isContext();
	    }
	    }
            
            bindingRemoved(n, isContext);
            return;
        }

        binding = (InternalBinding)bindings_.get(bindingHashString(n));
	if(binding == null)       
	    //
	    // Missing node if binding was not found.
	    //
            throw new NotFound(NotFoundReason.missing_node, n);
        
        //
        // The context in which to unbind.
        //
        NamingContext context = binding.context(n);
        
        //
        // Create the rest of the name.
        //
        NameComponent[] rest = new NameComponent[n.length - 1];
        System.arraycopy(n, 1, rest, 0, rest.length);
        
        //
        // Let the next context handle the resolving of the name.
        //
        context.unbind(rest);
    }

    public org.omg.CosNaming.NamingContext
    new_context()
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	OBNamingContext_impl nc = createNewContext();
	return nc._this();
    }

    public org.omg.CosNaming.NamingContext
    bind_new_context(NameComponent[] n)
        throws NotFound, AlreadyBound, CannotProceed, InvalidName
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	OBNamingContext_impl newContext = createNewContext();
	NamingContext nc = newContext._this();

	try
	{
	    bind_context(n, nc);
	}
	catch(SystemException ex)
	{
	    if(ex.completed == org.omg.CORBA.CompletionStatus.COMPLETED_NO)
	    {
		newContext.cleanup();

		throw ex;
	    }
	}
	catch(NotFound ex)
	{
	    newContext.cleanup();
	    throw ex;
	}
	catch(CannotProceed ex)
	{
	    newContext.cleanup();
	    throw ex;
	}
	catch(InvalidName ex)
	{
	    newContext.cleanup();
	    throw ex;
	}
	catch(AlreadyBound ex)
	{
	    newContext.cleanup();
	    throw ex;
	}

	return nc;
    }

    public void
    destroy()
        throws NotEmpty
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	if(!bindings_.isEmpty())
	    throw new NotEmpty();

	destroyed_ = true;

	cleanup();
    }

    public void
    list(int how_many, BindingListHolder bl, BindingIteratorHolder bi)
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	//
	// Note that we have to be careful in this method. Since we don't
	// synchronize the set of bindings_ it's possible that the set of
	// bindings is altered during processing.
	//
	java.util.Enumeration keys = bindings_.elements();

	java.util.Vector blvec = new java.util.Vector();

	//
	// Limit the maximum number of bindings, if requested
	//
	if(how_many > maxBindings_)
	    how_many = maxBindings_;

	//
	// Copy the first how_many bindings into bl, taking into
	// account that some bindings may not be present. We may not
	// use the calculated `num' above since some bindings may no
	// longer be present.
	//
	while(keys.hasMoreElements() && blvec.size() < how_many)
	{
	    InternalBinding binding = (InternalBinding)keys.nextElement();
	    if(binding != null)
                blvec.addElement(binding.toBinding());
	}
	bl.value = new Binding[blvec.size()];
        blvec.copyInto(bl.value);
	
	//
	// If we have more bindings then create a BindingIterator with the
	// remainder of the existing bindings, otherwise we return a nil
	// BindingIterator.
	//
	blvec.removeAllElements();
	while(keys.hasMoreElements())
	{
	    InternalBinding binding = (InternalBinding)keys.nextElement();
	    if(binding != null)
                blvec.addElement(binding.toBinding());
	}
	
	if(blvec.size() > 0)
	{
	    Binding[] rest = new Binding[blvec.size()];
            blvec.copyInto(rest);

	    BindingIterator_impl iteratorImpl =
		new BindingIterator_impl(rootPOA_, rest);
	    bi.value = iteratorImpl._this(orb_);
	}
	else
	{
	    bi.value = null;
	}
    }

    //
    // Implementation of CosNaming::NamingContextExt
    //
    public String
    to_string(NameComponent[] name)
    {
	String result = "";

	for(int i = 0 ; i < name.length ; i++)
	{
	    if(i != 0)
		result += '/';

	    result += addEscapes(name[i].id);
	    if(name[i].kind.length() > 0)
	    {
		result += '.';
		result += addEscapes(name[i].kind);
	    }
	}

	return result;
    }

    public NameComponent[]
    to_name(String s)
        throws InvalidName
    {
	if(s.length() == 0)
	    throw new InvalidName();

	com.ooc.OB.CORBANameParser parser = new com.ooc.OB.CORBANameParser(s);
	if(!parser.isValid())
	    throw new InvalidName();

	String[] contents = parser.getContents();

	NameComponent[] name = new NameComponent[contents.length / 2];
	int c = 0;
	for(int i = 0; i < contents.length; i += 2, c++)
	    name[c] = new NameComponent(contents[i], contents[i + 1]);

	return name;
    }

    public String
    to_url(String addr, String n)
    {
	String result = "corbaname:";

	result += addr;
	result += '#';

	result += addURLEscapes(n);

	return result;
    }

    public org.omg.CORBA.Object
    resolve_str(String n)
        throws NotFound, CannotProceed, InvalidName
    {
	return resolve(to_name(n));
    }

    //
    // Implementation of CosNaming::OBNamingContext
    //
    public ExtendedBinding[]
    list_extended()
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	java.util.Enumeration keys = bindings_.elements();

	java.util.Vector ebvec = new java.util.Vector();
	while(keys.hasMoreElements())
	{
	    InternalBinding binding = (InternalBinding)keys.nextElement();
	    if(binding != null)
                ebvec.addElement(binding.toExtBinding());
	}
	
	ExtendedBinding[] eb = new ExtendedBinding[ebvec.size()];
        ebvec.copyInto(eb);

	return eb;
    }

    public void
    rename(NameComponent[] oldName, NameComponent[] newName)
	throws NotFound, AlreadyBound, CannotProceed, InvalidName
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	//
	// Are the names invalid? (Why not pass NameComponents?)
	//
	if(oldName.length != 1 || newName.length != 1)
	    throw new InvalidName();

	if(newName[0].id.length() == 0 && newName[0].kind.length() == 0)
	    throw new InvalidName();

	//
	// If the names are the same then we have nothing to do.
	//
	if(oldName[0].id.equals(newName[0].id) &&
	   oldName[0].kind.equals(newName[0].kind))
	    return;

	BindingUnion bndUnion;

	//
	// We have to synchronize on the bindings and the store_ here
	// since we don't want the set of bindings changed in the middle
	// of the operation. Note that we have to acquire the lock on the
	// store_ before locking the binding set to avoid a deadlock.
	//

	//
	// This deals with the fact that store_ may be null.
	//
	java.lang.Object syncStore;
	if(store_ != null)
	    syncStore = store_;
	else
	    syncStore = bindings_;

	synchronized(syncStore)
	{
	synchronized(bindings_)
	{
            //
            // Locking the binding itself is not necessary below since the
            // bindings_ set is locked for the entire time that we have
            // the binding, and it's not possible to acquire a reference
            // to a binding without locking the bindings.
            //
            // Get the current binding.
            //
            InternalBinding binding = (InternalBinding)bindings_.get(
		bindingHashString(oldName));

	    if(binding == null)
		throw new NotFound(NotFoundReason.missing_node, oldName);

	    //
	    // Check if newName already exists
	    //
	    if(bindings_.containsKey(bindingHashString(newName)))
		throw new AlreadyBound();

	    //
	    // Remove the current, add the new.
	    //
	    bindings_.remove(bindingHashString(oldName));

            binding.rename(newName[0]);

	    bindings_.put(bindingHashString(newName), binding);

            bndUnion = binding.bindingUnion();

	    if(store_ != null)
	    {
		store_.unbind(key_, oldName);
		store_.bind(key_, newName, bndUnion,
			    System.currentTimeMillis() / 1000,
			    false);
	    }
	}
	}

	bindingChanged(bndUnion, oldName, newName);
    }

    public boolean
    add_listener(com.ooc.CosNaming.BindingListener listener)
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	if(!noUpdates_) synchronized(listeners_)
	{
	    //
	    // Set timeeut policy before using _is_equivalent() since
	    // _is_equivalent() takes policies into account
	    //
	    org.omg.CORBA.Object obj = listener._set_policy_override(
		timeoutPolicy_, org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
	    listener = BindingListenerHelper.narrow(obj);

	    //
	    // Check if listener is already registered.
	    //
	    java.util.Enumeration e = listeners_.elements();
	    while(e.hasMoreElements())
	    {
		com.ooc.CosNaming.BindingListener l =
		    (com.ooc.CosNaming.BindingListener)e.nextElement();
		if(listener._is_equivalent(l))
		    return true;
	    }

	    if(traceLevel_ >= 2)
		logger_.trace("naming", "adding listener");
	    listeners_.addElement(listener);
	    return true;
	}

	return false;
    }

    public boolean
    remove_listener(com.ooc.CosNaming.BindingListener listener)
    {
	if(destroyed_)
	    throw new OBJECT_NOT_EXIST();

	if(!noUpdates_) synchronized(listeners_)
	{
	    //
	    // Set timeeut policy before using _is_equivalent() since
	    // _is_equivalent() takes policies into account
	    //
	    org.omg.CORBA.Object obj = listener._set_policy_override(
		timeoutPolicy_, org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);
	    listener = BindingListenerHelper.narrow(obj);

	    java.util.Enumeration e = listeners_.elements();
	    while(e.hasMoreElements())
	    {
		com.ooc.CosNaming.BindingListener l =
		    (com.ooc.CosNaming.BindingListener)e.nextElement();
		if(listener._is_equivalent(l))
		{
		    if(traceLevel_ >= 2)
			logger_.trace("naming", "removing listener");
		    listeners_.removeElement(l);
		    return true;
		}
	    }
	}

	return false;
    }

    public boolean
    callbacks_active()
    {
	return !noUpdates_;
    }

    //
    // Internal methods.
    //

    byte[]
    _OB_getKey()
    {
	return key_;
    }

    void
    _OB_bindWithTimestamp(NameComponent[] n, BindingUnion bndUnion,
			  long timestamp)
	throws NotFound, AlreadyBound, CannotProceed, InvalidName
    {
	doBind(n, bndUnion, false, timestamp);
    }
}
