// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.CosNamingConsole.Util;

import java.io.*;
import java.util.*;
import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import com.ooc.CosNamingConsole.*;
import com.ooc.Util.*;
import javax.swing.text.html.parser.*;

final public class IO
{
    //
    // Load naming context
    //
    public static boolean
    load(ORB orb, File file, NamingContext nc)
	throws IOException
    {
	BufferedReader in = new BufferedReader(new FileReader(file));

	String str = in.readLine();
	while(str != null)
	{
	    if(str.startsWith("<obnamingcontext>"))
	    {
		load(orb, in, nc, true);
		in.close();

		return true;
	    }

	    str = in.readLine();
	}

	in.close();

	return false;
    }

    //
    // Load recursively
    //
    private static void
    load(ORB orb, BufferedReader in, NamingContext nc, boolean isRoot)
	throws IOException
    {
	do
	{
	    String tag, id = "", kind = "", ior = null;

	    tag = in.readLine();
	    if(tag == null)
		return;

	    if(tag.equals("</obnamingcontext>"))
		return;

	    if(!isRoot && tag.equals("</context>"))
		return;

	    while(tag != null)
	    {
		if(tag.equals("<context>") || tag.equals("<object>"))
		    break;

		tag = in.readLine();
	    }
	    if(tag == null)
		return;

	    String startTag = peekNextElement(in);
	    if(startTag == null)
		return;

	    while(!startTag.startsWith("/"))
	    {
		if(startTag.equals("object") || startTag.equals("context"))
		{
		    break;
		}
		else if(startTag.equals("id"))
		{
		    id = readNextElement(in);
		}
		else if(startTag.equals("kind"))
		{
		    kind = readNextElement(in);
		}
		else if(startTag.equals("ior"))
		{
		    ior = readNextElement(in);
		}
		//
		// Ignore unknown tags
		//
		else
		{
		    String endTag = "";
		    while(!endTag.equals("/" + startTag))
		    {
			endTag = in.readLine();
			if(endTag == null)
			    return;
		    }
		}

		startTag = peekNextElement(in);
		if(startTag == null)
		    return;
	    }

	    NameComponent[] name = new NameComponent[1];
	    name[0] = new NameComponent(id, kind);
	    org.omg.CORBA.Object obj = orb.string_to_object(ior);

	    try
	    {
		if(tag.equals("<context>"))
		{
		    NamingContext newContext = nc.bind_new_context(name);

		    load(orb, in, newContext, false);
		}
		else if(tag.equals("<object>"))
		{
		    nc.bind(name, obj);

		    //
		    // Skip </object> tag
		    //
		    in.readLine();
		}
	    }
	    catch(CannotProceed ex)
	    {
	    }
	    catch(InvalidName ex)
	    {
	    }
	    catch(NotFound ex)
	    {
	    }
	    catch(AlreadyBound ex)
	    {
	    }
	}
	while(true);
    }

    //
    // Save naming context
    //
    public static void
    save(ORB orb, File file, NamingContext nc, Binding binding,
	 MessageLogger logger)
	throws IOException
    {
	PrintWriter out = new PrintWriter(new FileOutputStream(file));

	//
	// Prolog
	//
	out.println("<?xml version=\"1.0\" ?>\n");

	out.println("<obnamingcontext>\n");
	save(orb, out, nc, binding, new Vector(), logger);
	out.println("\n</obnamingcontext>");

	out.close();
    }

    //
    // Save recursively
    //
    private static void
    save(ORB orb, PrintWriter out, org.omg.CORBA.Object ctx,
	 Binding binding, Vector previousContexts, MessageLogger logger)
    {
	BindingListHolder blh = new BindingListHolder();
	BindingIteratorHolder bih = new BindingIteratorHolder();

	try
	{
	    NamingContext nc = NamingContextHelper.narrow(ctx);
	    nc.list(Integer.MAX_VALUE, blh, bih);

	    for(int i = 0 ; i < blh.value.length ; i++)
	    {
		Binding b = blh.value[i];

		org.omg.CORBA.Object obj = null;
		try
		{
		    obj = nc.resolve(b.binding_name);
		}
		catch(InvalidName ex)
		{
		}
		catch(CannotProceed ex)
		{
		}
		catch(NotFound ex)
		{
		}

		if(b.binding_type == BindingType.ncontext)
		    out.println("\n<context>");
		else
		    out.println("\n<object>");

		if(b.binding_name[0].id.length() != 0)
		    out.println("<id>\n" + b.binding_name[0].id + "\n</id>");
		if(b.binding_name[0].kind.length() != 0)
		    out.println("<kind>\n" + b.binding_name[0].kind +
				"\n</kind>");
		out.println("<ior>\n" + orb.object_to_string(obj) + "\n</ior>");

		//
		// Next level
		//
		if(b.binding_type == BindingType.ncontext)
		{
		    String s = orb.object_to_string(obj);
		    if(!previousContexts.contains(s))
		    {
			Vector contextVec = (Vector)previousContexts.clone();
			contextVec.addElement(s);
			save(orb, out, obj, b, contextVec, logger);
		    }
		    else
		    {
			logger.println("Recursion detected at `" +
				       BindingUtil.getFullName(b) + "'");
		    }

		    out.println("</context>");
		}
		else
		    out.println("</object>");
	    }
	}
	catch(COMM_FAILURE ex)
	{
	    logger.println("Couldn't access NamingContext `" +
			   BindingUtil.getFullName(binding) + "'");
	}

	if(bih.value != null)
	    bih.value.destroy();
    }

    private static String
    peekNextElement(BufferedReader in)
	throws IOException
    {
	String tag = "";
	while(tag.equals(""))
	{
	    in.mark(1000);

	    tag = in.readLine();
	    if(tag == null)
		return null;
	}

	in.reset();

	return tag.substring(1, tag.length() - 1);
    }

    private static String
    readNextElement(BufferedReader in)
	throws IOException
    {
	String start = in.readLine();
	if(start == null)
	    return null;

	String content = in.readLine();
	if(content == null)
	    return null;

	String end = in.readLine();
	if(end == null)
	    return null;

	return content;
    }
}
