// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package benchmark;

import benchmark.Benchmark.*;

public class Server
{
    static void
    write_reference(String refFile, String ref)
    {
        try
        {
            java.io.FileOutputStream file =
                new java.io.FileOutputStream(refFile);
            java.io.PrintWriter out = new java.io.PrintWriter(file);
            out.println(ref);
            out.flush();
            file.close();
        }
        catch(java.io.IOException ex)
        {
            System.err.println("benchmark.Server: can't write to `" +
                               ex.getMessage() + "'");
        }
    }

    static int
    run(org.omg.CORBA.ORB orb)
        throws org.omg.CORBA.UserException
    {
        //
	// Resolve Root POA
	//
	org.omg.PortableServer.POA rootPOA =
	    org.omg.PortableServer.POAHelper.narrow(
	            orb.resolve_initial_references("RootPOA"));

	//
	// Get a reference to the POA manager
	//
	org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

	//
        // Create a servant for interface Benchmark::BasicTester.
        // 
        BasicTester_impl basicTesterImpl = new BasicTester_impl(orb, rootPOA);
	BasicTester basicTester = basicTesterImpl._this(orb);
	write_reference("Benchmark_BasicTester.ref",
			orb.object_to_string(basicTester));

	//
        // Create a servant for interface Benchmark::UserTypeTester.
        // 
	UserTypeTester_impl userTypeTesterImpl = 
	    new UserTypeTester_impl(orb, rootPOA);
	UserTypeTester userTypeTester = userTypeTesterImpl._this(orb);
	write_reference("Benchmark_UserTypeTester.ref",
			orb.object_to_string(userTypeTester));

	//
        // Create a servant for interface Benchmark::SequenceTester.
        // 
	SequenceTester_impl sequenceTesterImpl = 
	    new SequenceTester_impl(orb, rootPOA);
	SequenceTester sequenceTester = sequenceTesterImpl._this(orb);
	write_reference("Benchmark_SequenceTester.ref",
	                orb.object_to_string(sequenceTester));

	//
        // Activate the POA Manager and let the ORB process requests.
        //
        manager.activate();
        System.out.println("Waiting for requests...");
        orb.run();

	return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");
        props.put("ooc.orb.id", "Benchmark-Server");

        int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init(args, props);
            status = run(orb);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
