// ********************************************************************* 
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
// 
// All Rights Reserved
//
// **********************************************************************

class Controller_impl extends CCS.ControllerPOA
{
    // Set containing asset numbers of devices
    private java.util.Vector m_assets = new java.util.Vector();

    // POA for thermometers and thermostats
    private org.omg.PortableServer.POA m_poa = null;

    // Name of asset number file
    private String m_asset_file = null;

    public static final String         Controller_oid = "Controller";

    // Helper function to create object references.

    static public CCS.Thermometer
    make_dref(org.omg.PortableServer.POA poa, int anum)
    {
	// Convert asset number to OID.
	String s = Integer.toString(anum);
	byte [] oid = s.getBytes();
	
	// Look at the model via the network to determine
	// the repository ID.
	String buf = (String)ICP.get(anum, "model");
	if(buf == null)
	    throw new RuntimeException();
	String rep_id = buf.equals("Sens-A-Temp")
	    ? "IDL:acme.com/CCS/Thermometer:1.0"
	    : "IDL:acme.com/CCS/Thermostat:1.0";
	
	// Make a new reference.
	org.omg.CORBA.Object obj =
	    poa.create_reference_with_id(oid, rep_id);

	return CCS.ThermometerHelper.narrow(obj);
    }

    // IDL list operation.

    public CCS.Thermometer[] 
    list() 
    {
	// Create a new thermometer sequence. Because we know
	// the number of elements we will put onto the sequence,
	// we use the maximum constructor.
	CCS.Thermometer[] listv = new CCS.Thermometer[m_assets.size()];

        // Loop over the m_assets set and create a
        // reference for each device.
        for(int i = 0 ; i < m_assets.size() ; i++)
        {
	    Integer asset_element = (Integer)m_assets.elementAt(i);
            listv[i] = make_dref(m_poa, asset_element.intValue());
	}

        return listv;
    }

    // IDL find operation.

    public void 
    find(CCS.ControllerPackage.SearchSeqHolder slist)
    {
	java.util.Vector multiple_device_instance = new java.util.Vector();

	// Loop over input list and lookup each device.
	for(int i = 0 ; i < slist.value.length ; i++)
        {
	    // Assume we will not find a matching device.
	    slist.value[i].device = null;

	    // Work out whether we are searching by asset,
	    // model, or location.
	    CCS.ControllerPackage.SearchCriterion sc = 
	        slist.value[i].key.discriminator();
	    if(sc == CCS.ControllerPackage.SearchCriterion.ASSET)
            {
		// Search for matching asset number.
		Integer asset_num = 
		    new Integer(slist.value[i].key.asset_num());

		if(m_assets.contains(asset_num))
		    slist.value[i].device =
                        make_dref(m_poa, slist.value[i].key.asset_num());
	    }
            else
            {
		// Search for model or location string.
		String search_str = null;
		if(sc == CCS.ControllerPackage.SearchCriterion.LOCATION)
		    search_str = slist.value[i].key.loc();
		else if(sc == CCS.ControllerPackage.SearchCriterion.MODEL)
		    search_str = slist.value[i].key.model_desc();

		// Find the device with asset number=asset_num
		String buf = null;
		for(int j = 0 ; j < m_assets.size() ; j++)
                {
		    Integer asset_element = (Integer)m_assets.elementAt(j);
		    if(sc == CCS.ControllerPackage.SearchCriterion.LOCATION)
		        buf = (String)ICP.get(asset_element.intValue(), 
						   "location");

		    if(sc == CCS.ControllerPackage.SearchCriterion.MODEL)
		        buf = (String)ICP.get(asset_element.intValue(), 
						   "model");

		    if(buf != null)
                    {
			if(buf.equals(search_str))
                        {
			    if(i == 0)
                            {
				// First match overwrites reference
				// in search record.
				slist.value[i].device =
				    make_dref(m_poa, asset_element.intValue());
			    }
                            else
                            {
				// Further matches each append a new
				// element to the search sequence.
				CCS.ControllerPackage.SearchType searchSeqElt =
				    new CCS.ControllerPackage.SearchType();
				searchSeqElt.key = slist.value[i].key;
				searchSeqElt.device = 
				    make_dref(m_poa, asset_element.intValue());
				multiple_device_instance.addElement(
				    searchSeqElt);
			    }
			}
		    }
		}
	    }
	}

	// Append any multiple instances of the device in the
	// slist
	if(multiple_device_instance.size() != 0)
        {
	    // Create a new array 
	    int newArrSize = 
	        slist.value.length + multiple_device_instance.size();
	    CCS.ControllerPackage.SearchType[] search_seq = 
	        new CCS.ControllerPackage.SearchType[newArrSize];
	    System.arraycopy(slist.value, 0, search_seq, 0, 
			     slist.value.length);
	    for(int i = 0 ; i < multiple_device_instance.size() ; i++)
            {
	        search_seq[slist.value.length + i] = 
		    (CCS.ControllerPackage.SearchType)
		    multiple_device_instance.elementAt(i);
	    }

	    slist.value = search_seq;
	}
    }

    // IDL change operation.

    public void 
    change(CCS.Thermostat[] tlist, short delta) 
        throws CCS.ControllerPackage.EChange
    {
	java.util.Vector error_detail_list = new java.util.Vector();

	// We cannot add a delta value to a thermostat's temperature
	// directly, so for each thermostat, we read the nominal
	// temperature, add the delta value to it, and write
	// it back again.
	for(int i = 0 ; i < tlist.length ; i++)
        {
	    if(tlist[i] == null)
		continue;                       // Skip nil references

	    // Read nominal temp and update it.
	    short tnom = tlist[i].get_nominal();
	    tnom += delta;
	    try
            {
		tlist[i].set_nominal(tnom);
	    }
            catch(CCS.ThermostatPackage.BadTemp bt)
            {
		// If the update failed because the temperature
		// is out of range, we add the thermostat's info
		// to the errors sequence.
		CCS.ControllerPackage.ErrorDetails new_element = 
		    new CCS.ControllerPackage.ErrorDetails();
		new_element.tmstat_ref = tlist[i];
		new_element.info = bt.details;

		error_detail_list.addElement(new_element);
	    }
	}

	// If we encountered errors in the above loop,
	// we will have added elements to the errors sequence.
	if(error_detail_list.size() != 0)
        {
	    CCS.ControllerPackage.EChange ec =
	        new CCS.ControllerPackage.EChange();

	    ec.errors = 
	        new CCS.ControllerPackage.ErrorDetails[
		    error_detail_list.size()];

	    for(int i = 0 ; i < error_detail_list.size() ; i++)
		ec.errors[i] = 
		    (CCS.ControllerPackage.ErrorDetails)
		    error_detail_list.elementAt(i);

	    throw ec;
	}
    }

    public CCS.Thermometer 
    create_thermometer(int anum, String loc) 
        throws CCS.ControllerPackage.DuplicateAsset
    {
	if(anum % 2 == 0)               // Thermometers have odd numbers
	    throw new org.omg.CORBA.BAD_PARAM();   
	if(exists(anum))
	    throw new CCS.ControllerPackage.DuplicateAsset();

	if(ICP.online(anum) != 0)
	    throw new RuntimeException();
	if(ICP.set(anum, "location", loc) != 0)
	    throw new RuntimeException();
	add_impl(anum);

	return make_dref(m_poa, anum);
    }

    public CCS.Thermostat 
    create_thermostat(int anum, String loc, short Temp) 
        throws CCS.ControllerPackage.DuplicateAsset, 
	CCS.ThermostatPackage.BadTemp
    {
	if(anum % 2 != 0)    // Thermostats have even numbers
	    throw new org.omg.CORBA.BAD_PARAM();   
	if(exists(anum))
	    throw new CCS.ControllerPackage.DuplicateAsset();

	if(ICP.online(anum) != 0)
	    throw new RuntimeException();
	if(ICP.set(anum, "location", loc) != 0)
	    throw new RuntimeException();

	// Set the nominal temperature.
	Short temp = new Short(Temp);
	if(ICP.set(anum, "nominal_temp", temp) != 0)
        {
	    // If ICP_set() failed, read this thermostat's minimum
	    // and maximum so we can initialize the BadTemp exception.
	    CCS.ThermostatPackage.BtData btd =
                new CCS.ThermostatPackage.BtData();

	    Short permitted_vals = (Short)ICP.get( anum, "MIN_TEMP");
	    if(permitted_vals == null)
		throw new RuntimeException();
	    btd.min_permitted = permitted_vals.shortValue();
	    permitted_vals = (Short)ICP.get( anum, "MAX_TEMP");
	    if(permitted_vals == null)
		throw new RuntimeException();
	    btd.max_permitted = permitted_vals.shortValue();

	    btd.requested = temp.shortValue();
	    btd.error_msg = temp.shortValue() > btd.max_permitted ? 
		"Too hot" : "Too cold";
	    throw new CCS.ThermostatPackage.BadTemp(btd);
	}
	add_impl(anum);

	org.omg.CORBA.Object obj = make_dref(m_poa, anum);
	return CCS.ThermostatHelper.narrow(obj);
    }

    // Constructor

    Controller_impl(org.omg.PortableServer.POA poa, String asset_file) 
    {
	m_poa = poa;
	m_asset_file = asset_file;

	java.io.FileInputStream file = null;
	java.io.BufferedReader in = null;
	try
        {
	    file = new java.io.FileInputStream(m_asset_file);
	    in = new java.io.BufferedReader(
                new java.io.InputStreamReader(file));
        }
        catch(java.io.FileNotFoundException ex)
        {
            // File containing asset numbers is not available.
            return ;
        }
        catch(SecurityException ex)
        {
            ex.printStackTrace();
            throw new RuntimeException();
        }

	try
        {
	    // Read the list of asset numbers from the file 
	    // and put it in the set m_asset.
    	    String ref = null;
    	    while((ref = in.readLine()) != null)
            {
        	ref = ref.trim();
		Integer anum = new Integer(Integer.parseInt(ref));
	        m_assets.addElement(anum);
    	    }

	    file.close();
	}
	catch(java.io.IOException ex)
        {
	    System.err.println("Error reading file " + m_asset_file + 
			       " :" + ex.getMessage());
	    throw new RuntimeException();
	}
    }

    // Helper functions to allow access to the object map

    public void 
    add_impl(int anum)
    {
	Integer asset_num = new Integer(anum);
	if(!m_assets.contains(asset_num))
            m_assets.addElement(asset_num);
    }

    public void 
    remove_impl(int anum)
    {
	Integer asset_num = new Integer(anum);
        m_assets.removeElement(asset_num);
    }

    public boolean 
    exists(int anum)
    {
	Integer asset_num = new Integer(anum);
        return m_assets.contains(asset_num);
    }

    // Finalizer

    protected void
    finalize()
        throws Throwable
    {
	// Write out the current set of asset numbers
	// and clean up all servant instances.
	try
        {
	    java.io.FileOutputStream afile =
                new java.io.FileOutputStream(m_asset_file);
	    java.io.BufferedWriter out =
		new java.io.BufferedWriter(
                    new java.io.OutputStreamWriter(afile));

	    String anum_str = null;
	    Integer anum = null;
	    for(int i = 0 ; i < m_assets.size() ; i++)
            {
		anum = (Integer)m_assets.elementAt(i);
		anum_str = Integer.toString(anum.intValue());
	        out.write(anum_str, 0, anum_str.length());
	    }

	    afile.close();
	}
        catch(java.io.IOException ex)
        {
	    System.err.println("Error in handling file " +
			       m_asset_file + " :" + ex.getMessage());
	    throw new RuntimeException();
	}

        super.finalize();
    }
}
