// ********************************************************************* 
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
// 
// All Rights Reserved
//
// **********************************************************************

public class Server
{
    static org.omg.CosNaming.NamingContext
    resolve_init(org.omg.CORBA.ORB orb, String id)
        throws org.omg.CORBA.ORBPackage.InvalidName
    {
        org.omg.CORBA.Object obj = null;
        try
        {
            obj = orb.resolve_initial_references(id);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName e)
        {
            throw e;
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("Cannot get initial reference for " + id +
                               ": " + e);
            throw new RuntimeException();
        }
        if(obj == null)
            throw new RuntimeException();

        org.omg.CosNaming.NamingContext ref = null;
        try
        {
            ref = org.omg.CosNaming.NamingContextHelper.narrow(obj);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("Cannot narrow reference for " + id +
                               ": " + e);
            throw new RuntimeException();
        }
        if(ref == null)
        {
            System.err.println("Incorrect type of reference for " + id);
            throw new RuntimeException();
        }
        return ref;
    }

    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
	// Resolve Root POA
	org.omg.PortableServer.POA poa =
	    org.omg.PortableServer.POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));

	// Get a POA manager
	org.omg.PortableServer.POAManager poa_mgr = poa.the_POAManager();

	// Create a policy list. We use persistent objects with
	// user-assigned IDs, and explicit activation.
	org.omg.CORBA.Policy[] policy_list = new org.omg.CORBA.Policy[6];
	policy_list[0] = poa.create_lifespan_policy(
	    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);

	policy_list[1] = poa.create_id_assignment_policy(
	    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

	policy_list[2] = poa.create_implicit_activation_policy(
	    org.omg.PortableServer.ImplicitActivationPolicyValue.
	    NO_IMPLICIT_ACTIVATION);

	policy_list[3] = poa.create_request_processing_policy(
	    org.omg.PortableServer.RequestProcessingPolicyValue.
	    USE_SERVANT_MANAGER);

	policy_list[4] = poa.create_servant_retention_policy(
	    org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN);

	policy_list[5] = poa.create_thread_policy(
	    org.omg.PortableServer.ThreadPolicyValue.SINGLE_THREAD_MODEL);

	// Create a POA for CCS elements.
	org.omg.PortableServer.POA ccs_poa =
	    poa.create_POA("CCS_POA", poa_mgr, policy_list);

	// Create a controller and set static m_ctrl member
	// for thermostats and thermometers.
	Controller_impl ctrl_servant = 
	    new Controller_impl(ccs_poa, "/tmp/CCS_assets");
	Thermometer_impl.m_ctrl = ctrl_servant;
	Thermostat_impl.m_ctrl = ctrl_servant;

	// Create a reference for the controller and
	// create the corresponding CORBA object.
	byte [] oid = Controller_impl.Controller_oid.getBytes();
	org.omg.CORBA.Object ctrl =
	    ccs_poa.create_reference_with_id(
		oid, "IDL:acme.com/CCS/Controller:1.0");

	// Get reference to initial naming context.
	org.omg.CosNaming.NamingContext inc = resolve_init(orb, "NameService");

	// Attempt to create CCS context.
	org.omg.CosNaming.NameComponent[] n =
            new org.omg.CosNaming.NameComponent[1];
	n[0] = new org.omg.CosNaming.NameComponent();
	n[0].id = "CCS";
	n[0].kind = "";
	org.omg.CosNaming.NamingContext nc = null;
	try
        {
	    nc = inc.bind_new_context(n);
	}
	catch(org.omg.CosNaming.NamingContextPackage.AlreadyBound ex)
        { 
	    // Fine, CCS context already exists.
	}

	// Force binding of controller reference to make
	// sure it is always up-to-date.
	org.omg.CosNaming.NameComponent[] n_new =
            new org.omg.CosNaming.NameComponent[2];
	n_new[0] = n[0];
	n_new[1] = new org.omg.CosNaming.NameComponent();
	n_new[1].id = "Controller";
	n_new[1].kind = "";
        inc.rebind(n_new, ctrl);

	// Instantiate the servant locator for devices.
	DeviceLocator_impl my_locator = new DeviceLocator_impl(ctrl_servant);
	org.omg.PortableServer.ServantManager locator = my_locator._this(orb);

	// Set servant locator.
        ccs_poa.set_servant_manager(locator);

	// Activate the POA manager.
        poa_mgr.activate();

	// Accept requests.
	orb.run();

        return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
	    try
            {
		orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
