// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

public class Client
{
    static org.omg.CosTrading.Lookup
    resolve_init(org.omg.CORBA.ORB orb, String id)
        throws org.omg.CORBA.ORBPackage.InvalidName
    {
        org.omg.CORBA.Object obj = null;
        try
        {
            obj = orb.resolve_initial_references(id);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName e)
        {
            throw e;
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("Cannot get initial reference for " + id +
                               ": " + e);
            throw new RuntimeException();
        }
        if(obj == null)
            throw new RuntimeException();

        org.omg.CosTrading.Lookup ref = null;
        try
        {
            ref = org.omg.CosTrading.LookupHelper.narrow(obj);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("Cannot narrow reference for " + id +
                               ": " + e);
            throw new RuntimeException();
        }
        if(ref == null)
        {
            System.err.println("Incorrect type of reference for " + id);
            throw new RuntimeException();
        }
        return ref;
    }

    // Show the details for a thermometer or thermostat.

    static void
    print(java.io.PrintStream out, CCS.Thermometer t)
    {
        // Check for nil.
        if(t == null)
        {
            out.println("Cannot show state for nil reference.");
            return;
        }

        // Try to narrow and print what kind of device it is.
        CCS.Thermostat tmstat = null;
        try
        {
            tmstat = CCS.ThermostatHelper.narrow(t);
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
        }
        out.println((tmstat == null) ? "Thermometer:" : "Thermostat:");

        // Show attribute values.
        String model = t.model();
        String location = t.location();

        out.println("\tAsset number: " + t.asset_num());
        out.println("\tModel       : " + model);
        out.println("\tLocation    : " + location);
        out.println("\tTemperature : " + t.temperature());

        // If device is a thermostat, show nominal temperature.
        if(tmstat != null)
            System.out.println("\tNominal temp: " + tmstat.get_nominal());
    }

    // Show the information in a BtData struct.

    static void
    print(java.io.PrintStream out, CCS.ThermostatPackage.BtData btd)
    {
        out.println("CCS.Thermostat.BtData details:");
        out.println("\trequested    : " + btd.requested);
        out.println("\tmin_permitted: " + btd.min_permitted);
        out.println("\tmax_permitted: " + btd.max_permitted);
        out.println("\terror_msg    : " + btd.error_msg);
    }

    // Loop over the sequence of records in an EChange exception and
    // show the details of each record.

    static void
    print(java.io.PrintStream out, CCS.ControllerPackage.EChange ec)
    {
        for(int i = 0 ; i < ec.errors.length ; i++)
        {
            out.println("Change failed:");
            print(out, ec.errors[i].tmstat_ref);
            print(out, ec.errors[i].info);
        }
    }

    // Change the temperature of a thermostat

    static void
    set_temp(CCS.Thermostat tmstat, short new_temp)
    {
        if(tmstat == null) // Don't call via nil reference
            return;

        int anum = tmstat.asset_num();
        try
        {
            System.out.println("Setting thermostat " + anum + " to "
                               + new_temp + " degrees.");
            short old_nominal = tmstat.set_nominal(new_temp);
            System.out.println("Old nominal temperature was: " + old_nominal);
            System.out.println("New nominal temperature is: "
                               + tmstat.get_nominal());
        }
        catch(CCS.ThermostatPackage.BadTemp bt)
        {
            System.err.println("Setting of nominal temperature failed.");
            print(System.out, bt.details);
        }
    }

    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
	// Get Trading service
	org.omg.CosTrading.Lookup lookup = resolve_init(orb, "TradingService");

	// The policy sequence sets the return cardinality to 1
	// because we are interested only in a single offer.
	org.omg.CosTrading.Policy[] policies =
            new org.omg.CosTrading.Policy[1];
	policies[0] = new org.omg.CosTrading.Policy();
	policies[0].name = "return_card";
	policies[0].value = orb.create_any();
	policies[0].value.insert_ulong(1);

	// Don't return properties
	org.omg.CosTrading.LookupPackage.SpecifiedProps desired_props = 
	    new org.omg.CosTrading.LookupPackage.SpecifiedProps();
	desired_props.__default();

	org.omg.CosTrading.PolicyNameSeqHolder policies_applied =
	    new org.omg.CosTrading.PolicyNameSeqHolder();
	org.omg.CosTrading.OfferSeqHolder offers =
	    new org.omg.CosTrading.OfferSeqHolder();
	org.omg.CosTrading.OfferIteratorHolder iterator =
	    new org.omg.CosTrading.OfferIteratorHolder();

	// Run query without preferences using default policies.
        lookup.query(
            "CCS::Controllers", "Model == 'BFG-9000'", "",
            policies, desired_props, 1,
            offers, iterator, policies_applied);

	// Process results.
	CCS.Controller ctrl = null;
	if(offers.value.length == 0)
        {
	    System.err.println("Cannot locate matching controller.");
	    return 1;
	}
        else
        {
	    // Extract controller reference from returned offer.
	    try
            {
		ctrl = CCS.ControllerHelper.narrow(offers.value[0].reference);
	    }
	    catch(org.omg.CORBA.BAD_PARAM ex)
            {
		System.err.println("Service provider is not a controller!");
		return 1;
	    }
	}

        // Get list of devices
        CCS.Thermometer[] list = ctrl.list();

	// Show number of devices.
        int len = list.length;
        System.out.print("Controller has " + len + " device");
        if(len != 1)
            System.out.print("s");
        System.out.println(".");

        CCS.Thermometer t =
            ctrl.create_thermometer(27, "Room 1");
        CCS.Thermostat ts =
            ctrl.create_thermostat(28, "Room 2", (short)48);
        CCS.Thermostat ts2 =
            ctrl.create_thermostat(30, "Room 3", (short)48);
        CCS.Thermostat ts3 =
            ctrl.create_thermostat(32, "Room 3", (short)68);
        CCS.Thermostat ts4 =
            ctrl.create_thermostat(34, "Room 3", (short)68);
        CCS.Thermostat ts5 =
            ctrl.create_thermostat(36, "Room 3", (short)48);
        System.out.println(t.location());
        System.out.println(ts.location());
        System.out.println(ts2.location());
        t.remove();

        // Show details for each device.
        list = ctrl.list();
        for(int i = 0 ; i < list.length ; i++)
            print(System.out, list[i]);
        System.out.println();

        // Change the location of first device in the list
        int anum = list[0].asset_num();
        System.out.println("Changing location of device " + anum + ".");
        list[0].location("Earth");

        // Check that the location was updated
        System.out.println("New details for device " + anum + " are:");
        print(System.out, list[0]);

        // Find first thermostat in list.
        CCS.Thermostat tmstat = null;
        for(int i = 0 ; i < list.length && (tmstat == null) ; i++)
        {
            try
            {
                tmstat = CCS.ThermostatHelper.narrow(list[i]);
            }
            catch(org.omg.CORBA.BAD_PARAM ex)
            {
                // Fine, no thermostat was found.
            }
        }

        // Check that we found a thermostat on the list.
        if(tmstat == null)
        {
            System.out.println("No thermostat devices in list.");
        }
        else
        {
            // Set temperature of thermostat to
            // 50 degrees (should work).
            set_temp(tmstat, (short)50);
            System.out.println();

            // Set temperature of thermostat to
            // -10 degrees (should fail).
            set_temp(tmstat, (short)-10);
        }

        // Look for device in Rooms Earth and HAL. This must
        // locate at least one device because we earlier changed
        // the location of the first device to Room Earth.
        System.out.println("Looking for devices in Earth and HAL.");
        CCS.ControllerPackage.SearchType st[] =
            new CCS.ControllerPackage.SearchType[2];

        st[0] = new CCS.ControllerPackage.SearchType();
        st[1] = new CCS.ControllerPackage.SearchType();

        st[0].key = new CCS.ControllerPackage.KeyType();
        st[0].key.loc("Earth");

        st[1].key = new CCS.ControllerPackage.KeyType();
        st[1].key.loc("HAL");

        CCS.ControllerPackage.SearchSeqHolder ss =
            new CCS.ControllerPackage.SearchSeqHolder(st);
        ctrl.find(ss);

        // Show the devices found in that room.
        for(int i = 0 ; i < ss.value.length ; i++)
            print(System.out, ss.value[i].device);
        System.out.println();

        // Increase the temperature of all thermostats
        // by 40 degrees. First, make a new list (tss)
        // containing only thermostats.
        System.out.println("Increasing thermostats by 40 degrees.");
        java.util.Vector tss_holder = new java.util.Vector();
        for(int i = 0 ; i < list.length ; i++)
        {
            try
            {
                tmstat = CCS.ThermostatHelper.narrow(list[i]);
            }
            catch(org.omg.CORBA.BAD_PARAM se)
            {
            }
            if(tmstat == null)
                continue;                   // Skip thermometers
            tss_holder.addElement(tmstat);
        }

        CCS.Thermostat[] tss = new CCS.Thermostat[tss_holder.size()];
        tss_holder.copyInto(tss);

        // Try to change all thermostats.
        try
        {
            ctrl.change(tss, (short)40);
        }
        catch(CCS.ControllerPackage.EChange ec)
        {
            print(System.out, ec);
        }

        return 0;
    }

    public static void
    main(String args[])
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
	    try
            {
		orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
