// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

import org.omg.CosTradingRepos.ServiceTypeRepositoryPackage.*;

public class Create_types
{
    static org.omg.CosTrading.Lookup
    resolve_init(org.omg.CORBA.ORB orb, String id)
        throws org.omg.CORBA.ORBPackage.InvalidName
    {
        org.omg.CORBA.Object obj = null;
        try
        {
            obj = orb.resolve_initial_references(id);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName e)
        {
            throw e;
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("Cannot get initial reference for " + id +
                               ": " + e);
            throw new RuntimeException();
        }
        if(obj == null)
            throw new RuntimeException();

        org.omg.CosTrading.Lookup ref = null;
        try
        {
            ref = org.omg.CosTrading.LookupHelper.narrow(obj);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("Cannot narrow reference for " + id +
                               ": " + e);
            throw new RuntimeException();
        }
        if(ref == null)
        {
            System.err.println("Incorrect type of reference for " + id);
            throw new RuntimeException();
        }
        return ref;
    }

    public static int
    run(org.omg.CORBA.ORB orb, String args[])
        throws org.omg.CORBA.UserException
    {
        // Get reference to Lookup interface.
        org.omg.CosTrading.Lookup lookup = resolve_init(orb, "TradingService");

        // Read type_repos attribute to get IOR to type repository.
        org.omg.CORBA.Object type_repos_obj = lookup.type_repos();

        // Narrow.
        org.omg.CosTradingRepos.ServiceTypeRepository repos = null;
        try
        {
            repos = org.omg.CosTradingRepos.ServiceTypeRepositoryHelper.
                    narrow(type_repos_obj);
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
            System.err.println("Not a type repository reference");
            return 1;
        }

        // Fill in property definitions for controllers.
        PropStruct[] props = new PropStruct[5];
        for(int i = 0 ; i < props.length ; i++)
            props[i] = new PropStruct();
        org.omg.CORBA.TypeCode stringTC =
            orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_string);
        org.omg.CORBA.TypeCode ulongTC =
            orb.get_primitive_tc(org.omg.CORBA.TCKind.tk_ulong);

        props[0].name = "Model";
        props[0].value_type = stringTC;
        props[0].mode = PropertyMode.PROP_MANDATORY_READONLY;

        props[1].name = "Manufacturer";
        props[1].value_type = Manufacturing.AddressTypeHelper.type();
        props[1].mode = PropertyMode.PROP_NORMAL;

        props[2].name = "Phone";
        props[2].value_type = stringTC;
        props[2].mode = PropertyMode.PROP_MANDATORY;

        props[3].name = "Supports";
        props[3].value_type = Airconditioning.ModelTypeHelper.type();
        props[3].mode = PropertyMode.PROP_READONLY;

        props[4].name = "MaxDevices";
        props[4].value_type = ulongTC;
        props[4].mode = PropertyMode.PROP_NORMAL;

        // Create Controllers service type.
        String[] base_types = new String[0];
        repos.add_type( 
            "CCS::Controllers",
            "IDL:acme.com/CCS/Controller:1.0",
            props,
            base_types);

        return 0;
    }

    public static void
    main(String args[])
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
	    try
            {
		orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
