// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

public class Server
{
    static org.omg.CosTrading.Lookup
    resolve_init(org.omg.CORBA.ORB orb, String id)
        throws org.omg.CORBA.ORBPackage.InvalidName
    {
        org.omg.CORBA.Object obj = null;
        try
        {
            obj = orb.resolve_initial_references(id);
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName e)
        {
            throw e;
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("Cannot get initial reference for " + id +
                               ": " + e);
            throw new RuntimeException();
        }
        if(obj == null)
            throw new RuntimeException();

        org.omg.CosTrading.Lookup ref = null;
        try
        {
            ref = org.omg.CosTrading.LookupHelper.narrow(obj);
        }
        catch(org.omg.CORBA.SystemException e)
        {
            System.err.println("Cannot narrow reference for " + id +
                               ": " + e);
            throw new RuntimeException();
        }
        if(ref == null)
        {
            System.err.println("Incorrect type of reference for " + id);
            throw new RuntimeException();
        }
        return ref;
    }

    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
	// Resolve Root POA
	org.omg.PortableServer.POA poa =
	    org.omg.PortableServer.POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));

	// Get a POA manager
	org.omg.PortableServer.POAManager poa_mgr = poa.the_POAManager();

	// Create a policy list. We use persistent objects with
	// user-assigned IDs, and explicit activation.
	org.omg.CORBA.Policy[] policy_list = new org.omg.CORBA.Policy[6];
	policy_list[0] = poa.create_lifespan_policy(
	    org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);

	policy_list[1] = poa.create_id_assignment_policy(
	    org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

	policy_list[2] = poa.create_implicit_activation_policy(
	    org.omg.PortableServer.ImplicitActivationPolicyValue.
	    NO_IMPLICIT_ACTIVATION);

	policy_list[3] = poa.create_request_processing_policy(
	    org.omg.PortableServer.RequestProcessingPolicyValue.
	    USE_SERVANT_MANAGER);

	policy_list[4] = poa.create_servant_retention_policy(
	    org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN);

	policy_list[5] = poa.create_thread_policy(
	    org.omg.PortableServer.ThreadPolicyValue.SINGLE_THREAD_MODEL);

	// Create a POA for CCS elements.
	org.omg.PortableServer.POA ccs_poa =
	    poa.create_POA("CCS_POA", poa_mgr, policy_list);

	// Create a controller and set static m_ctrl member
	// for thermostats and thermometers.
	Controller_impl ctrl_servant = 
	    new Controller_impl(ccs_poa, "/tmp/CCS_assets");
	Thermometer_impl.m_ctrl = ctrl_servant;
	Thermostat_impl.m_ctrl = ctrl_servant;

	// Create a reference for the controller and
	// create the corresponding CORBA object.
	byte [] oid = Controller_impl.Controller_oid.getBytes();
	org.omg.CORBA.Object ctrl =
	    ccs_poa.create_reference_with_id( 
		oid, "IDL:acme.com/CCS/Controller:1.0");

	// Get reference to Lookup interface.
	org.omg.CosTrading.Lookup lookup = resolve_init(orb, "TradingService");

	// Navigate to Register interface.
	org.omg.CosTrading.Register regis = lookup.register_if();
	if(regis == null)
        {
	    System.err.println("Trader does not support updates.");
	    return 1;
	}

	// Read the offer ID of a previous offer from a file
	// If no offer was previously remembered, set offer id to 
	// an empty string.
	String offer_id_file = "/tmp/ctrl.offer_id";
	String offer_id = "";
	try
        {
	    java.io.FileReader fr = new java.io.FileReader(offer_id_file);
	    java.io.BufferedReader in = new java.io.BufferedReader(fr);
	    offer_id = in.readLine();
	    if(offer_id == null)
		offer_id = "";
	    fr.close();	
	}
	catch(java.io.IOException ex)
        {
	    // Fine no previous offer
	}

	// Attempt to withdraw the previous offer.
	try
        {
	    regis.withdraw(offer_id);
	} 
	catch(org.omg.CosTrading.UnknownOfferId ex)
        {
	    // Fine, there is no previous offer.
	}
        catch(org.omg.CosTrading.IllegalOfferId ex)
        {
	    // Fine, there is no previous offer.
	}

	// Fill in property definition for controller.
	org.omg.CosTrading.Property[] props = 
	    new org.omg.CosTrading.Property[3];
	for(int i = 0 ; i < 3 ; i++)
	    props[i] = new org.omg.CosTrading.Property();

	props[0].name = "Model";
	props[0].value = orb.create_any();
	props[0].value.insert_string("BFG-9000");

	props[1].name = "Phone";
	props[1].value = orb.create_any();
	props[1].value.insert_string("123 456-7890");

	props[2].name = "Description";
	props[2].value = orb.create_any();
	props[2].value.insert_string("Deluxe model for advanced users.");

	// Export the offer. (If your IDL compiler does not recognize
	// the C++ export keyword yet, replace "_cxx_export" with "export".
        offer_id = regis.export(ctrl, "CCS::Controllers", props);

	// Store the new offer ID in peristent storage
	try
        {
	    java.io.FileWriter fw = new java.io.FileWriter(offer_id_file);
	    java.io.BufferedWriter bw = new java.io.BufferedWriter(fw);
	    bw.write(offer_id, 0, offer_id.length());	
	    fw.close();
	}
	catch(java.io.IOException ex)
        {
	    System.err.println("Couldn't write to " + offer_id_file +
			       ex.getMessage());
	    return 1;
	}

	// Instantiate the servant locator for devices.
	DeviceLocator_impl my_locator = new DeviceLocator_impl(ctrl_servant);
	org.omg.PortableServer.ServantManager locator = my_locator._this(orb);

	// Set servant locator.
        ccs_poa.set_servant_manager(locator);

	// Activate the POA manager.
        poa_mgr.activate();

	// Accept requests.
	orb.run();

        return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
	    try
            {
		orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
