// ********************************************************************* 
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
// 
// All Rights Reserved
//
// **********************************************************************

class Controller_impl extends CCS.ControllerPOA 
{
    // Set containing asset numbers of devices
    private java.util.Hashtable m_assets = new java.util.Hashtable();

    // IDL list operation.

    public CCS.Thermometer[] 
    list() 
    {
	// Create a new thermometer sequence. Because we know
	// the number of elements we will put onto the sequence,
	// we use the maximum constructor.
	CCS.Thermometer[] listv = new CCS.Thermometer[m_assets.size()];

        // Loop over the m_assets set and create a
        // reference for each device.
	int count = 0;
	java.util.Enumeration e = m_assets.elements();
        while(e.hasMoreElements())
        {
            listv[count++] = (CCS.Thermometer)e.nextElement();
	}
        return listv;
    }

    // IDL find operation.

    public void 
    find(CCS.ControllerPackage.SearchSeqHolder slist)
    {
	java.util.Vector multiple_device_instance = new java.util.Vector();

	// Loop over input list and lookup each device.
	for(int i = 0 ; i < slist.value.length ; i++)
        {
	    // Assume we will not find a matching device.
	    slist.value[i].device = null;

	    // Work out whether we are searching by asset,
	    // model, or location.
	    CCS.ControllerPackage.SearchCriterion sc = 
	        slist.value[i].key.discriminator();
	    if(sc == CCS.ControllerPackage.SearchCriterion.ASSET)
            {
		// Search for matching asset number.
		Integer asset_num = 
		    new Integer(slist.value[i].key.asset_num());

		CCS.Thermometer element = 
		    (CCS.Thermometer)m_assets.get(asset_num);
		if(element != null)
		    slist.value[i].device = element;
	    }
            else
            {
		// Search for model or location string.
		String search_str = null;
		if(sc == CCS.ControllerPackage.SearchCriterion.LOCATION)
		    search_str = slist.value[i].key.loc();
		else if(sc == CCS.ControllerPackage.SearchCriterion.MODEL)
		    search_str = slist.value[i].key.model_desc();

		// Find first matching device (if any).
		String buf = null;
		int num_found = 0;
		boolean add_element = false;
		java.util.Enumeration e = m_assets.elements(); 
		while(e.hasMoreElements())
                {
		    add_element = false;

		    CCS.Thermometer asset_element = 
		        (CCS.Thermometer)e.nextElement();

		    if(sc == CCS.ControllerPackage.SearchCriterion.LOCATION)
		        if(search_str.equals(asset_element.location()))
			    add_element = true;

		    if(sc == CCS.ControllerPackage.SearchCriterion.MODEL)
		        if(search_str.equals(asset_element.model()))
			    add_element = true;

		    if(add_element)
                    {
		        if(num_found == 0)
                        {
			    // First match overwrites reference
                    	    // in search record.
                            slist.value[i].device = asset_element;
			}
                        else
                        {
			    // Further matches each append a new
                    	    // element to the search sequence.
			    CCS.ControllerPackage.SearchType searchSeqElt = 
			        new CCS.ControllerPackage.SearchType();
			    searchSeqElt.key = slist.value[i].key;
			    searchSeqElt.device = asset_element;
			    multiple_device_instance.addElement(searchSeqElt);
			}
                        num_found++;
		    }
		}
	    }
	}

	// Append any multiple instances of the device in the
	// slist
	if(multiple_device_instance.size() != 0)
        {
	    // Create a new array 
	    int newArrSize = 
		slist.value.length + multiple_device_instance.size();
	    CCS.ControllerPackage.SearchType[] searchSeq = 
	        new CCS.ControllerPackage.SearchType[newArrSize];
	    System.arraycopy(slist.value, 0, searchSeq, 0, slist.value.length);
	    for(int i = 0 ; i < multiple_device_instance.size() ; i++)
            {
	        searchSeq[slist.value.length + i] = 
		    (CCS.ControllerPackage.SearchType)multiple_device_instance.
		    elementAt(i);
	    }

	    slist.value = searchSeq;
	}
    }

    // IDL change operation.

    public void 
    change(CCS.Thermostat[] tlist, short delta) 
        throws CCS.ControllerPackage.EChange
    {
	java.util.Vector error_detail_list = new java.util.Vector();

	// We cannot add a delta value to a thermostat's temperature
	// directly, so for each thermostat, we read the nominal
	// temperature, add the delta value to it, and write
	// it back again.
	for(int i = 0 ; i < tlist.length ; i++)
        {
	    if(tlist[i] == null)
		continue;                       // Skip nil references

	    // Read nominal temp and update it.
	    short tnom = tlist[i].get_nominal();
	    tnom += delta;
	    try
            {
		tlist[i].set_nominal(tnom);
	    }
	    catch(CCS.ThermostatPackage.BadTemp bt)
            {
		// If the update failed because the temperature
		// is out of range, we add the thermostat's info
		// to the errors sequence.
		CCS.ControllerPackage.ErrorDetails new_element = 
		    new CCS.ControllerPackage.ErrorDetails();
		new_element.tmstat_ref = tlist[i];
		new_element.info = bt.details;

		error_detail_list.addElement(new_element);
	    }
	}

	// If we encountered errors in the above loop,
	// we will have added elements to the errors sequence.
	if(error_detail_list.size() != 0)
        {
	    CCS.ControllerPackage.EChange ec =
	        new CCS.ControllerPackage.EChange();

	    ec.errors = 
	        new CCS.ControllerPackage.ErrorDetails[
		    error_detail_list.size()];

	    for(int i = 0 ; i < error_detail_list.size() ; i++)
		ec.errors[i] = 
		    (CCS.ControllerPackage.ErrorDetails)
		    error_detail_list.elementAt(i);

	    throw ec;
	}
    }

    // Helper functions to allow access to the object map

    public void 
    add_impl(int anum, CCS.Thermometer tip)
    {
	Integer assetNum = new Integer(anum);
	m_assets.put(assetNum, tip);
    }

    public void 
    remove_impl(int anum)
    {
	Integer assetNum = new Integer(anum);
        m_assets.remove(assetNum);
    }
}
