// ********************************************************************* 
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
// 
// All Rights Reserved
//
// **********************************************************************

public class Server
{
    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
	// Resolve Root POA
        org.omg.PortableServer.POA poa =
            org.omg.PortableServer.POAHelper.narrow(
                orb.resolve_initial_references("RootPOA"));

	// Activate POA manager
        org.omg.PortableServer.POAManager mgr = poa.the_POAManager();
        mgr.activate();

	// Create a controller and set static m_ctrl member
	// for thermostats and thermometers.
        Controller_impl ctrl_servant = new Controller_impl();
	Thermometer_impl.m_ctrl = ctrl_servant;
	Thermostat_impl.m_ctrl = ctrl_servant;

	// Write a reference for the controller to stdout.
        CCS.Controller ctrl = ctrl_servant._this(orb);
        String ref = orb.object_to_string(ctrl);
        System.out.println(ref);

        // Create a few devices. (Thermometers have odd asset
        // numbers, thermostats have even asset numbers.)
        Thermometer_impl thermo1 = new Thermometer_impl(2029, "Deep Thought");
        ctrl_servant.add_impl(2029, thermo1._this(orb));

	Thermometer_impl thermo2 = new Thermometer_impl(8053, "HAL");
	ctrl_servant.add_impl(8053, thermo2._this(orb));

	Thermometer_impl thermo3 = new Thermometer_impl(1027, "ENIAC");
	ctrl_servant.add_impl(1027, thermo3._this(orb));

	try
        {
	    Thermostat_impl tmstat1 =
		new Thermostat_impl(3032, "Colossus", (short)68);
	    ctrl_servant.add_impl(3032, tmstat1._this(orb));

	    Thermostat_impl tmstat2 =
		new Thermostat_impl(4026, "ENIAC", (short)60);
	    ctrl_servant.add_impl(4026, tmstat2._this(orb));

	    Thermostat_impl tmstat3 =
		new Thermostat_impl(4088, "ENIAC", (short)50);
	    ctrl_servant.add_impl(4088, tmstat3._this(orb));

	    Thermostat_impl tmstat4 =
		new Thermostat_impl(8042, "HAL", (short)40);
	    ctrl_servant.add_impl(8042, tmstat4._this(orb));
	}
	catch(CCS.ThermostatPackage.BadTemp ex)
        {
	    System.err.println("Got a `BadTemp' exception: " +
			       ex.getMessage());
	    return 1;
	}

	// Accept requests.
        orb.run();

        return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
	    try
            {
		orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
