// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package factory;

//
// This factory uses a default servant to implement the record objects
//
public class DefaultFactory_impl extends factory.DBFactoryPOA
{
    private int nextKey_; // Next available key
    private Database database_;
    private org.omg.CORBA.ORB orb_;
    private org.omg.PortableServer.POA poa_;
    private org.omg.PortableServer.POA mypoa_;

    private factory.DBRecord
    makeRef(int key)
    {
        //
        // Create the object ID for the record
        //
        String keyBuff = "" + key;
        byte[] oid = keyBuff.getBytes();

        //
        // Manufacture an object reference for the record
        //
        org.omg.CORBA.Object obj =
            poa_.create_reference_with_id(oid, DBRecordHelper.id());
        factory.DBRecord result = DBRecordHelper.narrow(obj);
        return result;
    }

    public
    DefaultFactory_impl(org.omg.CORBA.ORB orb,
                        org.omg.PortableServer.POA parent)
    {
        nextKey_ = 0;
        database_ = new Database();
        orb_ = orb;
        mypoa_ = parent;

        //
        // Create POA for use by factory
        //
        org.omg.PortableServer.POAManager poaManager = parent.the_POAManager();
        org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[6];
        policies[0] = parent.create_id_assignment_policy(
            org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
        policies[1] = parent.create_lifespan_policy(
            org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
        policies[2] = parent.create_servant_retention_policy(
            org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN);
        policies[3] = parent.create_id_uniqueness_policy(
            org.omg.PortableServer.IdUniquenessPolicyValue.MULTIPLE_ID);
        policies[4] = parent.create_request_processing_policy(
            org.omg.PortableServer.RequestProcessingPolicyValue.
            USE_DEFAULT_SERVANT);
        policies[5] = parent.create_implicit_activation_policy(
            org.omg.PortableServer.ImplicitActivationPolicyValue.
            NO_IMPLICIT_ACTIVATION);

        try
        {
            poa_ = mypoa_.create_POA("default_factory", poaManager, policies);

            //
            // Install default servant
            //
            DSIRecord_impl defaultServant =
                new DSIRecord_impl(orb_, database_);
            poa_.set_servant(defaultServant);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            throw new RuntimeException();
        }
    }

    //
    // Create a new record
    //
    // To create a new key we have an constantly incrementing nextKey
    // member variable. For a real application this isn't good enough,
    // but for this demonstration it will suffice.
    //
    public factory.DBRecord
    createRecord()
    {
        //
        // Add record in database
        //
        int key = nextKey_++;
        database_.add(key);
        return makeRef(key);
    }

    //
    // Destroy the record with `key'. If the record isn't found
    // then raise a NoRecord exception.
    //
    public void
    destroyByKey(int key)
        throws factory.DBFactoryPackage.NoRecord
    {
        //
        // Ask the database to destroy the record
        //
        if(!database_.destroy(key))
            throw new factory.DBFactoryPackage.NoRecord();
    }

    //
    // Retrieve the record identified by `key'. If the record isn't
    // found then raise a NoRecord exception.
    //
    public factory.DBRecord
    getByKey(int key)
        throws factory.DBFactoryPackage.NoRecord
    {
        //
        // Look in the database
        //
        if(database_.find(key) != null)
            return makeRef(key);

        throw new factory.DBFactoryPackage.NoRecord();
    }

    public int[]
    getAllKeys()
    {
        int[] r = new int[database_.size()];
        for(int i = 0 ; i < database_.size() ; i++)
            r[i] = database_.recordAt(i).key;

        return r;
    }

    public org.omg.PortableServer.POA
    _default_POA()
    {
        return mypoa_;
    }
}
