// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package factory;

public class LocatorFactory_impl extends factory.DBFactoryPOA
{
    private int nextKey_;
    private Database database_;
    private org.omg.CORBA.ORB orb_;
    private org.omg.PortableServer.POA poa_;
    private org.omg.PortableServer.POA mypoa_;
    private Reaper reaper_;
    private ReapTimer timer_;
    private Locator locator_;

    private byte[]
    makeKey(int key)
    {
        String keyBuff = "" + key;
        return keyBuff.getBytes();
    }

    private DBRecord
    makeRef(int key)
    {
        //
        // Manufacture an object reference for the record
        //
        byte[] oid = makeKey(key);
        org.omg.CORBA.Object obj =
            poa_.create_reference_with_id(oid, DBRecordHelper.id());
        factory.DBRecord result = DBRecordHelper.narrow(obj);
        return result;
    }

    public
    LocatorFactory_impl(org.omg.CORBA.ORB orb,
                        org.omg.PortableServer.POA parent, boolean debug)
    {
        nextKey_ = 0;
        database_ = new Database();
        orb_ = orb;
        mypoa_ = parent;

        try
        {
            //
            // Create POA for use by factory
            //
            org.omg.PortableServer.POAManager poaManager =
                parent.the_POAManager();
            org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[6];
            policies[0] = parent.create_id_assignment_policy(
                org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
            policies[1] = parent.create_lifespan_policy(
                org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
            policies[2] = parent.create_servant_retention_policy(
                org.omg.PortableServer.ServantRetentionPolicyValue.NON_RETAIN);
            policies[3] = parent.create_id_uniqueness_policy(
                org.omg.PortableServer.IdUniquenessPolicyValue.UNIQUE_ID);
            policies[4] = parent.create_request_processing_policy(
                org.omg.PortableServer.RequestProcessingPolicyValue.
                USE_SERVANT_MANAGER);
            policies[5] = parent.create_implicit_activation_policy(
                org.omg.PortableServer.ImplicitActivationPolicyValue.
                NO_IMPLICIT_ACTIVATION);
            poa_ = mypoa_.create_POA("locator_factory", poaManager, policies);

            //
            // Create reaper and timer. The maxIdle value for the reaper is 2,
            // meaning objects that have been idle for 2+ seconds are eligible
            // for reaping. The timer will invoke the reaper once every second.
            //
            // Note: These timeout values are much too small for most
            // applications.
            //
            reaper_ = new LRUReaper(debug, 2, null);
            timer_ = new ReapTimer(reaper_, 1);

            //
            // Create and install servant locator
            //
            locator_ = new Locator(database_, reaper_, debug);
            org.omg.PortableServer.ServantLocator locator =
                locator_._this(orb_);
            poa_.set_servant_manager(locator);
        }
        catch(org.omg.CORBA.UserException ex)
        {
            throw new RuntimeException();
        }
    }

    //
    // Create a new record
    //
    // To create a new key we have a constantly incrementing nextKey
    // member variable. For a real application this isn't good enough,
    // but for this demonstration it will suffice.
    //
    public factory.DBRecord
    createRecord()
    {
        //
        // Add record in database - the servant locator will create
        // the servant on demand
        //
        int key = nextKey_++;
        database_.add(key);
        return makeRef(key);
    }

    //
    // Destroy the record with `key'. If the record isn't found
    // then raise a NoRecord exception.
    //
    public void
    destroyByKey(int key)
        throws factory.DBFactoryPackage.NoRecord
    {
        //
        // Ask the database to destroy the record
        //
        if(database_.find(key) == null)
            throw new factory.DBFactoryPackage.NoRecord();

        byte[] oid = makeKey(key);
        reaper_.removeServant(oid);
        database_.destroy(key);
    }

    //
    // Retrieve the record identified by `key'. If the record isn't
    // found then raise a NoRecord exception.
    //
    public DBRecord
    getByKey(int key)
        throws factory.DBFactoryPackage.NoRecord
    {
        //
        // Look in the database
        //
        if(database_.find(key) != null)
            return makeRef(key);

        throw new factory.DBFactoryPackage.NoRecord();
    }

    //
    // Return the keys of all records in the database
    //
    public int[]
    getAllKeys()
    {
        int[] r = new int[database_.size()];
        for(int i = 0 ; i < database_.size() ; i++)
            r[i] = database_.recordAt(i).key;

        return r;
    }

    public org.omg.PortableServer.POA
    _default_POA()
    {
        return mypoa_;
    }
}
