// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package factory;

public class Server
{
    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
        //
        // Check for debug option
        //
        boolean debug = false;
        if(args.length > 0 && args[0].equals("-d"))
            debug = true;

        //
        // Get the root POA
        //
        org.omg.PortableServer.POA root =
            org.omg.PortableServer.POAHelper.narrow(
                orb.resolve_initial_references("RootPOA"));

        //
        // Create persistent POA
        //
        org.omg.PortableServer.POAManager manager = root.the_POAManager();
        org.omg.CORBA.Policy[] policies = new org.omg.CORBA.Policy[2];
        policies[0] = root.create_id_assignment_policy(
            org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);
        policies[1] = root.create_lifespan_policy(
            org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);

        org.omg.PortableServer.POA poa =
            root.create_POA("persistent", manager, policies);

        //
        // Create default servant factory implementation object
        //
        DefaultFactory_impl defFactory = new DefaultFactory_impl(orb, poa);
        byte[] oid = "def_factory".getBytes();
        poa.activate_object_with_id(oid, defFactory);
        org.omg.CORBA.Object obj = defFactory._this(orb);

        //
        // Save reference in file
        //
        try
        {
            String myRef = "dfactory.ref";
            java.io.FileOutputStream file =
                new java.io.FileOutputStream(myRef);
            java.io.PrintWriter out = new java.io.PrintWriter(file);

            String ref = orb.object_to_string(obj);
            out.println(ref);
            out.flush();
            file.close();
        }
        catch(java.io.IOException ex)
        {
            System.err.println("factory.Server: can't write to `" +
                               ex.getMessage() + "'");
            return 1;
        }

        //
        // Create servant locator factory implementation object
        //
        LocatorFactory_impl locFactory =
            new LocatorFactory_impl(orb, poa, debug);
        oid = "loc_factory".getBytes();
        poa.activate_object_with_id(oid, locFactory);
        obj = locFactory._this(orb);

        //
        // Save reference in file
        //
        try
        {
            String myRef = "lfactory.ref";
            java.io.FileOutputStream file =
                new java.io.FileOutputStream(myRef);
            java.io.PrintWriter out = new java.io.PrintWriter(file);

            String ref = orb.object_to_string(obj);
            out.println(ref);
            out.flush();
            file.close();
        }
        catch(java.io.IOException ex)
        {
            System.err.println("factory.Server: can't write to `" +
                               ex.getMessage() + "'");
            return 1;
        }

        //
        // Run implementation
        //
        manager.activate();
        orb.run();

        return 0;
    }

    public static void
    main(String[] args)
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            orb = org.omg.CORBA.ORB.init(args, props);
            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
