// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package hello;

public class Server
{
    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
	//
	// Resolve Root POA
	//
	org.omg.PortableServer.POA rootPOA =
	    org.omg.PortableServer.POAHelper.narrow(
		orb.resolve_initial_references("RootPOA"));

	//
	// Get a reference to the POA manager
	//
	org.omg.PortableServer.POAManager manager = rootPOA.the_POAManager();

	//
	// Create new POA using the PERSISTENT and USER_ID policies
	//
        org.omg.CORBA.Policy[] pl = new org.omg.CORBA.Policy[2];
        pl[0] = rootPOA.create_lifespan_policy(
            org.omg.PortableServer.LifespanPolicyValue.PERSISTENT);
        pl[1] = rootPOA.create_id_assignment_policy(
            org.omg.PortableServer.IdAssignmentPolicyValue.USER_ID);

	org.omg.PortableServer.POA helloPOA =
            rootPOA.create_POA("hello", manager, pl);

	//
	// Create implementation object
	//
	Hello_impl helloImpl = new Hello_impl(orb, helloPOA);

	byte[] oid = ("hello").getBytes();
        helloPOA.activate_object_with_id(oid, helloImpl);
	org.omg.CORBA.Object hello = helloImpl._this(orb);

	//
	// Save reference
	//
	try
	{
	    String ref = orb.object_to_string(hello);
	    String refFile = "Hello.ref";
	    java.io.FileOutputStream file =
		new java.io.FileOutputStream(refFile);
	    java.io.PrintWriter out = new java.io.PrintWriter(file);
	    out.println(ref);
	    out.flush();
	    file.close();
	}
	catch(java.io.IOException ex)
	{
	    System.err.println("hello_imr.Server: can't write to `" +
                               ex.getMessage() + "'");
	    return 1;
	}

	//
	// Run implementation
	//
        manager.activate();
	orb.run();

	return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	org.omg.CORBA.ORB orb = null;

	try
	{
	    //
	    // Set server name if it has not already been set
	    //
	    args = com.ooc.CORBA.ORB.ParseArgs(args, props, null);
	    String server = props.getProperty("ooc.orb.server_name");
	    if(server == null)
	        props.put("ooc.orb.server_name", "hello");

	    orb = org.omg.CORBA.ORB.init(args, props);
	    status = run(orb, args);
	}
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}

	if(orb != null)
	{
	    //
	    // Since the standard ORB.destroy() method is not present in
	    // JDK 1.2.x, we must cast to com.ooc.CORBA.ORB so that this
	    // will compile with all JDK versions
	    //
	    try
	    {
		orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
