// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package perf;

final public class Client
{
    final private static String progName_ = "perf.Client";

    static void
    usage()
    {
	System.err.println("Usage: " + progName_ + " [--oneway] repetitions");
    }

    static int
    run(org.omg.CORBA.ORB orb, String[] args)
	throws org.omg.CORBA.UserException
    {
	boolean oneway = false;
	int reps;

	//
	// Get number of repetitions
	//
	if(args.length < 1)
	{
	    usage();
	    return 1;
	}

	if(args.length == 2)
	{
	    if(!args[0].equals("--oneway"))
	    {
		usage();
		return 1;
	    }
	    oneway = true;
	    reps = Integer.parseInt(args[1]);
	}
	else
	    reps = Integer.parseInt(args[0]);

	if(reps <= 0)
	{
	    System.err.println(
		progName_ + ": number of repetitions must be larger than 0");
	    return 1;
	}

	//
	// Get "I" object
	//
	org.omg.CORBA.Object obj =
	    orb.string_to_object("relfile:/I.ref");
	if(obj == null)
	{
	    System.err.println(progName_ + ": cannot read IOR from I.ref");
	    return 1;
	}

	I i = IHelper.narrow(obj);

	//
	// Force a connection to the server.
	//
	long tv = System.currentTimeMillis();
	i.f();
	tv = System.currentTimeMillis() - tv;
	System.out.println("Time needed for initialization: " + tv + " ms");

	//
	// Main loop
	//
	tv = System.currentTimeMillis();
	for(int j = 0; j < reps; ++j)
	    i.f();

	tv = System.currentTimeMillis() - tv;
	System.out.println("Total time needed: " + tv + " ms");
	System.out.println("Time per method call: " +
			   tv / (1.0 * reps) + " ms");

	return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
	          "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
	{
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
