// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package pi;

import java.io.*;

final class Client
{
    //
    // Menu string
    //
    private static final String menu =
	"Portable Interceptor Demo\n" +
	"~~~~~~~~~~~~~~~~~~~~~~~~~\n" +
	"m: Display this menu\n" +
	"l: Enable/disable logging\n" +
	"q: Quit client and shutdown server\n" +
	"1: Invoke \"void   noargs()\"\n" +
	"2: Invoke \"void   noargs_oneway()\"\n" +
	"3: Invoke \"string noargs_return()\"\n" +
	"4: Invoke \"void   withargs(in string, inout string, out string)\"\n" +
	"5: Invoke \"void   systemexception()\"\n" +
	"6: Invoke \"void   userexception()\"\n" +
	"7: Invoke \"void   switch_to_static_impl()\"\n" +
	"8: Invoke \"void   switch_to_dynamic_impl()\"\n" +
	"9: Invoke \"void   call_other_impl(in short)\"\n";

    //
    // Main client code
    //
    private static int
    run(org.omg.CORBA.ORB orb, String[] args)
	throws org.omg.CORBA.UserException
    {
	//
	// Get the logging level from the command-line
	//
	if(args.length > 1)
	{
	    System.err.println("Usage: client [log-level]");
	    return 1;
	}

	short logLevel = 1;
	if(args.length > 0)
	    logLevel = Short.parseShort(args[0]);

	//
	// Get PIDemo object reference
	//
	org.omg.CORBA.Object obj = orb.string_to_object("relfile:/PIDemo.ref");
	if(obj == null)
	{
	    System.err.println("Cannot read IOR from PIDemo.ref");
	    return 1;
	}

	//
	// Narrow the object reference
	//
	PIDemo piDemoWithoutLogging = null;
	try
	{
	    piDemoWithoutLogging = PIDemoHelper.narrow(obj);
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    System.err.println("IOR in PIDemo.ref is not a PIDemo IOR");
	    return 1;
	}

	//
	// Create a policy set on the object reference to set the
	// logging level
	//
	org.omg.CORBA.Any any = orb.create_any();
	any.insert_short(logLevel);
	org.omg.CORBA.Policy[] pl = new org.omg.CORBA.Policy[1];
	pl[0] = orb.create_policy(LOGGER_POLICY_ID.value, any);
	obj = obj._set_policy_override(pl,
	    org.omg.CORBA.SetOverrideType.ADD_OVERRIDE);

	//
	// Narrow new object reference
	//
	PIDemo piDemoWithLogging = null;
	try
	{
	    piDemoWithLogging = PIDemoHelper.narrow(obj);
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    System.err.println("IOR in PIDemo.ref is not a PIDemo IOR");
	    return 1;
	}

	//
	// Main loop
	//
	BufferedReader in =
	    new BufferedReader(new InputStreamReader(System.in));
	PIDemo piDemo = piDemoWithLogging;
	boolean logging = true;
	char cmd = 0;

	System.out.print(menu);
	do
	{
	    System.out.print("> ");

	    try
	    {
		String s = in.readLine();
		if(s.length() == 0)
		    continue;
		cmd = s.charAt(0);
	    }
	    catch(IOException ex)
	    {
		ex.printStackTrace();
		return 1;
	    }

	    switch(cmd)
	    {
	    case 'm':
		System.out.print(menu);
		break;

	    case 'l':
		logging = !logging;
		piDemo = logging ? piDemoWithLogging : piDemoWithoutLogging;
		System.out.println("Client logging is now " +
				   (logging ? "enabled" : "disabled"));
		break;

	    case 'q':
		piDemo.deactivate();
		break;

	    case '1':
		System.out.println("void PIDemo::noargs()");
		piDemo.noargs();
		break;

	    case '2':
		System.out.println("void PIDemo::noargs_oneway()");
		piDemo.noargs_oneway();
		break;

	    case '3':
		System.out.println("string PIDemo::noargs_return()");
		piDemo.noargs_return();
		break;

	    case '4':
	    {
		System.out.println("void PIDemo::withargs(in string, " +
				   "inout string, out string)");
		org.omg.CORBA.StringHolder strHolder1 =
		    new org.omg.CORBA.StringHolder();
		org.omg.CORBA.StringHolder strHolder2 =
		    new org.omg.CORBA.StringHolder();
		strHolder1.value = "";
		piDemo.withargs("", strHolder1, strHolder2);
		break;
	    }

	    case '5':
		System.out.println("void PIDemo::systemexception()");
		try
		{
		    piDemo.systemexception();
		}
		catch(org.omg.CORBA.SystemException ex)
		{
		}
		break;

	    case '6':
		System.out.println("void PIDemo::userexception()");
		try
		{
		    piDemo.userexception();
		}
		catch(pi.PIDemoPackage.User ex)
		{
		}
		break;

	    case '7':
		System.out.println("void PIDemo::switch_to_static_impl()");
		piDemo.switch_to_static_impl();
		break;

	    case '8':
		System.out.println("void PIDemo::switch_to_dynamic_impl()");
		piDemo.switch_to_dynamic_impl();
		break;

	    case '9':
	    {
		System.out.println("void PIDemo::call_other_impl(" +
				   "in short level)");
		System.out.print("Enter level> ");

		short level = 0;
		try
		{
		    level = Short.parseShort(in.readLine());
		}
		catch(IOException ex)
		{
		    ex.printStackTrace();
		    return 1;
		}

		piDemo.call_other_impl(level);
		break;
	    }
	    }
	}
	while(cmd != 'q');

	return 0;
    }

    //
    // Client main entry point
    //
    public static void
    main(String[] args)
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	org.omg.CORBA.ORB orb = null;

	try
	{
	    //
	    // Initialize the request interceptors
	    //
	    RILogger.init(props);

	    //
	    // Initialize the ORB
	    //
	    orb = org.omg.CORBA.ORB.init(args, props);

	    //
	    // Start the client
	    //
	    status = run(orb, args);
	}
	catch(Exception ex)
	{
	    ex.printStackTrace();
	    status = 1;
	}

	if(orb != null)
	{
	    try
	    {
		orb.destroy();
	    }
	    catch(Exception ex)
	    {
		ex.printStackTrace();
		status = 1;
	    }
	}

	System.exit(status);
    }
}
