// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package pi;

//
// Request interceptor logger base class
//
class RILogger
{
    //
    // Service context identifiers
    //
    protected final int REQUEST_CONTEXT_ID = 100;
    protected final int REPLY_CONTEXT_ID = 101;

    //
    // Used to display CORBA::Any
    //
    protected AnyWriter out_;

    //
    // Used to encode/decode the call stack information so that
    // it can be used as the service context
    //
    protected org.omg.IOP.Codec cdrCodec_;

    //
    // My slot id
    //
    protected int mySlotId_;

    // -----------------------------------------------------------------
    // RILogger private member implementation
    // -----------------------------------------------------------------

    //
    // Convert CORBA::Octet to string
    //
    protected String
    getOctet(byte octet)
    {
	String prefix;
	if (octet < 0x10)
	    prefix = "<0";
	else
	    prefix = "<";
	
	return prefix + Integer.toHexString(octet) + ">";
    }

    //
    // Display PortableInterceptor::RequestInfo
    //
    protected void
    displayRequestInfo(org.omg.PortableInterceptor.RequestInfo info,
		       short logLevel)
    {
	int i;

	//
	// Display request id
	//
	long request_id = info.request_id();
	out_.skip();
	out_.getWriter().println("request id = " + request_id);

	//
	// Display operation
	//
	String operation = info.operation();
	out_.skip();
	out_.getWriter().println("operation = " + operation);

	//
	// Display arguments
	//
	try
	{
	    org.omg.Dynamic.Parameter[] args = info.arguments();

	    out_.skip();
	    out_.getWriter().print("arguments = ");
	    if (args.length == 0)
	    {
		out_.getWriter().print("(no arguments)");
	    }
	    else
	    {
		out_.down();
		for (i = 0 ; i < args.length ; i++)
		{
		    out_.getWriter().println();

		    if(args[i].mode == org.omg.CORBA.ParameterMode.PARAM_IN)
		    {
			out_.skip();
			out_.getWriter().print("in ");
		    }
		    else if(args[i].mode ==
			    org.omg.CORBA.ParameterMode.PARAM_OUT)
		    {
			out_.skip();
			out_.getWriter().print("out ");
		    }
		    else // case PARAM_INOUT:
		    {
			out_.skip();
			out_.getWriter().print("inout ");
		    }

		    out_.print(args[i].argument);
		}
		out_.up();
	    }
	    out_.getWriter().println();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
	catch(org.omg.CORBA.NO_RESOURCES ex)
	{
	    // Operation not supported in this environment
	}

	//
	// Display exceptions
	//
	try
	{
	    org.omg.CORBA.TypeCode[] exceptions = info.exceptions();

	    out_.skip();
	    out_.getWriter().print("exceptions = ");
	    if (exceptions.length == 0)
	    {
		out_.getWriter().println("(no exceptions)");
	    }
	    else
	    {
		out_.getWriter().println();

		out_.down();
		for (i = 0 ; i < exceptions.length ; i++)
		    out_.print(exceptions[i]);
		out_.up();
	    }
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
	catch(org.omg.CORBA.NO_RESOURCES ex)
	{
	    // Operation not supported in this environment
	}

	//
	// Display result
	//
	try
	{
	    org.omg.CORBA.Any result = info.result();

	    out_.skip();
	    out_.getWriter().print("result = ");
	    out_.print(result);
	    out_.getWriter().println();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}
	catch(org.omg.CORBA.NO_RESOURCES ex)
	{
	    // Operation not supported in this environment
	}

	//
	// Display response expected
	//
	boolean response_expected = info.response_expected();

	out_.skip();
	out_.getWriter().print("response expected = ");
	if (response_expected)
	    out_.getWriter().print("true");
	else
	    out_.getWriter().print("false");
	out_.getWriter().println();

	//
	// Display reply status
	//
	try
	{
	    short status = info.reply_status();

	    out_.skip();
	    out_.getWriter().print("reply status = ");
	    if (status == org.omg.PortableInterceptor.SUCCESSFUL.value)
		out_.getWriter().print("SUCCESSFUL");
	    else if (status == org.omg.PortableInterceptor.
		     SYSTEM_EXCEPTION.value)
		out_.getWriter().print("SYSTEM_EXCEPTION");
	    else if (status == org.omg.PortableInterceptor.
		     USER_EXCEPTION.value)
		out_.getWriter().print("USER_EXCEPTION");
	    else if (status == org.omg.PortableInterceptor.
		     LOCATION_FORWARD.value)
		out_.getWriter().print("LOCATION_FORWARD");
	    else if (status == org.omg.PortableInterceptor.
		     TRANSPORT_RETRY.value)
		out_.getWriter().print("TRANSPORT_RETRY");
	    else
		out_.getWriter().print("UNKNOWN_REPLY_STATUS");
	    out_.getWriter().println();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}

	if(logLevel > 1)
	{
	    //
	    // Display slot data
	    //
	    try
	    {
		org.omg.CORBA.Any slotData = info.get_slot(mySlotId_);

		out_.skip();
		out_.getWriter().println("slot data " + mySlotId_ + " = ");

		out_.down();
		out_.skip();
		out_.print(slotData);
		out_.getWriter().println();
		out_.up();
	    }
	    catch(org.omg.PortableInterceptor.InvalidSlot ex)
	    {
	    }

	    //
	    // Display request service context
	    //
	    try
	    {
		org.omg.IOP.ServiceContext context =
		    info.get_request_service_context(REQUEST_CONTEXT_ID);

		out_.skip();
		out_.getWriter().print("request service context " +
				 REQUEST_CONTEXT_ID + " = ");
		for (i = 0 ; i < context.context_data.length ; i++)
		    out_.getWriter().print(getOctet(context.context_data[i]));
		out_.getWriter().println();
	    }
	    catch(org.omg.CORBA.BAD_INV_ORDER ex)
	    {
		// Operation not supported in this context
	    }
	    catch(org.omg.CORBA.BAD_PARAM ex)
	    {
		// No entry found
	    }

	    //
	    // Display reply service context
	    //
	    try
	    {
		org.omg.IOP.ServiceContext context =
		    info.get_reply_service_context(REPLY_CONTEXT_ID);

		out_.skip();
		out_.getWriter().print("reply service context " +
				 REPLY_CONTEXT_ID + " = ");
		for (i = 0 ; i < context.context_data.length ; i++)
		    out_.getWriter().print(getOctet(context.context_data[i]));
		out_.getWriter().println();
	    }
	    catch(org.omg.CORBA.BAD_INV_ORDER ex)
	    {
		// Operation not supported in this context
	    }
	    catch(org.omg.CORBA.BAD_PARAM ex)
	    {
		// No entry found
	    }
	}
    }

    // -----------------------------------------------------------------
    // RILogger constructor
    // -----------------------------------------------------------------

    public RILogger(org.omg.PortableInterceptor.ORBInitInfo info, int slotId)
    {
	mySlotId_ = slotId;

	//
	// Get the codec factory
	//
	org.omg.IOP.CodecFactory factory = info.codec_factory();
	if(factory == null)
	    throw new RuntimeException();

	//
	// Create codec
	//
	org.omg.IOP.Encoding how = new org.omg.IOP.Encoding();
	how.major_version = 1;
	how.minor_version = 0;
	how.format = org.omg.IOP.ENCODING_CDR_ENCAPS.value;

	try
	{
	    cdrCodec_ = factory.create_codec(how);
	}
	catch(org.omg.IOP.CodecFactoryPackage.UnknownEncoding ex)
	{
	    throw new RuntimeException();
	}
	if(cdrCodec_ == null)
	    throw new RuntimeException();

	//
	// Get the dynamic any factory
	//
	org.omg.DynamicAny.DynAnyFactory dynAnyFactory = null;
	try
	{
	    org.omg.CORBA.Object obj = 
		info.resolve_initial_references("DynAnyFactory");
	    dynAnyFactory = org.omg.DynamicAny.DynAnyFactoryHelper.narrow(obj);
	}
	catch(org.omg.PortableInterceptor.ORBInitInfoPackage.InvalidName ex)
	{
	    throw new RuntimeException();
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    throw new RuntimeException();
	}

	//
	// Create the AnyWriter object for displaying the CORBA::Any
	//
	out_ = new AnyWriter(dynAnyFactory);
	out_.down();
    }

    // -----------------------------------------------------------------
    // RILogger public member implementation
    // -----------------------------------------------------------------

    //
    // Register the Request Interceptor loggers
    //
    public static void init(java.util.Properties props)
    {
	//
	// Register the ORB initializer
	//
	props.put("org.omg.PortableInterceptor.ORBInitializerClass." +
		  "pi.RIORBInitializer_impl", "");
    }
}
