// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package repository;

public class Client
{
    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
        //
	// Install value factory
	//
	((org.omg.CORBA_2_3.ORB)orb).register_value_factory(
	    Demo.ValueHelper.id(), new ValueFactory_impl());

	//
	// Get Demo::I object reference
	//
	org.omg.CORBA.Object obj = orb.string_to_object("relfile:/Demo.ref");
	if(obj == null)
	{      
	    System.err.println("repository.Client: " +
			       "cannot read IOR from Demo.ref");
	    return 1;
	}

	Demo.I interf = Demo.IHelper.narrow(obj);

	//
	// Print interface repository content for Demo::I
	//
	obj = interf._get_interface_def();
	if(obj == null)
	{
	    System.err.println("repository.Client: " +
			       "no interface repository available");
	    return 1;
	}
	org.omg.CORBA.InterfaceDef idef =
	    org.omg.CORBA.InterfaceDefHelper.narrow(obj);

	org.omg.CORBA.InterfaceDefPackage.FullInterfaceDescription desc =
	    idef.describe_interface();
	
	int i;

	System.out.println("name = " + desc.name);
	System.out.println("id = " + desc.id);
	System.out.println("defined_in = " + desc.defined_in);
	System.out.println("version = " + desc.version);
	System.out.println("operations:");
	for(i = 0 ; i < desc.operations.length ; i++)
	{
	    System.out.println("" + i + ": " + desc.operations[i].name);
	}
	System.out.println("attributes:");
	for(i = 0 ; i < desc.attributes.length ; i++)
	{
	    System.out.println("" + i + ": " + desc.attributes[i].name);
	}
	System.out.println("base_interfaces:");
	for(i = 0 ; i < desc.base_interfaces.length ; i++)
	{
	    System.out.println("" + i + ": " + desc.base_interfaces[i]);
	}

	//
	// Get valuetype
	//
	Demo.BaseValue value = interf.get_value();

	//
	// Get ID of most-derived interface
	//
	String id = value._truncatable_ids()[0];

	//
	// Print interface repository content for Demo::Value
	//
	obj = ((org.omg.CORBA_2_3.ORB)orb).get_value_def(id);
	if(obj == null)
	{
	    System.err.println("repository.Client: " +
			       "no interface repository available");
	    return 1;
	}
	org.omg.CORBA.ValueDef vdef =
	    org.omg.CORBA.ValueDefHelper.narrow(obj);

	org.omg.CORBA.ValueDefPackage.FullValueDescription vdesc =
	    vdef.describe_value();

	System.out.println();
	System.out.println("name = " + vdesc.name);
	System.out.println("id = " + vdesc.id);
	System.out.println("defined_in = " + vdesc.defined_in);
	System.out.println("version = " + vdesc.version);
	System.out.println("operations:");
	for(i = 0 ; i < vdesc.operations.length ; i++)
	{
	    System.out.println("" + i + ": " + vdesc.operations[i].name);
	}
	System.out.println("attributes:");
	for(i = 0 ; i < vdesc.attributes.length ; i++)
	{
	    System.out.println("" + i + ": " + vdesc.attributes[i].name);
	}
	System.out.println("members:");
	for(i = 0 ; i < vdesc.members.length ; i++)
	{
	    System.out.println("" + i + ": " + vdesc.members[i].name);
	}
	System.out.println("base_value: " + vdesc.base_value);
	System.out.println("abstract_base_values:");
	for(i = 0 ; i < vdesc.abstract_base_values.length ; i++)
	{
	    System.out.println("" + i + ": " +
			       vdesc.abstract_base_values[i]);
	}

	return 0;
    }

    public static void
    main(String[] args)
    {
	java.util.Properties props = System.getProperties();
	props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
	props.put("org.omg.CORBA.ORBSingletonClass",
		  "com.ooc.CORBA.ORBSingleton");

	int status = 0;
	org.omg.CORBA.ORB orb = null;

        try
        {
	    orb = org.omg.CORBA.ORB.init(args, props);
	    status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
	    status = 1;
        }

  	if(orb != null)
	{
	    try
	    {
		orb.destroy();
	    }
	    catch(Exception ex)
	    {
                ex.printStackTrace();
                status = 1;
	    }
	}

        System.exit(status);
    }
}
