// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OB;

abstract public class CodeConverterBase
    implements CodeSetReader, CodeSetWriter
{
    //
    // Source and destination code set
    //
    protected CodeSetInfo from_;
    protected CodeSetInfo to_;

    //
    // The UTF-8 or fixed width reader/writer
    //
    private CodeSetReader reader_;
    private CodeSetWriter writer_;

    private static CodeSetInfo noneInstance_;

    CodeConverterBase(CodeSetInfo from, CodeSetInfo to)
    {
	if((from == null || to == null) && noneInstance_ == null)
	{
	    noneInstance_ = new CodeSetInfo("none", 0, 0, null, (short)2);
	}

	if(from == null)
	    from = noneInstance_;
	if(to == null)
	    to = noneInstance_;

        from_ = from;
        to_ = to;

	if(from.rgy_value == CodeSetDatabase.UTF8)
	    reader_ = new UTF8Reader();
	else if(from.max_bytes <= 2)
	    reader_ = new FixedWidth2Reader();
	else
	{
	    //
	    // Java doesn't support wide characters larger than 16 bit
	    //
	    Assert._OB_assert(false);
	}

	if(to.rgy_value == CodeSetDatabase.UTF8)
	    writer_ = new UTF8Writer();
	else if(to.max_bytes <= 2)
	    writer_ = new FixedWidth2Writer();
	else
	{
	    //
	    // Java doesn't support wide characters larger than 16 bit
	    //
	    Assert._OB_assert(false);
	}
    }

    final public boolean
    equals(java.lang.Object obj)
    {
        if(obj == null)
            return false;
        if(obj == this)
            return true;

        CodeConverterBase b = (CodeConverterBase)obj;

        return(from_.rgy_value == b.from_.rgy_value &&
               to_.rgy_value == b.to_.rgy_value);
    }

    final public char
    read_char(com.ooc.CORBA.InputStream in)
        throws org.omg.CORBA.DATA_CONVERSION
    {
	return reader_.read_char(in);
    }

    public char
    read_wchar(com.ooc.CORBA.InputStream in, int len)
        throws org.omg.CORBA.DATA_CONVERSION
    {
	return reader_.read_wchar(in, len);
    }

    public void
    write_char(com.ooc.CORBA.OutputStream out, char v)
        throws org.omg.CORBA.DATA_CONVERSION
    {
	writer_.write_char(out, v);
    }

    public void
    write_wchar(com.ooc.CORBA.OutputStream out, char v)
        throws org.omg.CORBA.DATA_CONVERSION
    {
	writer_.write_wchar(out, v);
    }

    public int
    count_wchar(byte v)
    {
	return reader_.count_wchar(v);
    }

    public int
    count_wchar(char v)
    {
	return writer_.count_wchar(v);
    }

    final public boolean
    readerRequired()
    {
	return from_.rgy_value == CodeSetDatabase.UTF8;
    }

    final public boolean
    writerRequired()
    {
	return to_.rgy_value == CodeSetDatabase.UTF8;
    }

    final public CodeSetInfo
    getFrom()
    {
	return from_;
    }

    final public CodeSetInfo
    getTo()
    {
	return to_;
    }

    //
    // Get conversion type
    //
    public abstract boolean conversionRequired();

    //
    // Convert narrow or wide character
    //
    public abstract char convert(char value);
}
