// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OB;

import com.ooc.PortableServer.*;

abstract class GIOPServerStarter
{
    protected ORBInstance orbInstance_; // The ORBInstance
    protected com.ooc.OCI.Acceptor acceptor_; // The acceptor
    protected OAInterface oaInterface_; // The object adapter interface
    protected java.util.Vector workers_ = new java.util.Vector(); // Workers

    public static final int StateActive  = 0;
    public static final int StateHolding = 1;
    public static final int StateClosed  = 2;

    protected int state_;

    // ----------------------------------------------------------------------
    // GIOPServer private and protected member implementation
    // ----------------------------------------------------------------------

    protected void
    finalize()
	throws Throwable
    {
        Assert._OB_assert(state_ == StateClosed);

        super.finalize();
    }

    //
    // Emit a trace message when closing the acceptor
    //
    protected void
    logCloseAcceptor()
    {
        CoreTraceLevels coreTraceLevels = orbInstance_.getCoreTraceLevels();
        if(coreTraceLevels.traceConnections() > 0)
        {
            com.ooc.OCI.AcceptorInfo info = acceptor_.get_info();
            String msg = "stopped accepting connections\n";
            msg += info.describe();
            orbInstance_.getLogger().trace("incoming", msg);
        }
    }

    protected void
    reapWorkers()
    {
        for(int i = 0 ; i < workers_.size() ;)
        {
            GIOPServerWorker worker = (GIOPServerWorker)workers_.elementAt(i);
            if(worker.destroyed())
                workers_.removeElementAt(i);
            else
                ++i;

        }
    }

    // ----------------------------------------------------------------------
    // GIOPServer public member implementation
    // ----------------------------------------------------------------------

    GIOPServerStarter(ORBInstance orbInstance, com.ooc.OCI.Acceptor acceptor,
		      OAInterface oaInterface)
    {
        orbInstance_ = orbInstance;
	acceptor_ = acceptor;
	oaInterface_ = oaInterface;
        state_ = StateHolding; // Must be holding initially

        try
        {
            //
            // Trace acceptor creation
            //
            CoreTraceLevels coreTraceLevels = orbInstance_.getCoreTraceLevels();
            if(coreTraceLevels.traceConnections() > 0)
            {
                com.ooc.OCI.AcceptorInfo info = acceptor_.get_info();
                String msg = "accepting connections\n";
                msg += info.describe();
                orbInstance_.getLogger().trace("incoming", msg);
            }

            //
            // Start listening
            //
            acceptor_.listen();
        }
        catch(org.omg.CORBA.SystemException ex)
        {
            acceptor_.close();
            state_ = StateClosed;
            throw ex;
        }
    }

    //
    // Change the state of the worker
    //
    abstract public void
    setState(int state);
}
