// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test.multi;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;

public class Server
{
    public static int
    run(ORB orb, String[] args)
	throws org.omg.CORBA.UserException
    {
	//
	// Get number of repetitions
	//
	if(args.length != 1)
	{
	    System.err.println("Usage: test.main.Server server-number");
	    return 1;
	}
	
	int num = 0;
	try
	{
	    num = Integer.parseInt(args[0]);
	}
	catch(NumberFormatException ex)
	{
	}
	
	if(num <= 0)
	{
	    System.err.println("test.main.Server: server number must " +
			       "be larger than 0");
	    return 1;
	}
	
	//
	// Resolve Root POA
	//
	POA poa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
	
        //
        // Activate the POA manager
        //
	POAManager manager = poa.the_POAManager();
	manager.activate();

	//
	// Create implementation object
	//
	Multi_impl multiImpl = new Multi_impl(orb, num);
	Multi multi = multiImpl._this(orb);

	//
	// Save reference. This must be done after POA manager
	// activation, otherwise there is a potential for a race
	// condition between the client sending request and the server
	// not being ready yet.
	//
	String s = orb.object_to_string(multi);
	
	String refFile = "Multi.ref." + num;
	
	try
	{
	    java.io.FileOutputStream file =
		new java.io.FileOutputStream(refFile);
	    java.io.PrintWriter out = new java.io.PrintWriter(file);
                out.println(s);
                out.flush();
                file.close();
	}
	catch(java.io.IOException ex)
	{
	    System.err.println("test.main.Server: can't write to `" +
			       ex.getMessage() + "'");
	    return 1;
	}
	
	//
	// Give up control to the ORB
	//
	orb.run();
	
	//
	// Clean up
	//
	new java.io.File(refFile).delete();

        return 0;
    }

    public static void
    main(String[] args)
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        ORB orb = null;

        try
        {
            org.omg.CORBA.StringSeqHolder argsH =
                new org.omg.CORBA.StringSeqHolder(args);
            orb = com.ooc.CORBA.ORB.init(argsH, props, null);
            status = run(orb, argsH.value);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
