// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package test;

import org.omg.CORBA.*;
import org.omg.PortableServer.*;
import org.omg.CosPropertyService.*;

final class Client
{
    static int
    run(ORB orb, String args[])
    {
        //
        // Get property service
        //
        org.omg.CORBA.Object obj = null;
        try
        {
	    obj = orb.resolve_initial_references("PropertyService");
        }
        catch(org.omg.CORBA.ORBPackage.InvalidName ex)
        {
	    System.err.println("test.Client: " +
	    		       "can't resolve `PropertyService'");
	    return 1;
        }

        if(obj == null)
        {
	    System.err.println("test.Client: `PropertyService' is a " +
	    		       "nil object reference");
	    return 1;    
        }

        PropertySetDefFactory factory = null;
	try
	{
	    factory = PropertySetDefFactoryHelper.narrow(obj);
	}
	catch(BAD_PARAM ex)
	{
	}

        if(factory == null)
        {
	    System.err.println("test.Client: `PropertyService' is not a " +
	    		       "PropertySetDefFactory object reference");
	    return 1;
        }

        try
        {
	    System.out.println("Testing unconstrained PropertySet");
	    TestUnconstrained.test(orb, factory);

	    System.out.println("Testing constrained PropertySet");
	    TestConstrained.test(orb, factory);

	    System.out.println("Testing initial PropertySet");
	    TestInitial.test(orb, factory);
        }
        catch(UserException ex)
        {
	    System.err.println(
	        "test.Client: Unexpected user exception was thrown");
	    return 1;
        }

        return 0;
    }

    public static void
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        ORB orb = null;

        try
        {
            orb = ORB.init(args, props);
            status = run(orb, args);
        }
        catch(RuntimeException ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
