// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package filetransfer;

public class Sender
{
    static int
    run(org.omg.CORBA.ORB orb, String[] args)
        throws org.omg.CORBA.UserException
    {
        //
        // Check the arguments
        //
        if(args.length < 2)
        {
            System.err.println("usage: java filetransfer.Sender " +
                               "source destination");
            return 1;
        }

        //
        // Get "filetransfer" object
        //
        org.omg.CORBA.Object obj =
            orb.string_to_object("relfile:/FileTransfer.ref");
        if(obj == null)
        {
            System.err.println("Can't read from FileTransfer.ref");
            return 1;
        }

        FileTransfer filetransfer = null;
        try 
        {
            filetransfer = FileTransferHelper.narrow(obj);
        }
        catch(org.omg.CORBA.BAD_PARAM ex)
        {
            // narrow() failed
            ex.printStackTrace();
            return 1;
        }

        String inFileName = args[0];
        String outFileName = args[1];

        //
        // Read data from the file to be transferred
        //
        java.io.FileInputStream inStream = null;
        try
        {
            inStream = new java.io.FileInputStream(inFileName);
        }
        catch(java.io.FileNotFoundException ex)
        {
            System.err.println("Can't open file " + inFileName + ": " +
                               ex.getMessage());
            return 1;
        }

        byte[] file = null;
        try
        {
            file = new byte[inStream.available()];
            inStream.read(file);
        }
        catch(java.io.IOException ex)
        {
            System.err.println("Can't read from " + inFileName + ": " +
                               ex.getMessage());
            return 1;
        }

        //
        // Send the file
        //
        filetransfer.send(outFileName, file);

        return 0;
    }

    public static void 
    main(String args[])
    {
        java.util.Properties props = System.getProperties();
        props.put("org.omg.CORBA.ORBClass", "com.ooc.CORBA.ORB");
        props.put("org.omg.CORBA.ORBSingletonClass",
                  "com.ooc.CORBA.ORBSingleton");

        int status = 0;
        org.omg.CORBA.ORB orb = null;

        try
        {
            //
            // Filter the ORB command-line options
            //
            args = com.ooc.CORBA.ORB.ParseArgs(args, props, null);

            //
            // Filter the UDP command-line options
            //
            args = com.ooc.OCI.udp.parse_args(args, props);

            //
            // Create ORB
            //
            orb = org.omg.CORBA.ORB.init(args, props);

            status = run(orb, args);
        }
        catch(Exception ex)
        {
            ex.printStackTrace();
            status = 1;
        }

        if(orb != null)
        {
            try
            {
                orb.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                status = 1;
            }
        }

        System.exit(status);
    }
}
