// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package imageshow;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

final class ReceiverWindow extends JFrame
{
    //
    // The receiver
    //
    private ImageShow_impl imageShow_;

    //
    // The "Widgets"
    //
    private JTextArea history_;
    private JFrame frame_;

    private ImageDisplay imageDisplay_;
    private InfoDialog aboutDialog_;

    //
    // Constructor
    //
    ReceiverWindow(org.omg.CORBA.ORB orb, org.omg.PortableServer.POA poa)
    {
        //
        // Create client window
        //
        super("ImageShow Demo: Receiver");
        frame_ = (JFrame)this;

        //
        // Add the MenuBar
        //
        JMenuBar menuBar = new JMenuBar();
        addMenus(menuBar);
        setJMenuBar(menuBar);

        // 
        // Build the UI
        //
        Border e = BorderFactory.createEtchedBorder();
        TitledBorder t1 = BorderFactory.createTitledBorder(e, "History");
        TitledBorder t2 = BorderFactory.createTitledBorder(e, "Image Display");
        t1.setTitleJustification(TitledBorder.LEFT);
        t2.setTitleJustification(TitledBorder.LEFT);

        history_ = new JTextArea();
        history_.setEditable(false);

        JScrollPane historyPane = new JScrollPane(history_);
        historyPane.setPreferredSize(new Dimension(124, 240));

        imageDisplay_ = new ImageDisplay();

        JScrollPane imagePane = new JScrollPane(imageDisplay_);
        imagePane.setPreferredSize(new Dimension(260, 240));
        imagePane.setViewportBorder(
            BorderFactory.createLineBorder(Color.black));

        JPanel p1 = new JPanel(new BorderLayout());
        p1.setBorder(t1);
        p1.add(historyPane, BorderLayout.CENTER);

        JPanel p2 = new JPanel(new BorderLayout());
        p2.setBorder(t2);
        p2.add(imagePane, BorderLayout.CENTER);

        JSplitPane sp = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, p1, p2);
        sp.setOneTouchExpandable(true);

        getContentPane().add(sp);
        pack();
        setVisible(true);

        aboutDialog_ = new InfoDialog(
            frame_, "About ImageShow", new java.io.File("./about.html"));

        //
        // Create and activate the servant for the Hello object
        //
        imageShow_ = new ImageShow_impl(this);

        addWindowListener(new WindowAdapter()
            {
                public void
                windowClosing(WindowEvent dummyEvent)
                {
                    imageShow_.quit();
                }
            });

        String imageShowBuf = "ImageShow";
        byte[] oid = imageShowBuf.getBytes();
        try
        {
            poa.activate_object_with_id(oid, imageShow_);
        }
        catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
        {
            ex.printStackTrace();
            throw new RuntimeException();
        }
        catch(org.omg.PortableServer.POAPackage.ObjectAlreadyActive ex)
        {
            ex.printStackTrace();
            throw new RuntimeException();
        }
        catch(org.omg.PortableServer.POAPackage.ServantAlreadyActive ex)
        {
            ex.printStackTrace();
            throw new RuntimeException();
        }

        //
        // Save a reference to the Hello object
        //
        org.omg.CORBA.Object refObj = null;
        try
        {
            refObj = poa.id_to_reference(oid);
        }
        catch(org.omg.PortableServer.POAPackage.WrongPolicy ex)
        {
            ex.printStackTrace();
            throw new RuntimeException();
        }
        catch(org.omg.PortableServer.POAPackage.ObjectNotActive ex)
        {
            ex.printStackTrace();
            throw new RuntimeException();
        }

        //
        // Save reference
        //
        try
        {
            String ref = orb.object_to_string(refObj);
            String refFile = "ImageShow.ref";
            java.io.FileOutputStream file =
                new java.io.FileOutputStream(refFile);
            java.io.PrintWriter out = new java.io.PrintWriter(file);
            out.println(ref);
            out.flush();
            file.close();
        }
        catch(java.io.IOException ex)
        {
            System.err.println("Can't write to `" + ex.getMessage() + "'");
            System.exit(1);
        }
    }

    //
    // Define menus and actions
    // 
    protected void 
    addMenus(JMenuBar menuBar) 
    { 
        JMenu menu = null;
        JMenuItem menuItem = null;

        menuBar.add(menu = new JMenu("File"));
        menuItem = new JMenuItem("Quit");
        menu.add(menuItem);
        menuItem.addActionListener(new ActionListener()
        {
            public void
            actionPerformed(ActionEvent e)
            {
                imageShow_.quit();
            }
        });

        menuBar.add(menu = new JMenu("Options"));

        menuItem = new JMenuItem("Clear History");
        menu.add(menuItem);
        menuItem.addActionListener(new ActionListener()
        {
            public void
            actionPerformed(ActionEvent e)
            {
                clearHistory();
            }
        });

        menuBar.add(Box.createHorizontalGlue());

        menuBar.add(menu = new JMenu("Help"));
        menuItem = new JMenuItem("About ImageShow...");
        menu.add(menuItem);
        menuItem.addActionListener(new ActionListener()
        {
            public void
            actionPerformed(ActionEvent e)
            {
                aboutDialog_.popUp();
            }
        });
    }

    //
    // Add image to list 
    //
    public void
    showImage(String name, byte imageByte [])
    {
        history_.append(name + "\n");
        imageDisplay_.setIcon(new ImageIcon(imageByte));
        imageDisplay_.revalidate();
    }

    //
    // Reset image list
    //
    private void
    clearHistory()
    {
        history_.setText("");
        imageDisplay_.setIcon(null);
        imageDisplay_.revalidate();
    }
}
