// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA  
//                  
// All Rights Reserved   
//   
// **********************************************************************

package imageshow;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;

final class SenderWindow extends JFrame
{
    private final int RESET = 0;
    private final int PLAY = 1;
    private final int STOP_PAUSE = 2;

    //
    // The ImageShow object reference
    //
    private ImageShow imageShow_;

    //
    // Reference to the ORB.
    //
    private org.omg.CORBA.ORB orb_;

    //
    // The "Widgets"
    //
    private JCheckBoxMenuItem thumbsCheck_;
    private JCheckBoxMenuItem loopCheck_;
    private JMenuItem playMenuItem_;
    private JMenuItem stopMenuItem_;
    private JMenuItem pauseMenuItem_;
    private JButton play_;
    private JButton stop_;
    private JButton pause_;
    private JToggleButton thumbsButton_;
    private JToggleButton loopButton_;
    private JFrame frame_;
    private JList imageList_;
    private JTextArea messages_;
    private JFileChooser chooser_;

    private DefaultListModel listModel_;

    private ImagePreview thumbsPane_;
    private InfoDialog aboutDialog_;
    private Timer timer_;

    //
    // Extra globals
    //
    private int listSize_;
    private int listIndex_;

    //
    // Actions
    //
    private AbstractAction playAction_;
    private AbstractAction stopAction_;
    private AbstractAction pauseAction_;
    private AbstractAction addAction_;
    private AbstractAction removeAction_;
    private AbstractAction thumbsAction_;
    private AbstractAction loopAction_;
    private AbstractAction quitAction_;
    private AbstractAction delayAction_;
    private AbstractAction clearAction_;
    private AbstractAction aboutAction_;
    private AbstractAction timerAction_;
    private SelectAction selectAction_;
    private CheckBoxAction checkBoxAction_;

    //
    // Image Icons
    //
    private ImageIcon playIcon_;
    private ImageIcon stopIcon_;
    private ImageIcon pauseIcon_;
    private ImageIcon addIcon_;
    private ImageIcon removeIcon_;
    private ImageIcon thumbsOnIcon_;
    private ImageIcon thumbsOffIcon_;
    private ImageIcon loopOnIcon_;
    private ImageIcon loopOffIcon_;

    //
    // Constructor
    //
    SenderWindow(ImageShow imageShow, org.omg.CORBA.ORB orb, String args[])
    {
        super("ImageShow Demo: Sender");

        imageShow_ = imageShow;
        orb_ = orb;

        //
        // Create server window
        //
        frame_ = (JFrame)this;

        initializeActions();
        initializeIcons();

        addWindowListener(new WindowAdapter() 
        { 
            public void 
            windowClosing(WindowEvent e)   
            {  
                System.exit(0);
            } 
        });

        //
        // Build UI
        //
        Border e = BorderFactory.createEtchedBorder();
        TitledBorder t1 = 
            BorderFactory.createTitledBorder(e, "Queue");
        TitledBorder t2 = 
            BorderFactory.createTitledBorder(e, "Messages");
        t1.setTitleJustification(TitledBorder.LEFT);
        t2.setTitleJustification(TitledBorder.LEFT);

        JToolBar toolBar = createToolBar();

        JMenuBar menuBar = createMenuBar();;
        setJMenuBar(menuBar);

        listModel_ = new DefaultListModel();
        imageList_ = new JList(listModel_);
        JScrollPane listPane = new JScrollPane(imageList_);

        messages_ = new JTextArea();
        messages_.setEditable(false);
        JScrollPane msgPane = new JScrollPane(messages_);

        thumbsPane_ = new ImagePreview();
        imageList_.addListSelectionListener(selectAction_);

        JPanel p1 = new JPanel();
        p1.add(thumbsPane_); 

        JPanel p2 = new JPanel(new BorderLayout());
        p2.setBorder(t1);
        p2.add(p1, BorderLayout.EAST);
        p2.add(listPane, BorderLayout.CENTER);
        p2.setPreferredSize(new Dimension(384, 150));

        JPanel p3 = new JPanel(new BorderLayout());
        p3.setBorder(t2);
        p3.add(msgPane, BorderLayout.CENTER);
        p3.setPreferredSize(new Dimension(384, 90));

        JSplitPane sp = new JSplitPane(JSplitPane.VERTICAL_SPLIT, p2, p3);
        sp.setOneTouchExpandable(true);

        Container container = getContentPane();
        container.setLayout(new BorderLayout());
        container.add(toolBar, BorderLayout.NORTH);
        container.add(sp, BorderLayout.CENTER);

        //
        // File Selection Dialog
        //
        chooser_ = new JFileChooser(new java.io.File("."));
        chooser_.setFileFilter(new ImageFileFilter());
        addCmdLineImages(args); 

        pack();
        setVisible(true);

        aboutDialog_ = new InfoDialog(
            frame_, "About ImageShow", new java.io.File("about.html"));

        //
        // Timer used for sending
        //
        timer_ = new Timer(1500, timerAction_);

        //
        // initialize the list index pointer
        //
        listIndex_ = 0;
    }

    //
    // Define actions
    //
    private void
    initializeActions()
    {
        playAction_ = new PlayAction();
        stopAction_ = new StopAction();
        pauseAction_ = new PauseAction();
        addAction_ = new AddAction();
        removeAction_ = new RemoveAction();
        thumbsAction_ = new ThumbsToggleAction();
        loopAction_ = new LoopToggleAction();
        quitAction_ = new QuitAction();
        delayAction_ = new DelayAction();
        clearAction_ = new ClearAction();
        aboutAction_ = new AboutAction();
        timerAction_ = new TimerAction();
        selectAction_ = new SelectAction();
        checkBoxAction_ = new CheckBoxAction();
    }

    //
    // Define image icons
    //
    private void 
    initializeIcons()
    {
        playIcon_ = new ImageIcon("images/play.gif");
        stopIcon_ = new ImageIcon("images/stop.gif");
        pauseIcon_ = new ImageIcon("images/pause.gif");
        addIcon_ = new ImageIcon("images/new_image.gif");
        removeIcon_ = new ImageIcon("images/delete.gif");
        thumbsOnIcon_ = new ImageIcon("images/thumbnails.gif");
        thumbsOffIcon_ = new ImageIcon("images/thumbnails_bw.gif");
        loopOnIcon_ = new ImageIcon("images/loop.gif");
        loopOffIcon_ = new ImageIcon("images/loop_bw.gif");
    }

    //
    // Create toolbar
    //
    private JToolBar 
    createToolBar() 
    {
        JButton button = null;
        JToolBar toolBar = new JToolBar();

        toolBar.setFloatable(false);

        play_ = toolBar.add(playAction_);
        play_.setIcon(playIcon_);
        play_.setToolTipText("Start Show");
        play_.setEnabled(false);

        stop_ = toolBar.add(stopAction_);
        stop_.setIcon(stopIcon_);
        stop_.setToolTipText("Stop Show");
        stop_.setEnabled(false);

        pause_ = toolBar.add(pauseAction_);
        pause_.setIcon(pauseIcon_);
        pause_.setToolTipText("Pause Show");
        pause_.setEnabled(false);

        toolBar.addSeparator();

        button = toolBar.add(addAction_);
        button.setIcon(addIcon_);
        button.setToolTipText("Add Image");

        button = toolBar.add(removeAction_); 
        button.setIcon(removeIcon_);
        button.setToolTipText("Remove Image");

        toolBar.addSeparator();

        toolBar.add(thumbsButton_ = new JToggleButton(thumbsOnIcon_));
        thumbsButton_.setSelectedIcon(thumbsOffIcon_);
        thumbsButton_.setToolTipText("Thumbnails");
        thumbsButton_.setMargin(new Insets(0, 0, 0, 0));
        thumbsButton_.addActionListener(thumbsAction_);
        thumbsButton_.setSelected(false);

        toolBar.add(loopButton_ = new JToggleButton(loopOnIcon_));
        loopButton_.setSelectedIcon(loopOffIcon_);
        loopButton_.setToolTipText("Loop Show");
        loopButton_.setMargin(new Insets(0, 0, 0, 0));
        loopButton_.addActionListener(loopAction_);
        loopButton_.setSelected(false);

        return toolBar;
    }

    // 
    // Create menubar 
    //
    private JMenuBar 
    createMenuBar()
    {
        JMenu menu = null;
        JMenuItem menuItem = null;
        JMenuBar menuBar = new JMenuBar();

        menu = menuBar.add(new JMenu("File"));

        menuItem = new JMenuItem("Add Image");
        menuItem.addActionListener(addAction_);
        menu.add(menuItem);

        menuItem = new JMenuItem("Remove Image");
        menuItem.addActionListener(removeAction_);
        menu.add(menuItem);

        menu.addSeparator();

        menuItem = new JMenuItem("Quit");
        menuItem.addActionListener(quitAction_);
        menu.add(menuItem);

        menu = menuBar.add(new JMenu("Show"));

        playMenuItem_ = new JMenuItem("Start");
        playMenuItem_.addActionListener(playAction_);
        playMenuItem_.setEnabled(false);
        menu.add(playMenuItem_);

        stopMenuItem_ = new JMenuItem("Stop");
        stopMenuItem_.addActionListener(stopAction_);
        stopMenuItem_.setEnabled(false);
        menu.add(stopMenuItem_);

        pauseMenuItem_ = new JMenuItem("Pause");
        pauseMenuItem_.addActionListener(pauseAction_);
        pauseMenuItem_.setEnabled(false);
        menu.add(pauseMenuItem_);

        menu = menuBar.add(new JMenu("Options"));

        menuItem = new JMenuItem("Set Delay");
        menuItem.addActionListener(delayAction_);
        menu.add(menuItem);

        menuItem = new JMenuItem("Clear Messages"); 
        menuItem.addActionListener(clearAction_); 
        menu.add(menuItem); 

        menu.addSeparator();

        thumbsCheck_ = new JCheckBoxMenuItem("Thumbnails");
        thumbsCheck_.setSelected(true);
        thumbsCheck_.addItemListener(checkBoxAction_); 
        menu.add(thumbsCheck_);

        loopCheck_ = new JCheckBoxMenuItem("Loop Show");
        loopCheck_.setSelected(true);
        loopCheck_.addItemListener(checkBoxAction_);
        menu.add(loopCheck_);

        menuBar.add(Box.createHorizontalGlue());
        menu = menuBar.add(new JMenu("Help"));

        menuItem = new JMenuItem("About ImageShow...");
        menuItem.addActionListener(aboutAction_);
        menu.add(menuItem);

        return menuBar;
    }

    //
    // Add images requested at command line.
    //
    private void
    addCmdLineImages(String ImageNames[]) 
    {
        java.io.File file = null;
        int size = ImageNames.length;

        for(int i = 0; i < size; i++ ) 
        {
            file = new java.io.File(ImageNames[i]);

            if(file.isFile())
            {
                try 
                {
                    newImageFile(new java.io.File(file.getCanonicalPath()));
                }
                catch(java.io.IOException e)
                {
                    newMessage("error: could not form canonical path for " 
                        + ImageNames[i]); 
                }
            }
            else
            {
                newMessage("did not find file(s) " + ImageNames[i]);
            }
        }
    }

    //
    // Toggle play/stop/pause buttons
    //
    private void
    setControl(int control)
    {
        switch(control) 
        {
            case RESET:
                play_.setEnabled(false);
                stop_.setEnabled(false);
                pause_.setEnabled(false);
                playMenuItem_.setEnabled(false);
                stopMenuItem_.setEnabled(false);
                pauseMenuItem_.setEnabled(false);
                break;
            case PLAY:
                play_.setEnabled(true);
                stop_.setEnabled(false);
                pause_.setEnabled(false);
                playMenuItem_.setEnabled(true);
                stopMenuItem_.setEnabled(false);
                pauseMenuItem_.setEnabled(false);
                break;
            case STOP_PAUSE:
                play_.setEnabled(false);
                stop_.setEnabled(true);
                pause_.setEnabled(true);
                playMenuItem_.setEnabled(false);
                stopMenuItem_.setEnabled(true);
                pauseMenuItem_.setEnabled(true);
        }
    }

    //
    // Add an image to the queue
    //
    private void 
    newImageFile(java.io.File image)
    {
        listModel_.addElement(image);
        listSize_ = listModel_.getSize();
        if(listSize_ > 0)
        {
            setControl(PLAY);
            imageList_.setSelectedIndex(0);            
        }
    }

    // 
    // Display a new message
    // 
    public void 
    newMessage(String msg)
    {
        messages_.append(msg + "\n");
    }

    // 
    // Send images
    //
    private void 
    sendImage(java.io.File file)    
    {  
        java.io.InputStream in;
        byte [] buffer = new byte[500000];   
        String filename = file.getPath();

        try  
        {   
            in = new java.io.FileInputStream(filename); 
        } 
        catch(java.io.FileNotFoundException ex) 
        { 
            newMessage("Can't read from `" + ex.getMessage() + "'");  
            return;   
        }    

        try 
        {     
            int len = 0;  
            while(in.available() > 0)   
            {   

                len = in.read(buffer);  
                if(len < 500000)  
                {   
                    byte [] buf2 = new byte[len]; 
                    System.arraycopy(buffer, 0, buf2, 0, len); 
                    imageShow_.show(file.getName(), buf2);      
                }    
                else    
                {  
                    imageShow_.show(file.getName(), buffer); 
                }   
                newMessage("read " + len + " KBytes from " + filename);  
            } 
            in.close();    
            newMessage("end client");
        }  
        catch(java.io.IOException ex) 
        {     
            newMessage("Can't read from `" + ex.getMessage() + "'"); 
            return ; 
        }  
    }

    //
    // Shutdown the client
    //
    private void 
    shutdown()    
    {
        if(orb_ != null)
        {
            try
            {
                orb_.destroy();
            }
            catch(Exception ex)
            {
                ex.printStackTrace();
                System.exit(1);
            }
        }
    }

    //
    // Add image listener
    //
    class AddAction extends AbstractAction
    {
        public void                                                         
        actionPerformed(ActionEvent e)                                      
        {
            int returnVal = 
                chooser_.showOpenDialog(SenderWindow.this);

            if(returnVal == JFileChooser.APPROVE_OPTION) 
            { 
                java.io.File file = chooser_.getSelectedFile();
                newImageFile(file);
            }
        }
    }

    //
    // Remove Image Listener
    //
    class RemoveAction extends AbstractAction
    {
        public void                                                         
        actionPerformed(ActionEvent e)                                      
        {
            int index = imageList_.getSelectedIndex();
            if(index > -1) 
            { 
                listModel_.remove(index);
                listSize_ = listModel_.getSize();

                if(listSize_ > 0) 
                {
                    if(index < listSize_)
                        imageList_.setSelectedIndex(index);
                    else
                        imageList_.setSelectedIndex(listSize_ - 1);
                }
                else  
                {
                    setControl(RESET);
                    if(timer_.isRunning())
                    {
                        newMessage("Stop Show");
                        timer_.stop();
                        listIndex_ = 0;
                    }
                }
                thumbsPane_.update();
            }
        }
    }

    //
    // Class handles play action
    //
    class PlayAction extends AbstractAction
    {
        public void                                                         
        actionPerformed(ActionEvent e)                                      
        {
            newMessage("Start Show");
            setControl(STOP_PAUSE);
            timer_.start();
        }
    }

    //
    // Class handles stop action
    //
    class StopAction extends AbstractAction
    {
        public void                                                         
        actionPerformed(ActionEvent e)                                      
        {
            newMessage("Stop Show");
            setControl(PLAY);
            timer_.stop();
            imageList_.setSelectedIndex(0);
            listIndex_ = 0;
        }
    }

    //
    // Class handles pause action
    //
    class PauseAction extends AbstractAction
    {
        public void                                                         
        actionPerformed(ActionEvent e)                                      
        {
            if(timer_.isRunning())
            {
                newMessage("Pause Show");
                pause_.setToolTipText("Resume Show");
                pauseMenuItem_.setText("Resume");
                setControl(STOP_PAUSE);
                timer_.stop();
            }
            else
            {
                newMessage("Resume Show");
                pause_.setToolTipText("Pause Show");
                pauseMenuItem_.setText("Pause");
                setControl(STOP_PAUSE);
                timer_.start();
            }
        }
    }

    //
    // Handle thumbnails toggle button actions
    //
    class ThumbsToggleAction extends AbstractAction
    {
        public void                                                         
        actionPerformed(ActionEvent e)                                      
        {
            thumbsCheck_.setSelected(!thumbsButton_.isSelected());
            thumbsPane_.update();
        }
    }

    //
    // Handle loop toggle button actions
    //
    class LoopToggleAction extends AbstractAction
    {
        public void                                                         
        actionPerformed(ActionEvent e)                                      
        {
            loopCheck_.setSelected(!loopButton_.isSelected());
        }
    }

    // 
    // Quit Action
    //
    class QuitAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            SenderWindow.this.shutdown();
            System.exit(0);
        }
    }

    //
    // Clear the message area action
    //
    class ClearAction extends AbstractAction
    {
        public void 
        actionPerformed(ActionEvent e) 
        { 
            messages_.setText(""); 
        } 
    }

    // 
    // Handle checkbox actions
    //
    class CheckBoxAction implements ItemListener
    {
        public void itemStateChanged(ItemEvent e) 
        {
            Object src = e.getItemSelectable();

            if(src == thumbsCheck_) 
            {
                thumbsButton_.setSelected(!thumbsCheck_.isSelected());
                thumbsPane_.update();
            }
            else if(src == loopCheck_)
            {
                loopButton_.setSelected(!loopCheck_.isSelected());
            }
        }
    }

    //
    // Display the about dialog
    //
    class AboutAction extends AbstractAction
    {
        public void
        actionPerformed(ActionEvent e)
        {
            aboutDialog_.popUp();
        }
    }

    //
    // Update thumbnail listener
    //
    class SelectAction implements ListSelectionListener
    {
        public void 
        valueChanged(ListSelectionEvent e) 
        {
            if(e.getValueIsAdjusting())
            {
                return;
            }

            listIndex_ = imageList_.getSelectedIndex();

            //
            // Display new thumbnail
            //
            thumbsPane_.update();
        }
    }

    //
    // Dialog to prompt for new delay setting.
    //
    class DelayAction extends AbstractAction
    {
        public void                                                         
        actionPerformed(ActionEvent e)                                      
        {
            int delay = 0;
            String str = (String)JOptionPane.showInputDialog(frame_, 
                                    "Delay (millisecs)", 
                                    "Delay Setting", 
                                    JOptionPane.QUESTION_MESSAGE, 
                                    null, null, 
                                    String.valueOf(timer_.getDelay()));
            try 
            {
                delay = Integer.parseInt(str);
                if(delay < 0)
                {
                    return;
                }
            }
            catch(NumberFormatException nfe) 
            {
                return;
            }

            // 
            // set valid delay interval
            //
            timer_.setDelay(delay);
        }
    }

    //
    // Handle events during mulitcast
    //
    class TimerAction extends AbstractAction  
    {
        public void 
        actionPerformed(ActionEvent evt) 
        {
            imageList_.setSelectedIndex(listIndex_);            
            sendImage((java.io.File)listModel_.getElementAt(listIndex_));
            listIndex_++;

            // 
            // check for end of image list
            //
            if(listIndex_ >= listSize_)
            {
                //
                // loop show, so reset index to list head
                //
                listIndex_ = 0;

                if(!loopCheck_.isSelected())
                {
                    newMessage("Stop Show");
                    timer_.stop();
                    setControl(PLAY);
                    imageList_.setSelectedIndex(listIndex_);            
                }
            }
        }
    }

    //
    // Class handles thumbnail duties
    //
    class ImagePreview extends JLabel 
    {
        //
        // Constructor
        //
        ImagePreview() 
        {
            setVerticalAlignment(SwingConstants.TOP);    
            setBorder(BorderFactory.createEtchedBorder());
            setPreferredSize(new Dimension(100, 100));
        }

        // 
        // Scale and display thumbnail from image
        //
        public void 
        update() 
        {
            if(imageList_.isSelectionEmpty() 
                || !thumbsCheck_.isSelected() || listSize_ <= 0) 
            {
                setIcon(null);
                return;
            }

            java.io.File file =
                (java.io.File)listModel_.getElementAt(listIndex_);
            ImageIcon thumbnail = null;

            //
            // Scale image
            //
            ImageIcon tmpIcon = new ImageIcon(file.getPath());
            if(tmpIcon.getIconWidth() > 100) 
            {
                thumbnail = new ImageIcon(
                    tmpIcon.getImage().getScaledInstance(100, -1,
                        Image.SCALE_DEFAULT));
            } 
            else 
            {
                thumbnail = tmpIcon;
            }

            //
            // set thumbnail
            //
            setIcon(thumbnail);
        }
    }
}
