// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

import com.ooc.CORBA.InputStream;
import com.ooc.CORBA.OutputStream;

final public class Util
{
    static public boolean
    extractProfileInfo(org.omg.IOP.IOR ior,
                       com.ooc.OCI.ProfileInfoHolder profileInfo)
    {
        com.ooc.OCI.ProfileInfoSeqHolder profileInfoSeq =
            new com.ooc.OCI.ProfileInfoSeqHolder();
        profileInfoSeq.value = new com.ooc.OCI.ProfileInfo[0];
        extractAllProfileInfos(ior, profileInfoSeq, false, null, 0, false);
        if(profileInfoSeq.value.length > 0)
        {
            com.ooc.OB.Assert._OB_assert(profileInfoSeq.value.length == 1);
            profileInfo.value = profileInfoSeq.value[0];
            return true;
        }

        return false;
    }

    static public void
    extractAllProfileInfos(org.omg.IOP.IOR ior,
                           com.ooc.OCI.ProfileInfoSeqHolder profileInfoSeq,
                           boolean performMatch,
                           String host, int port,
                           boolean loopbackMatches)
    {
        short portNo;
        if(port >= 0x8000)
            portNo = (short)(port - 0xffff - 1);
        else
            portNo = (short)port;

        java.util.Vector vec = new java.util.Vector();
        for(int i = 0; i < ior.profiles.length; i++)
        {
            if(ior.profiles[i].tag == com.ooc.UDPIOP.TAG_UDP_IOP.value)
            {
                //
                // Get the UDP profile body
                //
                byte[] data = ior.profiles[i].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                InputStream in = new InputStream(buf, 0, false);
                in._OB_readEndian();
                com.ooc.UDPIOP.ProfileBody body =
                    com.ooc.UDPIOP.ProfileBodyHelper.read(in);

                if(performMatch)
                {
                    //
                    // Compare ports
                    //
                    if(portNo != body.port)
                        continue;

                    //
                    // Direct host name comparison
                    //
                    if(!host.equals(body.host))
                    {
                        //
                        // Direct host name comparision failed - must look up
                        // addresses to be really sure if the hosts differ
                        //

                        try
                        {
                            java.net.InetAddress addr1 =
                                java.net.InetAddress.getByName(host);

                            java.net.InetAddress addr2 =
                                java.net.InetAddress.getByName(body.host);

                            if(!addr1.equals(addr2))
                            {
                                //
                                // Address comparison failed - shall I extract
                                // the key if the profile body contains the
                                // loopback address?
                                //
                                if(loopbackMatches)
                                {
                                    java.net.InetAddress loopback =
                                        java.net.InetAddress.getByName(
                                            "127.0.0.1");

                                    if(!addr2.equals(loopback))
                                        continue;
                                }
                                else
                                    continue;
                            }
                        }
                        catch(java.net.UnknownHostException ex)
                        {
                            //
                            // Continue on hostname lookup failure
                            //
                            continue;
                        }
                    }
                }

                //
                // OK, found a match
                //
                com.ooc.OCI.ProfileInfo profileInfo =
                    new com.ooc.OCI.ProfileInfo();
                profileInfo = new com.ooc.OCI.ProfileInfo();
                profileInfo.key = body.object_key;
                profileInfo.minor = body.giop_version.minor;
                profileInfo.major = body.giop_version.major;
                profileInfo.id = ior.profiles[i].tag;
                profileInfo.components = body.components;

                vec.addElement(profileInfo);
            }
        }

        if(vec.size() > 0)
        {
            int len = profileInfoSeq.value.length;
            if(len == 0)
            {
                profileInfoSeq.value = new com.ooc.OCI.ProfileInfo[vec.size()];
                vec.copyInto(profileInfoSeq.value);
            }
            else
            {
                com.ooc.OCI.ProfileInfo[] arr =
                    new com.ooc.OCI.ProfileInfo[len + vec.size()];
                System.arraycopy(profileInfoSeq.value, 0, arr, 0, len);
                for(int i = 0 ; i < vec.size() ; i++)
                    arr[len + i] = (com.ooc.OCI.ProfileInfo)vec.elementAt(i);
                profileInfoSeq.value = arr;
            }
        }
    }

    //
    // Compare two IORs containing UDP profiles
    //
    static public boolean
    equivalent(org.omg.IOP.IOR ior1, org.omg.IOP.IOR ior2)
    {
        int p1, p2, b1, b2;
        int cnt1 = 0, cnt2 = 0;
        com.ooc.UDPIOP.ProfileBody[] bodies1;
        com.ooc.UDPIOP.ProfileBody[] bodies2;

        //
        // Calculate number of UDPIOP profiles in ior1
        //
        for(p1 = 0; p1 < ior1.profiles.length; p1++)
            if(ior1.profiles[p1].tag == com.ooc.UDPIOP.TAG_UDP_IOP.value)
                cnt1++;

        //
        // Calculate number of UDPIOP profiles in ior2
        //
        for(p2 = 0; p2 < ior2.profiles.length; p2++)
            if(ior2.profiles[p2].tag == com.ooc.UDPIOP.TAG_UDP_IOP.value)
                cnt2++;

        //
        // Return false now if the number of UDPIOP profile bodies do
        // not match
        //
        if(cnt1 != cnt2)
            return false;

        //
        // Create an array with all UDPIOP profile bodies of ior1
        //
        bodies1 = new com.ooc.UDPIOP.ProfileBody[cnt1];
        for(p1 = 0, b1 = 0; p1 < ior1.profiles.length; p1++)
            if(ior1.profiles[p1].tag == com.ooc.UDPIOP.TAG_UDP_IOP.value)
            {
                byte[] data = ior1.profiles[p1].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                InputStream in = new InputStream(buf, 0, false);
                in._OB_readEndian();
                bodies1[b1++] = com.ooc.UDPIOP.ProfileBodyHelper.read(in);
            }

        if(b1 != cnt1)
            throw new InternalError();

        //
        // Create an array with all UDPIOP profile bodies of ior2
        //
        bodies2 = new com.ooc.UDPIOP.ProfileBody[cnt2];
        for(p2 = 0, b2 = 0; p2 < ior2.profiles.length; p2++)
            if(ior2.profiles[p2].tag == com.ooc.UDPIOP.TAG_UDP_IOP.value)
            {
                byte[] data = ior2.profiles[p2].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                InputStream in = new InputStream(buf, 0, false);
                in._OB_readEndian();
                bodies2[b2++] = com.ooc.UDPIOP.ProfileBodyHelper.read(in);
            }

        if(b2 != cnt2)
            throw new InternalError();

        //
        // Check for profile body matches
        //
        for(b1 = 0 ; b1 < cnt1 ; b1++)
        {
            for(b2 = 0 ; b2 < cnt2 ; b2++)
            {
                if(bodies2[b2] == null)
                    continue;

                //
                // Compare profile bodies
                //
                if(!compareBodies(bodies1[b1], bodies2[b2]))
                    continue;

                //
                // OK, found a match
                //
                bodies1[b1] = null;
                bodies2[b2] = null;
            }
        }

        //
        // Check whether there are any unmatched UDPIOP profile bodies
        //
        for(b1 = 0; b1 < cnt1; b1++)
            if(bodies1[b1] != null)
                return false;

        for(b2 = 0; b2 < cnt2; b2++)
            if(bodies2[b2] != null)
                return false;

        //
        // The two IORs are equal.
        //
        return true;
    }

    static public boolean
    compareBodies(com.ooc.UDPIOP.ProfileBody body1,
                  com.ooc.UDPIOP.ProfileBody body2)
    {
        //
        // Compare versions
        //
        if(body1.udpiop_version.major != body2.udpiop_version.major ||
           body1.udpiop_version.minor != body2.udpiop_version.minor)
            return false;
        if(body1.giop_version.major != body2.giop_version.major ||
           body1.giop_version.minor != body2.giop_version.minor)
            return false;

        //
        // Compare ports
        //
        if(body1.port != body2.port)
            return false;

        //
        // Compare object keys
        //
        if(body1.object_key.length != body2.object_key.length)
            return false;

        int k;
        for(k = 0 ; k < body1.object_key.length ; k++)
            if(body1.object_key[k] != body2.object_key[k])
                return false;

        if(k < body1.object_key.length)
            return false;

        //
        // Direct host name comparison
        //
        if(!body1.host.equals(body2.host))
        {
            //
            // Direct host name comparision failed - must look up
            // addresses to be really sure if the hosts differ
            //
            try
            {
                java.net.InetAddress addr1 =
                    java.net.InetAddress.getByName(body1.host);

                java.net.InetAddress addr2 =
                    java.net.InetAddress.getByName(body2.host);

                if(!addr1.equals(addr2))
                    return false;
            }
            catch(java.net.UnknownHostException ex)
            {
                //
                // Return false on hostname lookup failure
                //
                return false;
            }
        }

        //
        // OK, found a match
        //
        return true;
    }

    //
    // Calculate a hash for an IOR containing IIOP profiles
    //
    static int
    hash(org.omg.IOP.IOR ior, int maximum)
    {
        int hash = 0;

        for(int i = 0; i < ior.profiles.length; i++)
        {
            if(ior.profiles[i].tag == com.ooc.UDPIOP.TAG_UDP_IOP.value)
            {
                //
                // Get the UDP profile body
                //
                byte[] data = ior.profiles[i].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                InputStream in = new InputStream(buf, 0, false);
                in._OB_readEndian();
                com.ooc.UDPIOP.ProfileBody body =
                    com.ooc.UDPIOP.ProfileBodyHelper.read(in);


                //
                // Add port to hash
                //
                hash ^= body.port;

                //
                // Add object key to hash
                //
                for(int j = 0; j + 1 < body.object_key.length; j += 2)
                    hash ^= body.object_key[j + 1] * 256 +
                        body.object_key[j];
            }
        }

        return hash % (maximum + 1);
    }
}
