// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.LoadBalancing;

//
// ORB Initializer used to initialize a server for load balancing.
//
final public class BalancerInitializer
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ORBInitializer
{
    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    public void
    pre_init(org.omg.PortableInterceptor.ORBInitInfo info)
    {
    }

    public void
    post_init(org.omg.PortableInterceptor.ORBInitInfo info)
    {
	com.ooc.OBPortableInterceptor.ORBInitInfo obInfo =
	    com.ooc.OBPortableInterceptor.ORBInitInfoHelper.narrow(info);
	com.ooc.CORBA.ORB orb = (com.ooc.CORBA.ORB) obInfo.orb();

	//
	// Register the member policy factory.
	//
	org.omg.PortableInterceptor.PolicyFactory pf =
	    new MemberPolicyFactory_impl();
	info.register_policy_factory(MEMBER_POLICY_ID.value, pf);

	//
	// Register the GroupORT value factory.
	//
	org.omg.CORBA.portable.ValueFactory vf = new GroupORTFactory_impl();
	orb.register_value_factory(GroupORTHelper.id(), vf);

	//
	// Get the GroupFactory object.
	//
	GroupFactory factory = null;
	try
	{
	    factory = GroupFactoryHelper.narrow(
		orb.resolve_initial_references("Balancer"));
	}
	catch(org.omg.CORBA.ORBPackage.InvalidName ex)
	{
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	}
	if(factory == null)
	{
	    String msg = "unable to locate the Balancer";
	    throw new org.omg.CORBA.INITIALIZE(msg);
	}

	//
	// Create the LBIORInterceptor.
	//
	org.omg.PortableInterceptor.IORInterceptor iorInterceptor =
	    new LBIORInterceptor(factory);

	//
	// Register the IORInterceptor.
	//
	try
	{
	    info.add_ior_interceptor(iorInterceptor);
	}
	catch(org.omg.PortableInterceptor.ORBInitInfoPackage.DuplicateName ex)
	{
	    com.ooc.OB.Assert._OB_assert(false);
	}
    }
}

class LBIORInterceptor
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.IORInterceptor
{
    //
    // The group factory
    //
    private GroupFactory factory_;

    // ------------------------------------------------------------------
    // LBIORInterceptor private member implementation
    // ------------------------------------------------------------------

    //
    // Get the member policy value.  Will print error message and throw
    // INITIALIZE if conditions are not ment.
    //
    private MemberPolicyValue
    getPolicyValue(org.omg.PortableInterceptor.IORInfo info)
    {
	org.omg.CORBA.Policy p =
	    info.get_effective_policy(MEMBER_POLICY_ID.value);
	MemberPolicy groupPolicy = MemberPolicyHelper.narrow(p);
	MemberPolicyValue value = groupPolicy.value();

	//
	// Check the lifespan policy.
	//
	p = info.get_effective_policy(
	    org.omg.PortableServer.LIFESPAN_POLICY_ID.value);
	org.omg.PortableServer.LifespanPolicy lifespanPolicy =
	    org.omg.PortableServer.LifespanPolicyHelper.narrow(p);
	if(lifespanPolicy.value().value() !=
	   org.omg.PortableServer.LifespanPolicyValue._PERSISTENT)
	{
	    String msg = "a POA with the member policy also requires the " +
			 "PERSISTENT lifespan policy";
	    throw new org.omg.CORBA.INITIALIZE(msg);
	}

	//
	// Check the id assignment policy.
	//
	p = info.get_effective_policy(
		    org.omg.PortableServer.ID_ASSIGNMENT_POLICY_ID.value);
	org.omg.PortableServer.IdAssignmentPolicy idAssignmentPolicy =
	    org.omg.PortableServer.IdAssignmentPolicyHelper.narrow(p);
	if(idAssignmentPolicy.value().value() !=
	   org.omg.PortableServer.IdAssignmentPolicyValue._USER_ID)
	{
	    String msg = "a POA with the member policy also requires the " +
			 "USER_ID id assignment policy";
	    throw new org.omg.CORBA.INITIALIZE(msg);
	}

	return value;
    }

    // ------------------------------------------------------------------
    // LBIORInterceptor constructor
    // ------------------------------------------------------------------

    protected LBIORInterceptor(GroupFactory factory)
    {
	factory_ = factory;
    }

    // ------------------------------------------------------------------
    // IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    name()
    {
	return new String("LBIORInterceptor");
    }

    public void
    destroy()
    {
    }

    public void
    establish_components(org.omg.PortableInterceptor.IORInfo info)
    {
    }

    public void
    components_established(org.omg.PortableInterceptor.IORInfo info)
    {
	//
	// This method does nothing if the POA does not have the
	// member policy.
	//
	MemberPolicyValue value = null;
	try
	{
	    value = getPolicyValue(info);
	}
	catch(org.omg.CORBA.INV_POLICY ex)
	{
	    return;
	}

	//
	// Get the secondary object-reference template.
	//
	org.omg.PortableInterceptor.ObjectReferenceFactory factory =
	    info.current_factory();
	org.omg.PortableInterceptor.ObjectReferenceTemplate ort =  
	    (org.omg.PortableInterceptor.ObjectReferenceTemplate) factory;

	//
	// If the secondary object-reference template is not set,
	// then get the primary.
	//
	if(ort == null)
	    ort = info.adapter_template();

	//
	// Swap object reference templates with the group.
	//
	org.omg.PortableInterceptor.ObjectReferenceTemplate groupORT = null;
	try
	{
	    //
	    // Get the group.
	    //
	    Group grp = factory_.get(value.group_id);
	    InternalGroup group = InternalGroupHelper.narrow(grp);
	    
	    //
	    // Update the object-reference template in the group.
	    //
	    groupORT = group.update_member(value.member_id, ort);
	}
	catch(GroupNotFound ex)
	{
	    String msg = "group `" + value.group_id + "' does not exist";

	    throw new org.omg.CORBA.INITIALIZE(msg);
	}
	catch(MemberNotFound ex)
	{
	    String msg = "member `" + value.member_id + "' of group `" +
			 value.group_id + "' does not exist";

	    throw new org.omg.CORBA.INITIALIZE(msg);
	}
	catch(org.omg.CORBA.NO_PERMISSION ex)
	{
	    String msg = "cannot implicitely add a new member when the " +
			 "Balancer is running in non-administrative mode";

	    throw new org.omg.CORBA.INITIALIZE(msg);
	}
	catch(org.omg.CORBA.SystemException ex)
	{
	    String msg = "caught SystemException while registering " +
			 "member:\n" + ex.toString();

	    throw new org.omg.CORBA.INITIALIZE(msg);
	}

	//
	// Save the group object-reference template as the secondary
	// template.
	//
	info.current_factory(groupORT);
    }

    public void
    adapter_state_change(
        org.omg.PortableInterceptor.ObjectReferenceTemplate[] templates,
        short state)
    {
    }

    public void
    adapter_manager_state_change(int id, short state)
    {
    }
}
