// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.LoadBalancing.Util;

import com.ooc.LoadBalancing.*;

//
// Server Request Interceptor which redirects requests back to the
// Balancer.
//
final public class LoadAlertInterceptor
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ServerRequestInterceptor
{
    //
    // The ORB
    //
    private org.omg.CORBA.ORB orb_;

    //
    // The Logger
    //
    private Logger logger_;

    //
    // The POACurrent object
    //
    private org.omg.PortableServer.Current poaCurrent_;

    //
    // The alert expire time
    //
    private int expire_;

    //
    // The time when the alert started
    //
    private long start_;

    //
    // Forward the next request?
    //
    private boolean forwardNext_;

    // -----------------------------------------------------------------
    // LoadAlertInterceptor constructor
    // -----------------------------------------------------------------

    public LoadAlertInterceptor(org.omg.CORBA.ORB orb,
				Logger logger,
				int expire)
    {
	orb_ = orb;
	logger_ = logger;
	expire_ = expire;
	forwardNext_ = false;
    }

    // -----------------------------------------------------------------
    // LoadAlertInterceptor public member implementation
    // -----------------------------------------------------------------

    //
    // Redirect the next request back to the Balancer.
    //
    public synchronized void alert()
    {
	start_ = System.currentTimeMillis();

	if(forwardNext_ == false && logger_.traceLevel() >= 1)
	{
	    String msg = "load alert";

	    if(logger_.traceLevel() >= 3)
	    {
		java.text.SimpleDateFormat date =
		    new java.text.SimpleDateFormat("HH:mm:ss");

		msg = msg + " at " + date.format(new java.util.Date(start_));
	    }

	    logger_.trace("alert", msg);
	}

	forwardNext_ = true;
    }

    //
    // Retrive the alert expire time.
    //
    public synchronized int getExpire()
    {
	return expire_;
    }

    //
    // Set the alert expire time.
    //
    public synchronized void setExpire(int millis)
    {
	expire_ = millis;
    }

    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    public String
    name()
    {
	return "LoadAlertInterceptor";
    }

    public void
    destroy()
    {
    }

    public void
    receive_request_service_contexts(
    	org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
    }

    public synchronized void
    receive_request(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	if(forwardNext_)
	{
	    //
	    // Check to see if the alert has expired.
	    //
	    if(expire_ > 0)
	    {
		long now = System.currentTimeMillis();
		long elapsed = now - start_;
		if(elapsed >= expire_)
		{
		    if(logger_.traceLevel() >= 1)
		    {
			String msg = "load alert expired";

			if(logger_.traceLevel() >= 3)
			{
			    java.text.SimpleDateFormat date =
				new java.text.SimpleDateFormat("HH:mm:ss");

			    msg = msg + " at " +
				date.format(new java.util.Date(now));
			}

			logger_.trace("alert", msg);
		    }

		    forwardNext_ = false;
		    return;
		}
	    }

	    //
	    // Only forward the next request if POA has the member policy
	    // set.
	    //
	    try
	    {
		org.omg.CORBA.Policy p =
		    ri.get_server_policy(MEMBER_POLICY_ID.value);
	    }
	    catch(org.omg.CORBA.INV_POLICY ex)
	    {
		return;
	    }

	    //
	    // Get the POACurrent object if necessary.
	    //
	    if(poaCurrent_ == null)
	    {
		try
		{
		    poaCurrent_ =
			org.omg.PortableServer.CurrentHelper.narrow(
			    orb_.resolve_initial_references("POACurrent"));
		}
		catch(org.omg.CORBA.ORBPackage.InvalidName ex)
		{
		    com.ooc.OB.Assert._OB_assert(false); // Should not occur
		}
	    }

	    //
	    // Get the object reference template to create an object that
	    // refers to the Balancer.
	    //
	    org.omg.PortableServer.POA poa = null;
	    try
	    {
		poa = poaCurrent_.get_POA();
	    }
	    catch(org.omg.PortableServer.CurrentPackage.NoContext ex)
	    {
		com.ooc.OB.Assert._OB_assert(false); // Should not occur
	    }
	    com.ooc.OBPortableServer.POA obpoa =
		com.ooc.OBPortableServer.POAHelper.narrow(poa);
	    org.omg.PortableInterceptor.ObjectReferenceTemplate ort =
		(org.omg.PortableInterceptor.ObjectReferenceTemplate)
		    obpoa.current_factory();
	    if(ort == null)
		ort = obpoa.adapter_template();
	    com.ooc.OB.Assert._OB_assert(ort != null);

	    //
	    // Retrieve the object id and the repository id and then create the
	    // the object.
	    //
	    byte[] oid = ri.object_id();
	    String repoid = ri.target_most_derived_interface();
	    org.omg.CORBA.Object obj = ort.make_object(repoid, oid);

	    if(logger_.traceLevel() >= 1)
	    {
		String msg = "request redirected";

		if(logger_.traceLevel() >= 3)
		{
		    java.text.SimpleDateFormat date =
			new java.text.SimpleDateFormat("HH:mm:ss");

		    msg = msg + " at " +
			date.format(new java.util.Date(
					    System.currentTimeMillis()));
		}

		logger_.trace("alert", msg);
	    }

	    //
	    // Forward the request.
	    //
	    forwardNext_ = false;
	    throw new org.omg.PortableInterceptor.ForwardRequest(obj, true);
	}
    }
	
    public void
    send_reply(org.omg.PortableInterceptor.ServerRequestInfo ri)
    {
    }
	
    public void
    send_other(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
    }

    public void
    send_exception(org.omg.PortableInterceptor.ServerRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
    }
}
