// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

public class AccFactory_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.AccFactory
{
    //
    // The TransportManager
    //
    private TransportManager transportManager_;

    //
    // The acceptor factory delegate
    //
    private com.ooc.OCI.AccFactory delegate_;

    //
    // AccFactory information
    //
    private AccFactoryInfo_impl info_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return transportManager_.id();
    }

    public int
    tag()
    {
        return transportManager_.tag();
    }

    public com.ooc.OCI.AccFactoryInfo
    get_info()
    {
        return info_;
    }

    public com.ooc.OCI.Acceptor
    create_acceptor(String[] params)
        throws com.ooc.OCI.InvalidParam
    {
        com.ooc.OCI.Acceptor result = null;
        if(params.length > 0 && params[0].equals("--callback"))
        {
            if(params.length > 1)
                throw new com.ooc.OCI.InvalidParam("extra arguments for " +
                                                   "callback acceptor");
            if(transportManager_.hasCallbackAcceptor())
                throw new com.ooc.OCI.InvalidParam("at most one callback " +
                                                   "acceptor may be created");
            result = new CallbackAcceptor_impl(transportManager_);
        }
        else
        {
            //
            // Pass all parameters to delegate factory
            //
            com.ooc.OCI.Acceptor acc = delegate_.create_acceptor(params);
            result = new Acceptor_impl(transportManager_, acc);
        }

        return result;
    }

    public void
    change_key(org.omg.IOP.IORHolder ior, byte[] key)
    {
        //
        // Extract the BiDirIOP profile information from the provided IOR
        //
        for(int profile = 0 ; profile < ior.value.profiles.length ; profile++)
        {
            if(ior.value.profiles[profile].tag == tag())
            {
                //
                // Get ProfileBody
                //
                byte[] data = ior.value.profiles[profile].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf);
                in._OB_readEndian();
                com.ooc.BiDirIOP.ProfileBody body =
                    com.ooc.BiDirIOP.ProfileBodyHelper.read(in);

                //
                // Fill in the new object-key
                //
                body.object_key = key;

                //
                // Update the nested profiles
                //
                org.omg.IOP.IOR dummy = new org.omg.IOP.IOR();
                dummy.type_id = ior.value.type_id;
                dummy.profiles = body.profiles;
                org.omg.IOP.IORHolder dummyH =
                    new org.omg.IOP.IORHolder(dummy);
                delegate_.change_key(dummyH, key);
                body.profiles = dummyH.value.profiles;

                //
                // Remarshal the new body
                //
                com.ooc.OCI.Buffer buf2 = new com.ooc.OCI.Buffer();
                com.ooc.CORBA.OutputStream out =
                    new com.ooc.CORBA.OutputStream(buf2);
                out._OB_writeEndian();
                com.ooc.BiDirIOP.ProfileBodyHelper.write(out, body);

                ior.value.profiles[profile].profile_data =
                    new byte[buf2.length()];
                System.arraycopy(buf2.data(), 0,
                                 ior.value.profiles[profile].profile_data,
                                 0, buf2.length());
            }
        }
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    AccFactory_impl(com.ooc.OCI.AccFactory delegate,
                    TransportManager transportManager)
    {
        delegate_ = delegate;
        transportManager_ = transportManager;
        com.ooc.OCI.AccFactoryInfo info = delegate.get_info();
        info_ = new AccFactoryInfo_impl(transportManager_.id(),
                                        transportManager_.tag(), info);
    }

    TransportManager
    _OB_transportManager()
    {
        return transportManager_;
    }
}
