// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

public class Acceptor_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Acceptor
{
    //
    // The TransportManager
    //
    private TransportManager transportManager_;

    //
    // The Acceptor delegate
    //
    private com.ooc.OCI.Acceptor delegate_;

    //
    // Acceptor information
    //
    private AcceptorInfo_impl info_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return transportManager_.id();
    }

    public int
    tag()
    {
        return transportManager_.tag();
    }

    public int
    handle()
    {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public void
    close()
    {
        delegate_.close();
    }

    public void
    shutdown()
    {
        delegate_.shutdown();
    }

    public void
    listen()
    {
        //
        // This operation does nothing in the Java implementation
        //
    }

    public com.ooc.OCI.Transport
    accept(boolean block)
    {
        com.ooc.OCI.Transport delegateTransport = delegate_.accept(block);

        com.ooc.OCI.Transport t = null;
        if(delegateTransport != null)
        {
            t = transportManager_.addIncomingTransport(this,
                                                       delegateTransport);

            //
            // Call callbacks
            //
            com.ooc.OCI.TransportInfo info = t.get_info();
            try
            {
                info_._OB_callAcceptCB(info);
            }
            catch(org.omg.CORBA.SystemException ex)
            {
                t.close();
                throw ex;
            }
        }

        //
        // Return new transport
        //
        return t;
    }

    public com.ooc.OCI.Transport
    connect_self()
    {
        return delegate_.connect_self();
    }

    public void
    add_profiles(com.ooc.OCI.ProfileInfo profile_info,
                 org.omg.IOP.IORHolder ref)
    {
        //
        // Get IOR profiles from delegate using a dummy IOR
        //
        org.omg.IOP.IOR dummyIOR = new org.omg.IOP.IOR();
        dummyIOR.type_id = ref.value.type_id;
        dummyIOR.profiles = new org.omg.IOP.TaggedProfile[0];
        org.omg.IOP.IORHolder dummyIORH =
            new org.omg.IOP.IORHolder(dummyIOR);
        delegate_.add_profiles(profile_info, dummyIORH);

        //
        // Create BiDirIOP profile
        //
        com.ooc.BiDirIOP.ProfileBody body =
            new com.ooc.BiDirIOP.ProfileBody();
        body.bidiriop_version = new com.ooc.BiDirIOP.Version((byte)1, (byte)0);
        body.giop_version = new com.ooc.BiDirIOP.Version(profile_info.major,
                                                         profile_info.minor);
        body.peer = transportManager_.peer();
        body.object_key = profile_info.key;
        body.components = new org.omg.IOP.TaggedComponent[0];
        body.profiles = dummyIORH.value.profiles;

        //
        // Append the BiDirIOP profile to the IOR
        //
        int len = ref.value.profiles.length + 1;
        org.omg.IOP.TaggedProfile[] profiles =
            new org.omg.IOP.TaggedProfile[len];
        System.arraycopy(ref.value.profiles, 0, profiles, 0,
                         ref.value.profiles.length);
        ref.value.profiles = profiles;
        ref.value.profiles[len - 1] = new org.omg.IOP.TaggedProfile();
        ref.value.profiles[len - 1].tag = tag();
        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
        com.ooc.CORBA.OutputStream out = new com.ooc.CORBA.OutputStream(buf);
        out._OB_writeEndian();
        com.ooc.BiDirIOP.ProfileBodyHelper.write(out, body);
        ref.value.profiles[len - 1].profile_data = new byte[buf.length()];
        System.arraycopy(buf.data(), 0,
                         ref.value.profiles[len - 1].profile_data,
                         0, buf.length());
    }

    public com.ooc.OCI.ProfileInfo[]
    get_local_profiles(org.omg.IOP.IOR ref)
    {
        java.util.Vector seq = new java.util.Vector();
        for(int i = 0 ; i < ref.profiles.length ; i++)
        {
            if(ref.profiles[i].tag == tag())
            {
                //
                // Get ProfileBody
                //
                com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(
                    ref.profiles[i].profile_data,
                    ref.profiles[i].profile_data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf, 0, false);
                in._OB_readEndian();
                com.ooc.BiDirIOP.ProfileBody body =
                    com.ooc.BiDirIOP.ProfileBodyHelper.read(in);

                if(body.peer.equals(transportManager_.peer()) &&
                   body.profiles.length > 0)
                {
                    org.omg.IOP.IOR dummyIOR = new org.omg.IOP.IOR();
                    dummyIOR.type_id = ref.type_id;
                    dummyIOR.profiles = body.profiles;
                    com.ooc.OCI.ProfileInfo[] arr =
                        delegate_.get_local_profiles(dummyIOR);
                    if(arr.length > 0)
                    {
                        //
                        // Collect all components from delegate's profiles
                        //
                        java.util.Vector components = new java.util.Vector();
                        for(int j = 0 ; j < arr.length ; j++)
                            for(int k = 0 ; k < arr[j].components.length ; k++)
                                components.addElement(arr[j].components[k]);

                        com.ooc.OCI.ProfileInfo info =
                            new com.ooc.OCI.ProfileInfo();
                        info.major = body.giop_version.major;
                        info.minor = body.giop_version.minor;
                        info.key = body.object_key;
                        info.components =
                            new org.omg.IOP.TaggedComponent[components.size()];
                        components.copyInto(info.components);
                        seq.addElement(info);
                    }
                }
            }
        }

        com.ooc.OCI.ProfileInfo[] result =
            new com.ooc.OCI.ProfileInfo[seq.size()];
        seq.copyInto(result);
        return result;
    }

    public com.ooc.OCI.AcceptorInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    Acceptor_impl(TransportManager transportManager,
                  com.ooc.OCI.Acceptor delegate)
    {
        transportManager_ = transportManager;
        delegate_ = delegate;
        com.ooc.OCI.AcceptorInfo info = delegate.get_info();
        info_ = new AcceptorInfo_impl(transportManager.id(),
                                      transportManager.tag(),
                                      transportManager.peer(), info);
    }
}
