// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

//
// CallbackTransport_impl represents the "callback" virtual connection
//
public class CallbackTransport_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Transport, ShutdownStatus
{
    //
    // The protocol id
    //
    private String id_;

    //
    // The profile tag
    //
    private int tag_;

    //
    // The active transport
    //
    private ActiveTransport transport_;

    //
    // Transport information
    //
    private CallbackTransportInfo_impl info_;

    //
    // Have we been shutdown?
    //
    private boolean shutdown_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return id_;
    }

    public int
    tag()
    {
        return tag_;
    }

    public com.ooc.OCI.SendReceiveMode
    mode()
    {
        return com.ooc.OCI.SendReceiveMode.SendReceive;
    }

    public int
    handle()
    {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public void
    close()
    {
        shutdown_ = true;

        //
        // Call callbacks
        //
        info_._OB_callCloseCB(info_);

        transport_.close(true);
    }

    public void
    shutdown()
    {
        shutdown_ = true;

        transport_.shutdown(true);
    }

    public void
    receive(com.ooc.OCI.Buffer buf, boolean block)
    {
        //
        // There is no need to invoke the ActiveTransport if we
        // have been shutdown
        //
        if(shutdown_)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorRecv),
                com.ooc.OB.MinorCodes.MinorRecv,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        transport_.receive(buf, block, true, this);
    }

    public boolean
    receive_detect(com.ooc.OCI.Buffer buf, boolean block)
    {
        //
        // There is no need to invoke the ActiveTransport if we
        // have been shutdown
        //
        if(shutdown_)
            return false;

        return transport_.receiveDetect(buf, block, true, this);
    }

    public void
    receive_timeout(com.ooc.OCI.Buffer buf, int timeout)
    {
        //
        // There is no need to invoke the ActiveTransport if we
        // have been shutdown
        //
        if(shutdown_)
            return;

        transport_.receiveTimeout(buf, timeout, true, this);
    }

    public boolean
    receive_timeout_detect(com.ooc.OCI.Buffer buf, int timeout)
    {
        //
        // There is no need to invoke the ActiveTransport if we
        // have been shutdown
        //
        if(shutdown_)
            return false;

        return transport_.receiveTimeoutDetect(buf, timeout, true, this);
    }

    public void
    send(com.ooc.OCI.Buffer buf, boolean block)
    {
        transport_.send(buf, block, true);
    }

    public boolean
    send_detect(com.ooc.OCI.Buffer buf, boolean block)
    {
        return transport_.sendDetect(buf, block, true);
    }

    public void
    send_timeout(com.ooc.OCI.Buffer buf, int timeout)
    {
        transport_.sendTimeout(buf, timeout, true);
    }

    public boolean
    send_timeout_detect(com.ooc.OCI.Buffer buf, int timeout)
    {
        return transport_.sendTimeoutDetect(buf, timeout, true);
    }

    public com.ooc.OCI.TransportInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ShutdownStatus
    // ------------------------------------------------------------------

    public boolean
    isShutdown()
    {
        return shutdown_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    CallbackTransport_impl(String id, int tag, ActiveTransport t,
                           com.ooc.OCI.Connector connector)
    {
        id_ = id;
        tag_ = tag;
        transport_ = t;
        shutdown_ = false;
        info_ = new CallbackTransportInfo_impl(id, tag_, t.localPeer(),
                                               t.remotePeer(), connector,
                                               t.info());
    }

    CallbackTransport_impl(String id, int tag, ActiveTransport t,
                           com.ooc.OCI.Acceptor acceptor)
    {
        id_ = id;
        tag_ = tag;
        transport_ = t;
        shutdown_ = false;
        info_ = new CallbackTransportInfo_impl(id, tag_, t.localPeer(),
                                               t.remotePeer(), acceptor,
                                               t.info());
    }
}
