// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.BiDir;

public class CorbalocProtocol_impl extends org.omg.CORBA.LocalObject
                                   implements com.ooc.OB.CorbalocProtocol
{
    private String id_;
    private int tag_;
    private com.ooc.OB.CorbalocProtocol delegate_;

    CorbalocProtocol_impl(String id, int tag,
                          com.ooc.OB.CorbalocProtocol delegate)
    {
        id_ = id;
        tag_ = tag;
        delegate_ = delegate;
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    name()
    {
        return id_;
    }

    public org.omg.IOP.TaggedProfile
    parse_address(String addr, byte[] key)
    {
        //
        // The format of a bidir URL address is one of the following:
        //
        // <name>:<peer>
        // <name>:<peer>:<prot_addr>
        //
        // The first form indicates a callback-only profile, whereas
        // the second form includes address information for the
        // encapsulated protocol. The <peer> field represents the peer
        // ID of the server.
        //

        // 
        // Empty peer ID is illegal
        // 
        if(addr.length() == 0 || addr.charAt(0) == ':')
        {
            String err = name() + ": peer ID must be specified in URL";
            throw new org.omg.CORBA.BAD_PARAM(
                com.ooc.OB.MinorCodes.describeBadParam(
                    com.ooc.OB.MinorCodes.MinorBadAddress) + ": " + err,
                com.ooc.OB.MinorCodes.MinorBadAddress,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // Peer ID is terminated by ':', or by end of string
        //
        String peer;
        int colon = addr.indexOf(':');
        if(colon == -1)
            peer = addr;
        else
            peer = addr.substring(0, colon);

        //
        // Get nested profile, if necessary
        //
        org.omg.IOP.TaggedProfile nested = null;
        if(colon != -1)
            nested = delegate_.parse_address(addr.substring(colon + 1), key);

        //
        // Create profile
        //
        org.omg.IOP.TaggedProfile profile = new org.omg.IOP.TaggedProfile();
        profile.tag = tag_;

        com.ooc.BiDirIOP.ProfileBody body = new com.ooc.BiDirIOP.ProfileBody();
        body.bidiriop_version = new com.ooc.BiDirIOP.Version((byte)1, (byte)0);
        body.giop_version = new com.ooc.BiDirIOP.Version((byte)1, (byte)0);
        body.peer = peer;
        body.object_key = key;
        if(nested != null)
        {
            body.profiles = new org.omg.IOP.TaggedProfile[1];
            body.profiles[0] = nested;
        }
        else
            body.profiles = new org.omg.IOP.TaggedProfile[0];
        body.components = new org.omg.IOP.TaggedComponent[0];

        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
        com.ooc.CORBA.OutputStream out =
            new com.ooc.CORBA.OutputStream(buf);
        out._OB_writeEndian();
        com.ooc.BiDirIOP.ProfileBodyHelper.write(out, body);
        profile.profile_data = new byte[buf.length()];
        System.arraycopy(buf.data(), 0, profile.profile_data, 0,
                         buf.length());

        return profile;
    }

    public void
    destroy()
    {
    }
}
