// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package pi;

//
// Client Request Interceptor logger -- logs client activity
//
final class CRILogger_impl
    extends org.omg.CORBA.LocalObject
    implements org.omg.PortableInterceptor.ClientRequestInterceptor
{
    //
    // The RILogger object
    //
    private RILogger riLogger_;

    //
    // Set to riLogger_.out_ for ease of use
    //
    private AnyWriter out_;

    // -----------------------------------------------------------------
    // CRILogger private member implementation
    // -----------------------------------------------------------------

    //
    // Get the logging level
    //
    private short
    level(org.omg.PortableInterceptor.ClientRequestInfo ri)
    {
	short l = 0;

	try
	{
	    //
	    // Get the logger policy
	    //
	    org.omg.CORBA.Policy policy =
		ri.get_request_policy(LOGGER_POLICY_ID.value);
	    LoggerPolicy loggerPolicy = LoggerPolicyHelper.narrow(policy);
	    l = loggerPolicy.level();
	}
	catch(org.omg.CORBA.INV_POLICY ex)
	{
	    // Failed to get policy
	}
	catch(org.omg.CORBA.BAD_PARAM ex)
	{
	    // Failed to narrow policy
	}

	return l;
    }

    //
    // Display PortableInterceptor::ClientRequestInfo
    //
    private void
    displayClientRequestInfo(org.omg.PortableInterceptor.ClientRequestInfo info,
			     short logLevel)
    {
	int i;

	riLogger_.displayRequestInfo(info, logLevel);

	if(logLevel > 2)
	{
	    //
	    // Display effective profile
	    //
	    org.omg.IOP.TaggedProfile profile = info.effective_profile();

	    out_.skip();
	    out_.getWriter().print("effective profile = ");

	    if (profile.tag == org.omg.IOP.TAG_INTERNET_IOP.value)
		out_.getWriter().print("TAG_INTERNET_IOP");
	    else if (profile.tag == org.omg.IOP.TAG_MULTIPLE_COMPONENTS.value)
		out_.getWriter().print("TAG_MULTIPLE_COMPONENTS");
	    else
		out_.getWriter().print("UNKNOWN_TAG");

	    out_.getWriter().print(": ");
	    for (i = 0 ; i < profile.profile_data.length ; i++)
		out_.getWriter().print(
		    riLogger_.getOctet(profile.profile_data[i]));
	    out_.getWriter().println();
	}

	//
	// Display received exception
	//
	try
	{
	    org.omg.CORBA.Any exception = info.received_exception();

	    out_.skip();
	    out_.getWriter().println("received exception = ");

	    out_.down();
	    out_.skip();
	    out_.print(exception);
	    out_.getWriter().println();
	    out_.up();
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}

	//
	// Display received exception id
	//
	try
	{
	    String exception_id = info.received_exception_id();

	    out_.skip();
	    out_.getWriter().println("received exception id = " + exception_id);
	}
	catch(org.omg.CORBA.BAD_INV_ORDER ex)
	{
	    // Operation not supported in this context
	}

    }

    // -----------------------------------------------------------------
    // CRILogger constructor
    // -----------------------------------------------------------------

    public CRILogger_impl(org.omg.PortableInterceptor.ORBInitInfo info,
    			  int slotId)
    {
	riLogger_ = new RILogger(info, slotId);
	out_ = riLogger_.out_;
    }

    // -----------------------------------------------------------------
    // IDL to Java Mapping
    // -----------------------------------------------------------------

    //
    // The name of the interceptor.
    //
    public String
    name()
    {
	return "CRILogger";
    }

    //
    // Clean up prior to ORB destruction
    //
    public void
    destroy()
    {
    }

    //
    // This interception point allows an Interceptor to query request
    // information and modify the service context before the request
    // is sent.
    //
    public void
    send_request(org.omg.PortableInterceptor.ClientRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	short l = level(ri);
	if(l > 0)
	{
	    //
	    // Try to get the CallStack from slot
	    //
	    try
	    {
		org.omg.CORBA.Any any = ri.get_slot(riLogger_.mySlotId_);
		org.omg.CORBA.TypeCode tc = any.type();
		if(tc.kind() != org.omg.CORBA.TCKind.tk_null)
		{
		    //
		    // Encode CallStack
		    //
		    byte[] data = riLogger_.cdrCodec_.encode_value(any);

		    //
		    // Add encoded CallStack to service context
		    //
		    org.omg.IOP.ServiceContext sc =
			new org.omg.IOP.ServiceContext();
		    sc.context_id = riLogger_.REQUEST_CONTEXT_ID;
		    sc.context_data = data;
		    ri.add_request_service_context(sc, false);
		}
	    }
	    catch(org.omg.PortableInterceptor.InvalidSlot ex)
	    {
		// Ignore failure in this demo
	    }
	    catch(org.omg.IOP.CodecPackage.InvalidTypeForEncoding ex)
	    {
		// Ignore failure in this demo
	    }

	    out_.getWriter().println("send request:");
	    displayClientRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }

    //
    // This interception point allows an Interceptor to query
    // information during a TimeIndependent Infocation (TII)
    // polling get reply sequence.
    //
    public void
    send_poll(org.omg.PortableInterceptor.ClientRequestInfo ri)
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("send poll:");
	    displayClientRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }

    //
    // This interception point allows an Interceptor to query
    // the information on a reply after it is returned from the
    // server and before control is returned to the client.
    // 
    public void
    receive_reply(org.omg.PortableInterceptor.ClientRequestInfo ri)
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("receive reply:");
	    displayClientRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }

    //
    // This interception point allows an Interceptor to query
    // the exception information before the exception is raised
    // to the client.
    //
    public void
    receive_exception(org.omg.PortableInterceptor.ClientRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("receive exception:");
	    displayClientRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }

    //
    // This interception point allows an Interceptor to query
    // the information available when a request results in
    // something other than a normal reply or an exception.
    //
    public void
    receive_other(org.omg.PortableInterceptor.ClientRequestInfo ri)
	throws org.omg.PortableInterceptor.ForwardRequest
    {
	short l = level(ri);
	if(l > 0)
	{
	    out_.getWriter().println("receive other:");
	    displayClientRequestInfo(ri, l);
	    out_.getWriter().println();
	}
    }
}
