// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

final class Acceptor_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Acceptor
{
    //
    // The acceptor information object
    //
    private AcceptorInfo_impl info_;

    //
    // The packet handler
    //
    // NOTE: Must not be private, because AcceptorInfo_impl accesses
    // this member
    //
    ServerPacketHandler serverPacketHandler_;

    //
    // Set no-loopback mode
    //
    private void setNoLoopback(java.net.MulticastSocket mcs,
			       com.ooc.OB.Logger logger)
    {
	// 
	// mcs.setLoopbackMode(false);
	// result = mcs.getLoopbackMode();
	//
	// JDK 1.4.0 documentation wrongly states that 'true' 
	// switches off loopback mode
	//
	// This is only a hint to the OS -> check and log if not done
	//
	// We have to use reflection here because the setLoopbackMode() 
	// method is not available in JDK < 1.4
	//
	try
	{
	    //
	    // Disable loopback mode
	    //
	    Class c = java.net.MulticastSocket.class;
	    Class[] parameterTypes = new Class[] {Boolean.TYPE};
	    java.lang.reflect.Method m = 
		c.getMethod("setLoopbackMode", parameterTypes);
	    Boolean disable = new Boolean(false);
	    Object[] arguments = new Object[] {disable};
	    m.invoke(mcs, arguments);
	    
	    //
	    // Check if mode was set
	    //
	    parameterTypes = new Class[] {};
	    m = c.getMethod("getLoopbackMode", parameterTypes);
	    arguments = new Object[] {};
	    Boolean result = (Boolean) m.invoke(mcs, arguments);
	    if (result.booleanValue() != false)
	    {
		logger.trace("UDP", "loopback mode not disabled");
	    }
	}
	catch (NoSuchMethodException ex)
	{
	    // method not supported in this JDK version
	    logger.trace("UDP", 
			 "setLoopbackMode() not available in JDK < 1.4");
	}
	catch (IllegalAccessException ex)
	{
	    throw new InternalError(ex.toString());
	}
	catch (java.lang.reflect.InvocationTargetException ex)
	{
	    Throwable t = ex.getTargetException();
	    if(t instanceof java.net.SocketException)
	    {
		mcs.close();
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorGetsockopt) +
		    ": " + ex.toString(),
		    com.ooc.OB.MinorCodes.MinorGetsockopt,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	    }
	    else
	    {
		throw new InternalError(t.toString());
	    }
	}
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
        return com.ooc.UDPIOP.TAG_UDP_IOP.value;
    }

    public int
    handle()
    {
        throw new org.omg.CORBA.NO_IMPLEMENT();
    }

    public void
    close()
    {
        serverPacketHandler_.close();

        //
        // Wait for the helper thread to terminate
        //
        for(;;)
        {
            try
            {
                serverPacketHandler_.join();
                break;
            }
            catch(InterruptedException ex)
            {
            }
        }

        //
        // Destroy the info object
        //
        info_._OB_destroy();
    }

    public void
    shutdown()
    {
        serverPacketHandler_.shutdown();
    }

    public void
    listen()
    {
        //
        // This operation does nothing in the Java implementation
        //
    }

    public com.ooc.OCI.Transport
    accept(boolean block)
    {
        //
        // Get new transport
        //
        com.ooc.OCI.Transport tr = serverPacketHandler_.getNewTransport(block);

        if(tr != null)
        {
            //
            // Call callbacks
            //
            com.ooc.OCI.TransportInfo trInfo = tr.get_info();
            try
            {
                info_._OB_callAcceptCB(trInfo);
            }
            catch(org.omg.CORBA.SystemException ex)
            {
                tr.close();
                throw ex;
            }
        }

        //
        // Return new transport
        //
        return tr;
    }

    public com.ooc.OCI.Transport
    connect_self()
    {
        com.ooc.OCI.Transport t = new SelfTransport_impl();
        serverPacketHandler_.connectSelf(t);
        return t;
    }

    public void
    add_profiles(com.ooc.OCI.ProfileInfo profileInfo,
                 org.omg.IOP.IORHolder ior)
    {
        com.ooc.UDPIOP.ProfileBody body = new com.ooc.UDPIOP.ProfileBody();
        body.udpiop_version = new com.ooc.UDPIOP.Version();
        body.udpiop_version.major = 1;
        body.udpiop_version.minor = 0;
        body.giop_version = new com.ooc.UDPIOP.Version();
        body.giop_version.major = profileInfo.major;
        body.giop_version.minor = profileInfo.minor;

        body.host = serverPacketHandler_.getHost();
        int port = serverPacketHandler_.getPort();
        if(port >= 0x8000)
	{
            body.port = (short)(port - 0xffff - 1);
	}
        else
	{
            body.port = (short)port;
	}

        body.object_key = profileInfo.key;        
        body.components = profileInfo.components;

        int len = ior.value.profiles.length + 1;    
        org.omg.IOP.TaggedProfile[] profiles =
            new org.omg.IOP.TaggedProfile[len];
        System.arraycopy(ior.value.profiles, 0, profiles, 0,
                         ior.value.profiles.length);
        ior.value.profiles = profiles;
        ior.value.profiles[len - 1] = new org.omg.IOP.TaggedProfile();
        ior.value.profiles[len - 1].tag = tag();
        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
        com.ooc.CORBA.OutputStream out = new com.ooc.CORBA.OutputStream(buf);
        out._OB_writeEndian();
        com.ooc.UDPIOP.ProfileBodyHelper.write(out, body);
        ior.value.profiles[len - 1].profile_data = new byte[buf.length()];
        System.arraycopy(buf.data(), 0,
                         ior.value.profiles[len - 1].profile_data,
                         0, buf.length());
    }

    public com.ooc.OCI.ProfileInfo[]
    get_local_profiles(org.omg.IOP.IOR ior)
    {
        com.ooc.OCI.ProfileInfoSeqHolder profileInfoSeq =
            new com.ooc.OCI.ProfileInfoSeqHolder();
        profileInfoSeq.value = new com.ooc.OCI.ProfileInfo[0];

        Util.extractAllProfileInfos(ior, profileInfoSeq, true,
                                    serverPacketHandler_.getHost(),
                                    serverPacketHandler_.getPort(),
                                    true);

        return profileInfoSeq.value;
    }

    public com.ooc.OCI.AcceptorInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    Acceptor_impl(String host, int port, 
		  boolean multicast, boolean noLoopback, 
		  int ttl, int trace, int bufferSize, 
		  int messageTimeout, int transportTimeout,
                  com.ooc.OB.Logger logger)
    {
        info_ = new AcceptorInfo_impl(this);

        //
        // Create socket and bind to all network interfaces
        //
        java.net.DatagramSocket socket = null;
        try
        {
            if(multicast)
            {
                java.net.InetAddress group = null;
                try
                {
                    group = java.net.InetAddress.getByName(host);
                }
                catch(java.net.UnknownHostException ex)
                {
                    throw new org.omg.CORBA.COMM_FAILURE(
                        com.ooc.OB.MinorCodes.describeCommFailure(
                            com.ooc.OB.MinorCodes.MinorGethostbyname) +
                            ": " + ex.toString(),
                        com.ooc.OB.MinorCodes.MinorGethostbyname,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                }

                java.net.MulticastSocket mcs =
                    new java.net.MulticastSocket(port);

		//
		// Set TTL (time-to-live) if specified
		//
		if (ttl != -1)
		{
		    mcs.setTimeToLive(ttl);
		}

		//
		// Set no-loopback mode
		//
		if (noLoopback)
		{
		    setNoLoopback(mcs, logger);
		}

		//
		// Join multicast group
		//
                mcs.joinGroup(group);

                socket = mcs;
            }
	    else
	    {
                socket = new java.net.DatagramSocket(port);
	    }

            //
            // Read back the port. This is needed if the port was selected by
            // the operating system.
            //
            port = socket.getLocalPort();

	    //
	    // Set receive buffer size
	    //
	    if(bufferSize > 0)
	    {
		socket.setReceiveBufferSize(bufferSize);
	    }

	    if(trace > 0)
	    {
		int size = socket.getReceiveBufferSize();
		logger.trace("UDP", "receive buffer size is " + size);
	    }
	}
        catch(java.net.BindException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorBind) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorBind,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        java.net.InetAddress addr = socket.getLocalAddress();

        //
        // Create the ServerPacketHandler and start the
        // ServerPacketHandler thread
        //
        serverPacketHandler_ = new ServerPacketHandler(info_, host, port, 
						       addr, socket, trace,
                                                       messageTimeout,
                                                       transportTimeout,
                                                       logger);
        serverPacketHandler_.start();
    }

    public
    Acceptor_impl(String address, String host, int port, 
		  boolean multicast, boolean noLoopback, 
		  int ttl, int trace, int bufferSize, 
		  int messageTimeout, int transportTimeout, 
		  com.ooc.OB.Logger logger)
    {
        info_ = new AcceptorInfo_impl(this);

        //
        // Get the bind address
        //
        java.net.InetAddress bindAddress = null;
        try
        {
            bindAddress = java.net.InetAddress.getByName(address);
        }
        catch(java.net.UnknownHostException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorGethostbyname) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorGethostbyname,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // Create socket and bind to requested network interface
        //
        java.net.DatagramSocket socket = null;
        try
        {
            if(multicast)
            {
                java.net.InetAddress group = null;
                try
                {
                    group = java.net.InetAddress.getByName(host);
                }
                catch(java.net.UnknownHostException ex)
                {
                    throw new org.omg.CORBA.COMM_FAILURE(
                        com.ooc.OB.MinorCodes.describeCommFailure(
                            com.ooc.OB.MinorCodes.MinorGethostbyname) +
                            ": " + ex.toString(),
                        com.ooc.OB.MinorCodes.MinorGethostbyname,
                        org.omg.CORBA.CompletionStatus.COMPLETED_NO);
                }

                java.net.MulticastSocket mcs =
                    new java.net.MulticastSocket(port);

		//
		// Set bind address
		//
                mcs.setInterface(bindAddress);

		//
		// Set TTL (time-to-live) if specified
		//
		if (ttl != -1)
		{
		    mcs.setTimeToLive(ttl);
		}

		//
		// Set no-loopback mode 
		//
		if (noLoopback)
		{
		    setNoLoopback(mcs, logger);
		}

		//
		// Join multicast group
		//
                mcs.joinGroup(group);

                socket = mcs;
            }
            else
	    {
                socket = new java.net.DatagramSocket(port, bindAddress);
	    }

            //
            // Read back the port. This is needed if the port was selected by
            // the operating system.
            //
            port = socket.getLocalPort();

	    //
	    // Set receive buffer size
	    //
	    if(bufferSize > 0)
	    {
		socket.setReceiveBufferSize(bufferSize);
	    }

	    if(trace > 0)
	    {
		int size = socket.getReceiveBufferSize();
		logger.trace("UDP", "receive buffer size is " + size);
	    }
        }
        catch(java.net.BindException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorBind) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorBind,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        java.net.InetAddress addr = socket.getLocalAddress();

        //
        // Create the ServerPacketHandler and start the
        // ServerPacketHandler thread
        //
        serverPacketHandler_ = new ServerPacketHandler(info_, host, port, 
						       addr, socket, trace,
                                                       messageTimeout,
                                                       transportTimeout,
                                                       logger);
        serverPacketHandler_.start();
    }
}
