// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

final public class ConFactory_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.ConFactory
{
    //
    // The trace level
    //
    private int trace_;

    //
    // The buffer size
    //
    private int bufferSize_;

    //
    // The packet size
    //
    private int packetSize_;

    //
    // The packet delay
    //
    private int packetDelay_;

    //
    // The multicast no loopback policy
    //
    private boolean noLoopback_;

    //
    // The TTL (time to live) value for multicast
    //
    private int ttl_;

    //
    // The logger
    //
    private com.ooc.OB.Logger logger_;

    //
    // The connector factory information object
    //
    private ConFactoryInfo_impl info_;

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
        return com.ooc.UDPIOP.TAG_UDP_IOP.value;
    }

    public String
    describe_profile(org.omg.IOP.TaggedProfile profile)
    {
        com.ooc.OB.Assert._OB_assert(profile.tag ==
                                 com.ooc.UDPIOP.TAG_UDP_IOP.value);

        //
        // Get the profile body
        //
        byte[] data = profile.profile_data;
        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer(data, data.length);
        com.ooc.CORBA.InputStream in = new com.ooc.CORBA.InputStream(buf);
        in._OB_readEndian();
        com.ooc.UDPIOP.ProfileBody body =
            com.ooc.UDPIOP.ProfileBodyHelper.read(in);

        StringBuffer result = new StringBuffer();

        //
        // Show general info
        //
        result.append("udpiop_version: " + (int)body.udpiop_version.major + '.'
            + (int)body.udpiop_version.minor + '\n');
        result.append("giop_version: " + (int)body.giop_version.major + '.'
            + (int)body.giop_version.minor + '\n');
        result.append("host: " + body.host + '\n');
        int port;
        if(body.port < 0)
            port = 0xffff + (int)body.port + 1;
        else
            port = body.port;
        result.append("port: " + port + '\n');
        result.append("object_key: (" + body.object_key.length + ")\n");
        String tmp = com.ooc.OB.IORUtil.dump_octets(body.object_key, 0,
                                                    body.object_key.length);
        result.append(tmp);

        //
        // Components
        //
        for(int i = 0 ; i < body.components.length ; i++)
        {
            String desc =
                com.ooc.OB.IORUtil.describe_component(body.components[i]);
            result.append(desc);
        }

        return result.toString();
    }

    public com.ooc.OCI.Connector[]
    create_connectors(org.omg.IOP.IOR ior, org.omg.CORBA.Policy[] policies)
    {
        //
        // Check whether policies are satisfied
        //
        for(int i = 0; i < policies.length; i++)
        {
            if(policies[i].policy_type() == 
               com.ooc.OB.PROTOCOL_POLICY_ID.value)
            {
                com.ooc.OB.ProtocolPolicy p =
                    com.ooc.OB.ProtocolPolicyHelper.narrow(policies[i]);
                if(!p.contains(id()))
                    return new com.ooc.OCI.Connector[0];
            }
        }

        //
        // Create Connectors from profiles
        //
        java.util.Vector seq = new java.util.Vector();
        for(int i = 0 ; i < ior.profiles.length ; i++)
        {
            if(ior.profiles[i].tag == tag())
            {
                //
                // Get the UDPIOP profile body
                //
                byte[] data = ior.profiles[i].profile_data;
                com.ooc.OCI.Buffer buf =
                    new com.ooc.OCI.Buffer(data, data.length);
                com.ooc.CORBA.InputStream in =
                    new com.ooc.CORBA.InputStream(buf, 0, false);
                in._OB_readEndian();
                com.ooc.UDPIOP.ProfileBody body =
                    com.ooc.UDPIOP.ProfileBodyHelper.read(in);

                //
                // Create new connector for this profile
                //
                int port;
                if(body.port < 0)
		{
                    port = 0xffff + (int)body.port + 1;
		}
                else
		{
                    port = (int)body.port;
		}
                com.ooc.OCI.ConnectCB[] cbs = info_._OB_getConnectCBSeq();
                com.ooc.OCI.Connector con =
                    new Connector_impl(body.host, port, 
				       trace_, bufferSize_,
                                       packetSize_, packetDelay_, 
				       noLoopback_, ttl_, 
				       cbs, logger_);
                seq.addElement(con);
            }
        }

        com.ooc.OCI.Connector[] result = new com.ooc.OCI.Connector[seq.size()];
        seq.copyInto(result);
        return result;
    }

    public boolean
    equivalent(org.omg.IOP.IOR ior1, org.omg.IOP.IOR ior2)
    {
        return Util.equivalent(ior1, ior2);
    }

    public int
    hash(org.omg.IOP.IOR ior, int max)
    {
        return Util.hash(ior, max);
    }

    public com.ooc.OCI.ConFactoryInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    public
    ConFactory_impl(int trace, int bufferSize, int packetSize, int packetDelay,
		    boolean noLoopback, int ttl, com.ooc.OB.Logger logger)
    {
        trace_ = trace;
        bufferSize_ = bufferSize;
        packetSize_ = packetSize;
        packetDelay_ = packetDelay;
	noLoopback_ = noLoopback;
	ttl_ = ttl;
        logger_ = logger;

        info_ = new ConFactoryInfo_impl();
    }
}
