// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

final class Connector_impl extends org.omg.CORBA.LocalObject
    implements com.ooc.OCI.Connector
{
    //
    // The hostname
    //
    // NOTE: Must not be private, because ConnectorInfo_impl accesses
    // this member
    //
    String host_;

    //
    // The port
    //
    // NOTE: Must not be private, because ConnectorInfo_impl accesses
    // this member
    //
    int port_;

    //
    // The connector information object
    //
    private ConnectorInfo_impl info_;

    //
    // The transport
    //
    private ClientTransport_impl transport_;

    // ------------------------------------------------------------------
    // Private and protected functions
    // ------------------------------------------------------------------

    private void
    close()
    {
        //
        // Destroy the info object
        //
        info_._OB_destroy();
    }

    //
    // Set no-loopback mode
    //
    private void setNoLoopback(java.net.MulticastSocket mcs,
			       com.ooc.OB.Logger logger)
    {
	// 
	// mcs.setLoopbackMode(false);
	// result = mcs.getLoopbackMode();
	//
	// JDK 1.4.0 documentation wrongly states that 'true' 
	// switches off loopback mode
	//
	// This is only a hint to the OS -> check and log if not done
	//
	// We have to use reflection here because the setLoopbackMode() 
	// method is not available in JDK < 1.4
	//
	try
	{
	    //
	    // Disable loopback mode
	    //
	    Class c = java.net.MulticastSocket.class;
	    Class[] parameterTypes = new Class[] {Boolean.TYPE};
	    java.lang.reflect.Method m = 
		c.getMethod("setLoopbackMode", parameterTypes);
	    Boolean disable = new Boolean(false);
	    Object[] arguments = new Object[] {disable};
	    m.invoke(mcs, arguments);
	    
	    //
	    // Check if mode was set
	    //
	    parameterTypes = new Class[] {};
	    m = c.getMethod("getLoopbackMode", parameterTypes);
	    arguments = new Object[] {};
	    Boolean result = (Boolean) m.invoke(mcs, arguments);
	    if (result.booleanValue() != false)
	    {
		logger.trace("UDP", "loopback mode not disabled");
	    }
	}
	catch (NoSuchMethodException ex)
	{
	    // method not supported in this JDK version
	    logger.trace("UDP", 
			 "setLoopbackMode() not available in JDK < 1.4");
	}
	catch (IllegalAccessException ex)
	{
	    throw new InternalError(ex.toString());
	}
	catch (java.lang.reflect.InvocationTargetException ex)
	{
	    Throwable t = ex.getTargetException();
	    if(t instanceof java.net.SocketException)
	    {
		mcs.close();
		throw new org.omg.CORBA.COMM_FAILURE(
		    com.ooc.OB.MinorCodes.describeCommFailure(
			com.ooc.OB.MinorCodes.MinorGetsockopt) +
		    ": " + ex.toString(),
		    com.ooc.OB.MinorCodes.MinorGetsockopt,
		    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	    }
	    else
	    {
		throw new InternalError(t.toString());
	    }
	}
    }

    // ------------------------------------------------------------------
    // Standard IDL to Java Mapping
    // ------------------------------------------------------------------

    public String
    id()
    {
        return PLUGIN_ID.value;
    }

    public int
    tag()
    {
        return com.ooc.UDPIOP.TAG_UDP_IOP.value;
    }

    public com.ooc.OCI.SendReceiveMode
    mode()
    {
        return com.ooc.OCI.SendReceiveMode.ReceiveOnly;
    }

    public com.ooc.OCI.Transport
    connect()
    {
        return transport_;
    }

    public com.ooc.OCI.Transport
    connect_timeout(int t)
    {
        return connect();
    }

    public com.ooc.OCI.ProfileInfo[]
    get_usable_profiles(org.omg.IOP.IOR ior, org.omg.CORBA.Policy[] policies)
    {
        //
        // Make sure that the set of policies is met
        //
        for(int i = 0 ; i < policies.length ; i++)
        {
            if(policies[i].policy_type() ==
               com.ooc.OB.PROTOCOL_POLICY_ID.value)
            {
                com.ooc.OB.ProtocolPolicy p =
                    com.ooc.OB.ProtocolPolicyHelper.narrow(policies[i]);
                if(!p.contains(id()))
		{
                    return new com.ooc.OCI.ProfileInfo[0];
		}
            }
        }

        com.ooc.OCI.ProfileInfoSeqHolder profileInfoSeq =
            new com.ooc.OCI.ProfileInfoSeqHolder();
        profileInfoSeq.value = new com.ooc.OCI.ProfileInfo[0];
        Util.extractAllProfileInfos(ior, profileInfoSeq, true, host_, port_,
                                    false);

        return profileInfoSeq.value;
    }

    public boolean
    equal(com.ooc.OCI.Connector con)
    {
        Connector_impl impl = null;
        try
        {
            impl = (Connector_impl)con;
        }
        catch(ClassCastException ex)
        {
            return false;
        }

        //
        // Compare ports
        //
        if(port_ != impl.port_)
	{
            return false;
	}

        //
        // Direct host name comparison
        //
        if(!host_.equals(impl.host_))
        {
            //
            // Direct host name comparision failed - must look up
            // addresses to be really sure if the hosts differ
            //
            try
            {
                java.net.InetAddress addr1 =
                    java.net.InetAddress.getByName(host_);

                java.net.InetAddress addr2 =
                    java.net.InetAddress.getByName(impl.host_);

                if(!addr1.equals(addr2))
		{
                    return false;
		}
            }
            catch(java.net.UnknownHostException ex)
            {
                //
                // Return false on hostname lookup failure
                //
                return false;
            }
        }

        //
        // OK, connectors are the same
        //
        return true;
    }


    public com.ooc.OCI.ConnectorInfo
    get_info()
    {
        return info_;
    }

    // ------------------------------------------------------------------
    // ORBacus internal functions
    // Application programs must not use these functions directly
    // ------------------------------------------------------------------

    Connector_impl(String host, int port, 
		   int trace, int bufferSize,
                   int packetSize, int packetDelay,
		   boolean noLoopback, int ttl,
                   com.ooc.OCI.ConnectCB[] cbs, 
		   com.ooc.OB.Logger logger)
    {
        host_ = host;
        port_ = port;
        info_ = new ConnectorInfo_impl(this, cbs);

        //
        // Get the address
        //
        java.net.InetAddress address = null;
        try
        {
            address = java.net.InetAddress.getByName(host_);
        }
        catch(java.net.UnknownHostException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorGethostbyname) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorGethostbyname,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // A signed applet running in Netscape may be granted
        // permission to make unrestricted connections. We use
        // reflection to invoke the Netscape PrivilegeManager
        // class in an attempt to enable the necessary privileges.
        // If the applet hasn't been granted these privileges, an
        // exception will be raised. The privileges are enabled
        // only for the duration of this method.
        //
        if(System.getProperty("java.vendor").indexOf("Netscape") != -1)
        {
            try
            {
                Class nspm =
                    Class.forName("netscape.security.PrivilegeManager");
                Class[] enablePrivilegeArgTypes = { String.class };
                java.lang.reflect.Method enablePrivilege =
                    nspm.getMethod("enablePrivilege", enablePrivilegeArgTypes);
                java.lang.Object[] enablePrivilegeArgs = { null };
                String[] privileges = { "IIOPRuntime", "UniversalConnect" };
                for(int i = 0 ; i < privileges.length ; i++)
                {
                    enablePrivilegeArgs[0] = privileges[i];
                    enablePrivilege.invoke(null, enablePrivilegeArgs);
                }
            }
            catch(Exception ex)
            {
                // Ignore
            }
        }

        //
        // TODO: Also need rights for Listen?
        //

        //
        // Create the socket
        //
        java.net.DatagramSocket socket = null;
        try
        {
            if(address.isMulticastAddress())
	    {
		java.net.MulticastSocket mcs = new java.net.MulticastSocket();
		
		//
		// Set TTL (time-to-live) if specified
		//
		if (ttl != -1)
		{
		    mcs.setTimeToLive(ttl);
		}

		//
		// Set no-loopback mode
		//
		if (noLoopback)
		{
		    setNoLoopback(mcs, logger);
		}

                socket = mcs;
	    }
            else
	    {
                socket = new java.net.DatagramSocket();
	    }

	    if(bufferSize > 0)
	    {
		socket.setSendBufferSize(bufferSize);
	    }

	    if(trace > 0)
	    {
		int size = socket.getSendBufferSize();
		logger.trace("UDP", "send buffer size is " + size);
	    }	    

	    //
	    // Connect the socket. This does not establish any connection
	    // to the server, it just establishes the default destination
	    // for packets.
	    //
	    socket.connect(address, port_);
        }
        catch(java.io.IOException ex)
        {
            throw new org.omg.CORBA.COMM_FAILURE(
                com.ooc.OB.MinorCodes.describeCommFailure(
                    com.ooc.OB.MinorCodes.MinorSocket) +
                    ": " + ex.toString(),
                com.ooc.OB.MinorCodes.MinorSocket,
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }
	catch(RuntimeException ex)
	{
	    socket.close();
	    throw new org.omg.CORBA.COMM_FAILURE(
		com.ooc.OB.MinorCodes.describeCommFailure(
		    com.ooc.OB.MinorCodes.MinorConnect) +
		": " + ex.toString(),
		com.ooc.OB.MinorCodes.MinorConnect,
		org.omg.CORBA.CompletionStatus.COMPLETED_NO);
	}

        //
        // Create the ClientTransport
        //
        transport_ = new ClientTransport_impl(this, socket, trace, 
					      packetSize, packetDelay, 
					      logger);
    }
}
