// **********************************************************************
//
// Copyright (c) 2002
// IONA Technologies, Inc.
// Waltham, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

package com.ooc.OCI.UDP;

public class CorbalocProtocol_impl extends org.omg.CORBA.LocalObject
                                   implements com.ooc.OB.CorbalocProtocol
{
    public String
    name()
    {
        return "udp";
    }

    public org.omg.IOP.TaggedProfile
    parse_address(String addr, byte[] key)
    {
        byte major = (byte)1, minor = (byte)0;

        //
        // Empty hostname is illegal (as is port ':YYYY' by itself)
        //
        if(addr.length() == 0 || addr.charAt(0) == ':')
            throw new org.omg.CORBA.BAD_PARAM(
                com.ooc.OB.MinorCodes.describeBadParam(
                    com.ooc.OB.MinorCodes.MinorBadAddress) +
                    ": udp hostname must be specified",
                com.ooc.OB.MinorCodes.MinorBadAddress, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);

        //
        // Hostname is terminated by ':port'
        //
        String host;
        int colon = addr.indexOf(':', 0);
        if(colon == -1)
            throw new org.omg.CORBA.BAD_PARAM(
                com.ooc.OB.MinorCodes.describeBadParam(
                    com.ooc.OB.MinorCodes.MinorBadAddress) +
                    ": udp port must be specified",
                com.ooc.OB.MinorCodes.MinorBadAddress, 
                org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        else
            host = addr.substring(0, colon);

        //
        // Valid range for port is 1 - 65535
        //
        int port = 0;
        {
            String str = addr.substring(colon + 1);
            try
            {
                port = Integer.valueOf(str).intValue();
            }
            catch(NumberFormatException ex)
            {
                throw new org.omg.CORBA.BAD_PARAM(
                    com.ooc.OB.MinorCodes.describeBadParam(
                        com.ooc.OB.MinorCodes.MinorBadAddress) +
                        ": udp port is invalid",
                    com.ooc.OB.MinorCodes.MinorBadAddress, 
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
            }
            if(port < 1 || port > 65535)
                throw new org.omg.CORBA.BAD_PARAM(
                    com.ooc.OB.MinorCodes.describeBadParam(
                        com.ooc.OB.MinorCodes.MinorBadAddress) +
                        ": udp port must be between 1 and 65535",
                    com.ooc.OB.MinorCodes.MinorBadAddress, 
                    org.omg.CORBA.CompletionStatus.COMPLETED_NO);
        }

        //
        // Create profile
        //
        org.omg.IOP.TaggedProfile profile = new org.omg.IOP.TaggedProfile();
        profile.tag = com.ooc.UDPIOP.TAG_UDP_IOP.value;

        com.ooc.UDPIOP.ProfileBody body = new com.ooc.UDPIOP.ProfileBody();
        body.udpiop_version = new com.ooc.UDPIOP.Version(major, minor);
        body.giop_version = new com.ooc.UDPIOP.Version((byte)1, (byte)0);
        body.host = host;
        if(port >= 0x8000)
            body.port = (short)(port - 0xffff - 1);
        else
            body.port = (short)port;
        body.object_key = key;
        body.components = new org.omg.IOP.TaggedComponent[0];

        com.ooc.OCI.Buffer buf = new com.ooc.OCI.Buffer();
        com.ooc.CORBA.OutputStream out = new com.ooc.CORBA.OutputStream(buf);
        out._OB_writeEndian();
        com.ooc.UDPIOP.ProfileBodyHelper.write(out, body);
        profile.profile_data = new byte[buf.length()];
        System.arraycopy(buf.data(), 0, profile.profile_data, 0,
                         buf.length());

        return profile;
    }

    public void
    destroy()
    {
    }
}
